/*
	Copyright Carl Troein 2000-2004

	Syntax: noded_assume_position [input_file output_file]
	Default values: "n_XX.conn2", "n_XX.ures"
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <cstddef>

#include "constants.h"
#include "contact_translation.h"

#define die(text) { printf(text); exit(1); }


// To be able to do the contact removal/translation for both P and H in
// each step, the array is stored right after it's created from the tree.
// Its size determines where to store it. If larger than the size defined
// here, the array will end up on disk rather than in RAM.
//#define MAXARRAYSIZE_NOFILE 524288
#define MAXARRAYSIZE_NOFILE 2097152

// When recursing, less memory will be needed for d_arr the deeper we go.
// To reduce the memory useage (though not the peak usage) we free d_arr
// and allocate a smaller array as we go deeper. For efficiency reason this
// should not be done if the array is already small, and this defines 'small'.
#define MAXARRAYSIZE_NOREALLOC 4194304
#define MAXARRAYNODES_NOREALLOC ((int)(MAXARRAYSIZE_NOREALLOC / sizeof(struct node_d)))

#define OUTBUFSIZE 65536
unsigned int outbuf[OUTBUFSIZE];
int outbuflen = 0;
FILE *outfile;

// For the subset thing
struct subset
{
	struct subset *next;
	unsigned char wilds, len;
	unsigned char conns[1];	// really [len]
};
struct forkend
{
	unsigned int number;			// The highest bit is a flag, degen.
	struct subset *subsets;		// The subsets, if any.
};
struct connfork
{
	union h
	{
		struct forkend *fe;		// if isend
		struct connfork *cf;		// if not isend
	} has;
	struct connfork *hasnot;
	unsigned char conn;
	unsigned char isend;
};
struct connfork *firstfork[MAXNRCONNS + 1];

struct node_d *d_arr;

// For mymalloc
void *mempool = NULL;
int bytesleft = 0;

char tempfilename[CHAINLENGTH][32];


// Convert H/Pmask to string
char *tostring(unsigned int Pmask, unsigned int Hmask)
{
	static char buf[CHAINLENGTH+1];
	buf[CHAINLENGTH] = 0;
	for(int i=0; i < CHAINLENGTH; i++)
	{
		if(Pmask & (1<<i) && Hmask & (1<<i)) buf[i] = 'e';
		else if(Pmask & (1<<i)) buf[i] = 'P';
		else if(Hmask & (1<<i)) buf[i] = 'H';
		else buf[i] = 'x';
	}
	return buf;
}

int loadinput(char *fname)
{
	FILE *infile;
	int flen;

	infile = fopen(fname, "rb");
	if(!infile)
	{
		perror("Can't open input file");
		exit(1);
	}
	if(!fseek(infile, 0, SEEK_END))
	{
		flen = ftell(infile);
		if(flen > 0)
		{
			if(!fseek(infile, 0, SEEK_SET))
			{
				d_arr = (struct node_d *)malloc(flen);
				if(!d_arr) die("Out of memory 1!\n");

				if(fread(d_arr, flen, 1, infile) != 1)
				{
					perror("Read error on input file");
					exit(1);
				}
				fclose(infile);
				int nodes = flen / sizeof(struct node_d);
				printf("Loaded %u bytes (%d entries) from input file.\n", flen, nodes);
				return nodes;
			}
		}
	}
	fclose(infile);
	perror("Seek error on input file");
	exit(1);
	return 0;
}


// My own memory allocation without a free()
// Call with len -1 to empty a pool and -2 to initialize all
void *mymalloc(int len)
{
//	static const int PUDDLESIZE = 262144;
	static const int PUDDLESIZE = 65536;

	if(len < 0)
	{
		void *curpuddle = mempool;
		while(curpuddle)
		{
			void *nextpuddle = *(void **)curpuddle;
			free(curpuddle);
			curpuddle = nextpuddle;
		}
		mempool = NULL;
		bytesleft = 0;
		return NULL;
	}

	if(len > bytesleft)
	{
		void *pointer = malloc(PUDDLESIZE);
		if(!pointer)
		{
			printf("Out of bloody memory! Buy some more!\n");
			exit(1);
		}
		*(void **)pointer = mempool;
		mempool = pointer;
		bytesleft = PUDDLESIZE - sizeof(void *);
	}
	void *poin = ((char *)mempool) + PUDDLESIZE - bytesleft;
	bytesleft -= len;
	return poin;
}

inline int issubset(int wdiff, int len, unsigned char *conns, struct subset *ss)
{
	int p1=0, p2=0;
	while(p2 < ss->len)
	{
		if(p1 >= len) return 1;
		unsigned char c1 = conns[p1], c2 = ss->conns[p2];
		if(c1 < c2)
		{
			if(--wdiff < 0) return 0;
			p1++;
		}
		else if(c1 == c2)
		{
			p1++;
			p2++;
		}
		else
		{
			p2++;
		}
	}
	if(wdiff < len - p1) return 0;
	return 1;
}

// returns true if it was added
int try_add_subset(struct node_d *node)
{
	int len = node->connlen;
	int wilds = node->wilds;
	if(!len && !wilds) return 0;
	unsigned char *conns = node->conns;

	struct connfork *superfork = NULL;
	struct connfork *curfork;
	int cpos;		// Position in conns.

//printf("  meep meep, len=%d, wilds=%d numb=%x  ",len,wilds,node->number);
//for(int i=0;i<len;i++)printf(" %d",conns[i]);
//printf("\n");

	for(int w = MAXNRCONNS - wilds; w >= 0; w--)
	{
		if(!firstfork[wilds + w]) continue;
		if(len <= w)	// Instant match
		{
//printf("instant, w=%d\n",w);
			if(len < w) return 0;	// Never as good as the best
			if(superfork) return 0;	// There are several supersets!
			struct connfork *cfork = firstfork[wilds + w];
			while(!cfork->isend)		// Not an end node
			{
				cfork = cfork->has.cf;
				if(cfork->hasnot) return 0;	// There are several supersets!
			}
			if(cfork->has.fe->number & 0x80000000) return 0; // Degenerate.
			superfork = cfork;		// This might be something good.
			continue;	// We've come to an end here.
		}

		struct connfork *cfstack[MAXCONNR];
		int cposstack[MAXCONNR];
		int wildstack[MAXCONNR];
		int depth = 1;
		cfstack[0] = firstfork[wilds + w];
		cposstack[0] = 0;
		wildstack[0] = w;
		int cwild;		// Number of wilds not used up yet

		while(depth)
		{
			curfork = cfstack[--depth];
			cpos = cposstack[depth];
			cwild = wildstack[depth];
			while(1)		// While not in a dead end.
			{
				if(curfork->conn < conns[cpos])
				{
//printf("%d < %d\n",curfork->conn, conns[cpos]);
					// Our set does not have this connection.
					// Test both ways, if needed. 'has' exists always if isend is not set.
					if(curfork->hasnot)	// Stack it, possibly.
					{
						if(!curfork->isend)	// Two different ways.
						{
							cfstack[depth] = curfork->hasnot;
							wildstack[depth] = cwild;
							cposstack[depth++] = cpos;
							curfork = curfork->has.cf;
						}
						else
						{
							curfork = curfork->hasnot;
							// curfork->conn is not in the sets we're considering. Don't change the mask.
						}
					}
					else if(!curfork->isend)	// Only do non-endnodes.
					{
						curfork = curfork->has.cf;
					}
					else
					{
						break;
					}
				}
				else if(curfork->conn == conns[cpos])
				{
//printf("%d == %d\n",curfork->conn, conns[cpos]);
					cpos++;
					if(len - cpos <= cwild)	// Enough connections have been matched!
					{
						if(superfork) return 0;	// There are several supersets!
						while(!curfork->isend)	// Not an end node
						{
							curfork = curfork->has.cf;
							if(curfork->hasnot) return 0;	// There are several supersets!
						}
						if(curfork->has.fe->number & 0x80000000) return 0; // Degenerate.
						superfork = curfork;		// This might be something good.
						break;	// We've come to an end here.
					}
					else
					{
						if(!curfork->isend)
						{
							// Not all our conns matched yet, but there is more on this branch
							// Can we do without this conn?
							if(cwild && curfork->hasnot)
							{
								cfstack[depth] = curfork->hasnot;
								wildstack[depth] = cwild - 1;
								cposstack[depth++] = cpos;
							}
							curfork = curfork->has.cf;		// Walk deeper.
						}
						else if(cwild && curfork->hasnot)
						{
							// There might still be a match on this branch, but we have to
							// sacrifice one wildcard
							cwild--;
							curfork = curfork->hasnot;
						}
						else break; 	// Ignore end nodes that don't have enough conns.
					}
				}
				else	// Sacrifice wildcards and try to get back on track.
				{
//printf("%d > %d\n",curfork->conn, conns[cpos]);
					if(cwild)
					{
						cwild--;
						if(++cpos == len)
						{
							printf("bug #60113\n");
							exit(1);
						}
					}
					else break;		// Dead end.
				}
			}
		}
	}

	if(!superfork)	// No supersets found. Let's add this stuff, then.
	{
		if(!firstfork[wilds])
		{
//printf("adding node as firstfork\n");
			struct connfork *fork = (struct connfork *)mymalloc(sizeof(struct connfork));
			fork->conn = (len > 0) ? conns[0] : 0; // Allow empty sets
			fork->hasnot = NULL;
			firstfork[wilds] = fork;
			if(len > 1)
			{
				struct connfork *lfork = fork;
				for(int i = 1; i < len; i++)
				{
					fork->isend = 0;
					fork = (struct connfork *)mymalloc(sizeof(struct connfork));
					fork->conn = conns[i];
					lfork->has.cf = fork;
					fork->hasnot = NULL;
					lfork = fork;
				}
			}
			fork->isend = 1;
			fork->has.fe = (struct forkend *)mymalloc(sizeof(struct forkend));
			fork->has.fe->subsets = NULL;
			fork->has.fe->number = node->number;
			return 1;
		}
//printf("adding node\n");
		cpos = 0;
		curfork = firstfork[wilds];
		struct connfork *newfork, *oldfork = NULL;
		while(cpos < len)
		{
			if(curfork->conn < conns[cpos])	// We don't have this one.
			{
				if(curfork->hasnot)
				{
					oldfork = curfork;
					curfork = curfork->hasnot;
				}
				else	// Add a 'has not' thingy and a bunch of stuff.
				{
					newfork = (struct connfork *)mymalloc(sizeof(struct connfork));
					newfork->hasnot = NULL;
					newfork->conn = conns[cpos++];
					newfork->isend = 0;
					curfork->hasnot = newfork;
					curfork = newfork;
					while(cpos < len)
					{
						newfork = (struct connfork *)mymalloc(sizeof(struct connfork));
						newfork->hasnot = NULL;
						newfork->conn = conns[cpos++];
						newfork->isend = 0;
						curfork->has.cf = newfork;
						curfork = newfork;
					}
					curfork->isend = 1;
					curfork->has.fe = (struct forkend *)mymalloc(sizeof(struct forkend));
					curfork->has.fe->subsets = NULL;
					curfork->has.fe->number = node->number;
					return 1;
				}
			}
			else if(curfork->conn == conns[cpos])	// We have this one.
			{
				oldfork = curfork;
				curfork = curfork->has.cf;
				cpos++;
			}
			else	// We have to insert our tree branch between oldfork and curfork.
			{
				newfork = (struct connfork *)mymalloc(sizeof(struct connfork));
				newfork->conn = conns[cpos++];
				newfork->isend = 0;
				newfork->hasnot = curfork;
				if(oldfork)
				{
					// What way were we going?
					if(oldfork->has.cf == curfork)
					{
						oldfork->has.cf = newfork;
					}
					else
					{
						oldfork->hasnot = newfork;
					}
				}
				else
				{
					firstfork[wilds] = newfork;
				}
				curfork = newfork;
				while(cpos < len)
				{
					newfork = (struct connfork *)mymalloc(sizeof(struct connfork));
					newfork->conn = conns[cpos++];
					newfork->isend = 0;
					curfork->has.cf = newfork;
					newfork->hasnot = NULL;
					curfork = newfork;
				}
				curfork->isend = 1;
				curfork->has.fe = (struct forkend *)mymalloc(sizeof(struct forkend));
				curfork->has.fe->subsets = NULL;
				curfork->has.fe->number = node->number;
				return 1;
			}
		}
	}
	else		// Really simple case. I need it after the 150 lines above...
	{
		// This is a subset of another set. Redundant?
		struct forkend *fe = superfork->has.fe;
		if(!fe->subsets)
		{
//printf("adding first subset to some node\n");
//			fe->subsets = (struct subset *)mymalloc(4 + 2 + len);
			fe->subsets = (struct subset *)mymalloc(offsetof(struct subset, conns) + len);
			fe->subsets->next = NULL;
			fe->subsets->wilds = wilds;
			fe->subsets->len = len;
			memcpy(fe->subsets->conns, conns, len);
			return 1;
		}
		else
		{
//printf("considering adding subset\n");
			for(struct subset *ss = fe->subsets; ss; ss = ss->next)
			{
				// The difference in number of wilds is never <0 (because of sorting).
				int diff = ss->wilds - wilds;
				if(diff > len) return 0;
				if(issubset(diff, len, conns, ss)) return 0;
			}
		}
//printf("  ...adding subset\n");
//		struct subset *ss = (struct subset *)mymalloc(4 + 2 + len);
		struct subset *ss = (struct subset *)mymalloc(offsetof(struct subset, conns) + len);
		ss->next = fe->subsets;
		fe->subsets = ss;
		ss->wilds = wilds;
		ss->len = len;
		memcpy(ss->conns, conns, len);
		return 1;
	}
	return 0;
}

int do_the_subset_thing(int nodes)
{
	struct node_d *node, *oldnode;
	int added = 0;

	if(!nodes) return 0;
	node = d_arr;
	oldnode = node;

	for(int i = 1; i < nodes; i++)
	{
		node++;

//printf("node  len=%d, wilds=%d deg=%d  ",node->connlen,node->wilds,node->number);
//for(int f=0;f<MAXNRCONNS;f++)printf(" %d",node->conns[f]);
//printf("\n");

		// Do they differ?
		if(node->wilds != oldnode->wilds || node->connlen != oldnode->connlen ||
			memcmp(node->conns, oldnode->conns, node->connlen))
		{
			if(try_add_subset(oldnode)) added ++;
			oldnode = node;
		}
		else oldnode->number |= 0x80000000;
	}
	if(try_add_subset(oldnode)) added ++;
	return added;
}

struct forkend *nextforkend(int cont, unsigned char *conns, int *length, int *wildcnt)
{
	static struct connfork *cfstack[MAXCONNR];
	static int lenstack[MAXCONNR];
	static int depth, wilds;
	struct connfork *curfork;
	int len;

	if(!cont)
	{
		for(wilds = MAXNRCONNS; !firstfork[wilds]; wilds--)
		{
			if(!wilds) return NULL;
		}
		lenstack[0] = 0;
		depth = 1;
		cfstack[0] = firstfork[wilds];
	}
	if(!depth)
	{
		if(!wilds) return NULL;
		for(wilds--; !firstfork[wilds]; wilds--)
			if(!wilds) return NULL;
		lenstack[0] = 0;
		depth = 1;
		cfstack[0] = firstfork[wilds];
	}
	curfork = cfstack[--depth];
	len = lenstack[depth];
	*wildcnt = wilds;
	while(1)
	{
		if(curfork->hasnot)
		{
			lenstack[depth] = len;
			cfstack[depth++] = curfork->hasnot;
		}
		conns[len++] = curfork->conn;
		if(curfork->isend)
		{
			if(len == 1 && conns[0] == 0) *length = 0;
			else *length = len;
			return curfork->has.fe;
		}
		curfork = curfork->has.cf;
	}
	return NULL;	// Silly compiler.
}

void saveresults_rec(unsigned int number, unsigned int seq, unsigned char *bits)
{
	if(*bits == 255)
	{
		if(outbuflen == OUTBUFSIZE)
		{
			fwrite(outbuf, OUTBUFSIZE*4, 1, outfile);
			outbuflen = 0;
		}
		outbuf[outbuflen++] = number;
		outbuf[outbuflen++] = seq;
//printf("Saved: conf=%d, seq = %08x\n", number, seq);
	}
	else
	{
		saveresults_rec(number, seq, bits + 1);
		saveresults_rec(number, seq | (1 << *bits), bits + 1);
	}
}

void saveresults(unsigned int number, unsigned int Pmask, unsigned int Hmask)
{
//printf("Found: conf=%d, hp = %s\n", number, tostring(Pmask, Hmask));
	if(number == (unsigned int)-1)
	{
		if(outbuflen)
		{
			fwrite(outbuf, outbuflen*4, 1, outfile);
			outbuflen = 0;
		}
	}
	else
	{
		unsigned int fixed = Pmask | Hmask;
		if(fixed == ((1 << CHAINLENGTH) - 1))
		{
			unsigned char b = 255;
			saveresults_rec(number, Hmask, &b);
		}
		else
		{
			unsigned char b[CHAINLENGTH + 1];
			int l = 0;
			for(int i = CHAINLENGTH - 1; i >= 0; i--)
				if(!(fixed & (1 << i))) b[l++] = i;
			b[l] = 255;
			saveresults_rec(number, Hmask, b);
		}
	}
}

int cmpconn(unsigned char *a, unsigned char *b)
{
	if(*a < *b) return -1;
	return 1;
}

// Alter the contact maps
void conn_translate(const int nodes, const int position, const int hp,
	unsigned int Pmask, unsigned int Hmask)
{
	if(hp)	// H - 
	{
		Hmask |= (1 << position);

		// The position of the Nth H, counting even and odd positions separately
		char Hpos[CHAINLENGTH / 2 + 1][2];
		for(int i = 0, eh = 0, oh = 0; i <= position; i++)
		{
			if(Hmask & (1 << i))
			{
				if(i & 1) Hpos[oh++][0] = i;
				else Hpos[eh++][1] = i;
			}
		}

		// Position of the last H of all H->x conns for each x, offset by one.
		char xHconns[CHAINLENGTH];
		char *xHclr = &xHconns[position+1];

		for(int i = 0; i < nodes; i++)
		{
			struct node_d *d = &d_arr[i];
			unsigned char *conns = d->conns;
			int len = 0;
			int needSort = 0;

			// Set to 0 for the possibly affectable positions
			memset(xHclr, 0, CHAINLENGTH - 1 - position);

			for(int j = 0; j < d->connlen; j++)
			{
				int t = conn_to[conns[j]];

				if(t == position)		// Must be HH
				{
					d->wilds++;
				}
				else
				{
					int f = conn_from[conns[j]];
					
					if(f == position)
					{
						int nf = Hpos[(int)xHconns[t]][t & 1];
						if(nf == f)
							conns[len++] = conns[j];
						else
						{
							conns[len++] = conn_fromto[nf][t];
							needSort = 1;
						}
					}
					else
					{
						if((1 << f) & Hmask)
							xHconns[t]++;

						conns[len++] = conns[j];
					}
				}
			}
			d->connlen = len;

			if(needSort)
				qsort(conns, len, 1, (int (*)(const void *,const void *))cmpconn);
		}
	}
	else		// P - just remove all to/from this position
	{
		unsigned int mask = 1 << position;
		for(int i = 0; i < nodes; i++)
		{
			struct node_d *d = &d_arr[i];
			unsigned char *conns = d->conns;
			int len = 0;
			for(int j = 0; j < d->connlen; j++)
			{
				if(!(conn_mask[conns[j]] & mask))
					conns[len++] = conns[j];
			}
			d->connlen = len;
		}
	}
}

int comparenodes(struct node_d *n1, struct node_d *n2)
{
	if(n1->wilds == n2->wilds)
	{
		if(n1->connlen == n2->connlen)
		{
			return memcmp(n1->conns, n2->conns, n1->connlen);
		}
		return (n1->connlen > n2->connlen) ? -1 : 1;
	}
	return (n1->wilds > n2->wilds) ? -1 : 1;
}

void quick_sort(int length)
{
	qsort(d_arr, length, sizeof(struct node_d),
		(int (*)(const void *,const void *))comparenodes);
}

// Saves d_arr in some way. Pass -nodes to load.
void loadsavearray(int id, int nodes)
{
	static void *arrbuf[CHAINLENGTH];
	static char savetype[CHAINLENGTH];

	int size = nodes * sizeof(struct node_d);
	if(nodes < 0)
	{
		if(!savetype[id])
		{
			d_arr = (struct node_d *)arrbuf[id];
		}
		else
		{
			d_arr = (struct node_d *)malloc(-size);
			if(!d_arr) die("Out of memory 2!\n");
			rewind((FILE *)arrbuf[id]);
			fread(d_arr, -size, 1, (FILE *)arrbuf[id]);
			fclose((FILE *)arrbuf[id]);
			remove(tempfilename[id]);
		}
	}
	else
	{
		if(size <= MAXARRAYSIZE_NOFILE)
		{
			arrbuf[id] = malloc(size);
			if(!arrbuf[id]) die("Out of memory 3!\n");
			memcpy(arrbuf[id], d_arr, size);
			savetype[id] = 0;
		}
		else
		{
			arrbuf[id] = fopen(tempfilename[id], "wb+");
			if(!arrbuf[id]) die("Temp file error\n");
			fwrite(d_arr, size, 1, (FILE *)arrbuf[id]);
			savetype[id] = 1;
		}
	}
}

int tree_to_array(int nodes, unsigned int Pmask, unsigned int Hmask)
{
	if(!nodes) return 0;

//printf("At %s nodes=%d\n", tostring(Pmask, Hmask),nodes);

	if((Pmask & Hmask) == (1 << CHAINLENGTH) - 1)
	{
//if(nodes > 1) die("tree error 3\n");
		struct node_d dummynode;
		int nolen, wilds = 0;
		struct forkend *fe = nextforkend(0, dummynode.conns, &nolen, &wilds);
//if(nolen) die("tree error 1\n");
//if(!fe || nextforkend(1, dummynode.conns, &nolen, &nolen)) die("tree error 2\n");

		if(fe->number & 0x80000000) return 0;	// Degenerate
		if(!wilds) return 0;
		saveresults(fe->number & ~0x80000000, Pmask, Hmask);
		return 0;
	}

	struct node_d *node = d_arr;
	int len, wilds;
	unsigned char conns[MAXNRCONNS];
	int nondegenerate = 0;

	for(struct forkend *fe = nextforkend(0, conns, &len, &wilds); fe;
		fe = nextforkend(1, conns, &len, &wilds))
	{
		node->connlen = len;
		node->wilds = wilds;
		node->number = fe->number;
		if(!(fe->number & 0x80000000)) nondegenerate = 1;
		memcpy(node->conns, conns, len);
//		if(len != MAXNRCONNS) memset(&node->conns[len], 0, MAXNRCONNS - len);

//printf("normal node, len=%d, wilds=%d deg=%d  ",len, node->wilds, node->degen);
//for(int q=0;q<MAXNRCONNS;q++)printf(" %d",node->conns[q]);
//printf("\n");

		node++;

		for(struct subset *ss = fe->subsets; ss; ss = ss->next)
		{
			memcpy(node->conns, ss->conns, ss->len);
			node->connlen = ss->len;
			node->wilds = ss->wilds;

//printf("subset node, len=%d, wilds=%d  ",node->connlen,node->wilds);
//for(int q=0;q<MAXNRCONNS;q++)printf(" %d",node->conns[q]);
//printf("\n");
			node++;
		}
	}

	if(nondegenerate)
	{
		if(nodes == 1 && d_arr->connlen == 0)
			saveresults(d_arr->number & ~0x80000000, Pmask, Hmask);
		else return 1;
	}
	return 0;
//else printf("all degen at %s\n", tostring(Pmask, Hmask));
}


// This function frees the memory pointed to by d_arr(when it is called)
// before it returns.
void newrecursion(int nodes, int depth, unsigned int Pmask, unsigned int Hmask)
{
	loadsavearray(depth, nodes);

	conn_translate(nodes, depth, 0, Pmask, Hmask);
	quick_sort(nodes);
	for(int i = 0; i < MAXNRCONNS + 1; i++) firstfork[i] = NULL;
	int added = do_the_subset_thing(nodes);
	if(added)
	{
		if(nodes > MAXARRAYNODES_NOREALLOC)
		{
			d_arr = (struct node_d *)realloc(d_arr, added * sizeof(struct node_d));
			if(!d_arr) die("Out of memory 4!\n");
		}
		int recurse = tree_to_array(added, Pmask | (1 << depth), Hmask);
		mymalloc(-1);	// Free the tree
		if(recurse) newrecursion(added, depth + 1, Pmask | (1 << depth), Hmask);
		else free(d_arr);
	}
	else
	{
		mymalloc(-1);
		free(d_arr);
	}
	// Now d_arr is gone, and we'll get a new d_arr with out saved data.

	loadsavearray(depth, -nodes);

	if(depth > 2 || (depth == 2 && Hmask & 1))
	{
		conn_translate(nodes, depth, 1, Pmask, Hmask);
		quick_sort(nodes);
		for(int i = 0; i < MAXNRCONNS + 1; i++) firstfork[i] = NULL;
		added = do_the_subset_thing(nodes);
		if(added)
		{
			if(nodes > MAXARRAYNODES_NOREALLOC)
			{
				d_arr = (struct node_d *)realloc(d_arr, added * sizeof(struct node_d));
				if(!d_arr) die("Out of memory 5!\n");
			}
			int recurse = tree_to_array(added, Pmask, Hmask | (1 << depth));
			mymalloc(-1);	// Free the tree
			if(recurse) newrecursion(added, depth + 1, Pmask, Hmask | (1 << depth));
			else free(d_arr);
		}
		else
		{
			mymalloc(-1);
			free(d_arr);
		}
	}
	else	// Special case: 2 first positions give no change to the CMs if H.
	{
		newrecursion(nodes, depth + 1, Pmask, Hmask | (1 << depth));
	}
}

void do_sort(char *infn, char *outfn)
{
	outfile = fopen(outfn, "wb");
	if(!outfile)
	{
		perror("Can't open output file");
		exit(1);
	}

	for(int i = 0; i < CHAINLENGTH; i++)
		sprintf(tempfilename[i], "%02d_%02d.tmp", CHAINLENGTH, i);

	mymalloc(-2);
	initconntrans();

	int nodes = loadinput(infn);
	if(nodes) newrecursion(nodes, 0, 0, 0);
	saveresults((unsigned int)-1, 0, 0);

	fclose(outfile);
}


int main(int argc, char **argv)
{
	char fnamebuf[2][32];

	setvbuf(stdout, NULL, _IOLBF, BUFSIZ);

	printf("[noded_assume_position]\n");
	printf("CHAINLENGTH,MAXNRCONNS,MAXCONNR,node_d: %d,%d,%d,%zd\n",
		CHAINLENGTH, MAXNRCONNS, MAXCONNR, sizeof(struct node_d));

	if(argc == 3) do_sort(argv[1], argv[2]);
	else if(argc < 2)
	{
		sprintf(fnamebuf[0], "n_%02d.conn2", CHAINLENGTH);
		sprintf(fnamebuf[1], "n_%02d.ures", CHAINLENGTH);
		do_sort(fnamebuf[0], fnamebuf[1]);
	}
	else
	{
		printf("Syntax: noded_assume_position [input_file output_file]\n");
	}

	fflush(stdout);
//	system("ps v");

	return 0;
}
