#include <vector>
#include <iostream>
#include <fstream>
#include <map>
#include <set>
#include <ctime>
#include <cstdio>
#include <numeric>
#include "knet.h"

using namespace std;

#define DEPENDALL 0

void Knet::killNonRegRecursion(knet& net, int n)
{
	knode &k = net[n];
	// Set the rule to invalid to be able to recognize already-deleted nodes
	k.rule.makeBad();		// Used to be k.r = badrule
	
	for(unsigned int j = 0; j < k.inputs.size(); j++)
	{
		int n = k.inputs[j];
		if(!--net[n].extra)
			killNonRegRecursion(net, n);
	}
}

void Knet::killNonRegulatingNodes(knet& net)
{
	int s = net.size();
	// Count references to each node
	for(int i = 0; i < s; i++)
		net[i].extra = 0;
	for(int i = 0; i < s; i++)
	{
		knode &k = net[i];
		for(unsigned int j = 0; j < k.inputs.size(); j++)
			net[k.inputs[j]].extra++;
	}
	for(int n = 0; n < s; n++)
	{
		knode &k = net[n];
		if(!k.rule.isBad() && !k.extra)
			killNonRegRecursion(net, n);
	}
}


void Knet::shrinkNet(const knet& net, knet& net2)
{
	int s = net.size(), s2 = 0;
	vector<nodeid> mapto(s);
	for(int i = 0; i < s; i++)
	{
		const knode &k = net[i];
		if(!k.rule.isBad())
			mapto[i] = s2++;
	}
	net2.resize(s2);

	for(int i = 0, i2 = 0; i < s; i++)
	{
		const knode &k = net[i];
		if(!k.rule.isBad())
		{
			knode &k2 = net2[i2++];
			k2.rule = k.rule;
			k2.inputs.resize(k.inputs.size());
			for(unsigned int j = 0; j < k.inputs.size(); j++)
				k2.inputs[j] = mapto[k.inputs[j]];
		}
	}
}

void Knet::prepareNet(const knet &rawnet, knet& net)
{
	knet tnet(rawnet);

	rule_t::randomize(tnet, DEPENDALL);
	rule_t::clampNodes(tnet);
	killNonRegulatingNodes(tnet);
	shrinkNet(tnet, net);
}

void Knet::prepareNetNoshrink(knet& net)
{
	rule_t::randomize(net, DEPENDALL);
}

void Knet::statecompress(state &st, smallstate &sst)
{
	size_t s = st.size();
	sst.resize((st.size() + 31) / 32);
	unsigned int b = 0;
	for(size_t i = 0; i < s; i++)
	{
		b |= st[i] << (i & 31);
		if((i & 31) == 31)
		{
			sst[i / 32] = b;
			b = 0;
		}
	}
	if(s & 31)
		sst[s / 32] = b;
}

// in: s1 is the start state and s1 == ssave.
void Knet::findlowest(knet &net, state &s1, state &s2, state &ssave, cyclen_t len)
{
	for(int i = 0; i < len - 1; i++)
	{
		rule_t::evaluate(net, s1, s2);
		if(s2 < ssave)
			ssave = s2;
		s1.swap(s2);
	}
}

void Knet::findcycle(knet &net, state &s1, state &s2, state &ssave, cyclen_t &len,
	bool rand, const int turnlimit)
{
	if(rand)
		s1.randomize();
	for(int turns = 1; ; turns <<= 1)
	{
		if(turns >= turnlimit)
		{
			len = 0;
			return;
		}
		ssave = s1;
		for(int i = 0; i < turns; i++)
		{
			rule_t::evaluate(net, s1, s2);
			s1.swap(s2);
			if(s1 == ssave)
			{
				len = i + 1;
				findlowest(net, s1, s2, ssave, len);
				return;
			}
		}
	}
}

void Knet::gatherStatisticsOnCycles(const knet &rawnet, const int starts,
	const int nets, ostream &err, bool synch, int maxlen,
	vector<double> &means)
{
	if((int)means.size() != maxlen + 1)
	{
		err << "Error in Knet::gatherStatisticsOnCycles(): "
			"storage vectors the wrong size\n" <<
			means.size() << "/" << maxlen + 1 << "\n";
		exit(1);
	}
	vector<double> C(maxlen + 1, 0);
	knet net;

	for(int j = 0; j < nets; j++)
	{
//		if(!(j % 50))
//			err << "\n" << ":"<< j << " ";
//		err << "-";
//		err.flush();

		map<cycle,int> ccount;
		prepareNet(rawnet, net);

		state s1(net.size()), s2_maybe(net.size()), ssave(net.size());
		
		state &s2 = synch ? s2_maybe : s1;
		smallstate sst;
		cycle c(net.size());

		for(int i = 0; i < starts; i++)
		{
			findcycle(net, s1, s2, ssave, c.len);
			if(c.len > 0)
			{
				statecompress(ssave, c.s);
				ccount[c]++;
			}
		}

		vector<int> lenc(maxlen+1, 0);
		lenc[0] = ccount.size();
		for(map<cycle,int>::iterator i = ccount.begin(); i != ccount.end(); i++)
		{
			int l = (*i).first.len;
			if(l <= maxlen) lenc[l]++;
		}
		for(int i = 0; i <= maxlen; i++)
			C[i] += double(lenc[i]);
	}
//	err << "\n\n";

	for(int i = 0; i <= maxlen; i++)
		means[i] += C[i] / double(nets);
}

