/**
 * Filename     : compartmentNeighborhood.cc
 * Description  : Classes describing compartment neighborhoods
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : September 2005
 * Revision     : $Id: compartmentNeighborhood.cc,v 1.12 2006/03/28 08:54:59 henrik Exp $
 */
#include<cmath>
#include<cassert>
#include"compartmentNeighborhood.h"
#include"baseCompartmentNeighborhood.h"
#include <set>

    
//!Constructor for the SphereDistance class
NeighborhoodDistanceSphere::
NeighborhoodDistanceSphere(std::vector<double> &paraValue, 
			   std::vector< std::vector<size_t> > 
			   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "NeighborhoodDistanceSphere::"
	      << "NeighborhoodDistanceSphere() "
	      << "Uses three parameters r_frac, upDateFlag and dt_min.\n";
    exit(0);
  }
  if( indValue.size() ) {
    std::cerr << "NeighborhoodDistanceSphere::"
	      << "NeighborhoodDistanceSphere() "
	      << "No variable index is used.\n";
    exit(0);
  }
  //Check for consistency in parameter values
  //////////////////////////////////////////////////////////////////////
  if( paraValue[0]<0.0 ) {
    std::cerr << "NeighborhoodDistanceSphere::"
	      << "NeighborhoodDistanceSphere() "
	      << "parameter(0) (r_frac) must be larger than zero.\n";
    exit(0);
  }
  if( paraValue[1]!=0.0 && paraValue[1]!=1.0 ) {
    std::cerr << "NeighborhoodDistanceSphere::"
	      << "NeighborhoodDistanceSphere() "
	      << "parameter(1) (updateFlag) must be 0 or 1.\n";
    exit(0);
  }
  
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("neighborhoodDistanceSphere");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "r_frac";
  tmp[1] = "updateFlag";
  tmp[2] = "dt_min";
  setParameterId( tmp );
}

//! Creates the (potential) neighborhood before simulation
/*! Simple O(N^2) function for finding neighboring compartments.
  rFactor gives a factor of the sum of the two cell radiuses for which
  a threshold is set. If the distance is larger than the threshold,
  the cells are not considered as neighbors. It returns the total
  number of neighbors found.
*/
unsigned int NeighborhoodDistanceSphere::
create(Organism &O, std::vector< std::vector<double> > &y,
       double t ) {
  
  size_t N = O.numCompartment();
  std::vector< std::vector<size_t> > neighbor(N);
  double d,r;
  unsigned int numNeigh=0;
  
  size_t rIndex = O.numTopologyVariable()-1;
  size_t eStart = 0;
  
  for(size_t i=0 ; i<N ; i++ )
    for(size_t j=i+1 ; j<N ; j++ ) {
      r = y[i][rIndex] + y[j][rIndex];
      d=0.;
      for( size_t dim=eStart ; dim<rIndex ; dim++ )
	d += (y[i][dim]-y[j][dim])*(y[i][dim]-y[j][dim]);
      d = std::sqrt( d );
      
      if( d<=r*parameter(0) ) {
        //Add neighbor
        neighbor[i].push_back(j);
        neighbor[j].push_back(i);
        numNeigh++;
      }
    }
  //Copy the new neighbours into the cells (Caveat: Not a good solution)
  for(size_t i=0 ; i<N ; i++ )
    O.compartment(i).setNeighbor( neighbor[i] );

  //Set the previosu time variable to current t value
  setPreviousTime(t);
  setNumNeighbor(numNeigh);

  return numNeigh;
}

//! Updates neighborhood during simulation
unsigned int NeighborhoodDistanceSphere::
update(Organism &O, std::vector< std::vector<double> > &y,
       double t ) {
  
  if( parameter(1) == 1.0 && t-previousTime()>parameter(2) )
    return create(O,y,t);

  return numNeighbor();
}

