/**
 * Filename     : creation.cc
 * Description  : Classes describing creation updates
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 *              : Pontus Melke
 * Created      : December 2003
 * Revision     : $Id: creation.cc,v 1.7 2006/03/18 12:23:08 henrik Exp $
 */
#include"creation.h"
#include"baseReaction.h"
#include<cmath>

//!Constructor for the CreationZero class
CreationZero::CreationZero(std::vector<double> &paraValue, 
			   std::vector< std::vector<size_t> > 
			   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "CreationZero::CreationZero() "
	      << "Uses only zero parameter k_c\n";
    exit(0);
  }
  if( indValue.size() ) {
    std::cerr << "CreationZero::CreationZero() "
	      << "No variable index is used.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("creationZero");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_c";
  setParameterId( tmp );
}

//! Derivative contribution for a creation proportional to concentrations
/*! Deriving the time derivative contribution from a creation using
  the values in y and add the results to dydt. Compartment is the cell
  and varIndex is the index of the variable to be updated. The
  parameter is k_c in dc/dt = k_c .*/
void CreationZero::derivs(Compartment &compartment,size_t varIndex,
			  std::vector< std::vector<double> > &y,
			  std::vector< std::vector<double> > &dydt ) {
  
  dydt[compartment.index()][varIndex] += parameter(0);
}

//!Constructor for the CreationOne class
CreationOne::CreationOne(std::vector<double> &paraValue, 
			 std::vector< std::vector<size_t> > 
			 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "CreationOne::CreationOne() "
	      << "Uses only one parameter k_c\n";
    exit(0);
  }
  if( indValue.size() != 1 || indValue[0].size() != 1  ) {
    std::cerr << "CreationOne::CreationOne() "
	      << "One variable index needed.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("creationOne");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_c";
  setParameterId( tmp );
}

//! Derivative contribution for a creation proportional to one concentration
/*! Deriving the time derivative contribution from a creation
  using the values in y and add the results to dydt. Compartment is
  the cell and varIndex is the index of the variable to be
  updated. The parameter is k_c in dc_i/dt = k_c * c_i
*/
void CreationOne::derivs(Compartment &compartment,size_t varIndex,
			 std::vector< std::vector<double> > &y,
			 std::vector< std::vector<double> > &dydt ) {
  
  dydt[compartment.index()][varIndex] += parameter(0)*
    y[compartment.index()][ variableIndex(0,0) ];
}


