/**
 * Filename     : massAction.cc
 * Description  : Classes describing mass action reactions
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : November 2003
 * Revision     : $Id: massAction.cc,v 1.3 2006/03/18 00:05:14 henrik Exp $
 */
#include"massAction.h"
#include"baseReaction.h"


//!Constructor for the MassActionEnzymatic class
MassActionEnzymatic::MassActionEnzymatic(std::vector<double> &paraValue, 
					 std::vector< std::vector<size_t> > 
					 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=3 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "Three levels of variable indices are used.\n"
	      << "One for reactants, one for products and one for enzymes\n";
    exit(0);
  }
  if( indValue[0].size()<1 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "If no reactants given, there will be no reaction...\n";
    exit(0);
  }
  if( indValue[2].size()<1 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "If no enzymes given, there will be no reaction...\n";
    exit(0);
  }
  
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("massActionEnzymatic");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

//! Derivative contribution for a one way mass action reaction with enzymes
/*! Deriving the time derivative contribution from an enzymatic mass action
  reaction using the values in y and add the results to dydt. Compartment is
  the cell and varIndex is not used. Instead reactants [R], products [P] and
  enzymes [E] are stored in variableIndexLevel 0/1/2. [R] and [P] are
  updated. The parameter is k_f in d[P]/dt = k_f * \Prod [E] * \Prod [R]
  d[R]/dt = - k_f \Prod [E] \Prod [R] */
void MassActionEnzymatic::derivs(Compartment &compartment,size_t varIndex,
				 std::vector< std::vector<double> > &y,
				 std::vector< std::vector<double> > &dydt ) {
  
  double rate = parameter(0);
  if( numVariableIndex(0) )
    for( size_t i=0 ; i< numVariableIndex(0) ; i++ )
      rate *= y[compartment.index()][variableIndex(0,i)];
  else //No reaction defined...
    rate *= 0.;
  
  if( numVariableIndex(2) )
    for( size_t i=0 ; i< numVariableIndex(2) ; i++ )
      rate *= y[compartment.index()][variableIndex(2,i)];
  else //No reaction defined...
    rate *= 0.;
  
  for( size_t i=0 ; i< numVariableIndex(0) ; i++ )
    dydt[compartment.index()][variableIndex(0,i)] -= rate;
  
  for( size_t i=0 ; i< numVariableIndex(1) ; i++ )
    dydt[compartment.index()][variableIndex(1,i)] += rate;
}

