/**
 * Filename     : rk5Adaptive.cc
 * Description  : Functions for a simulator class defined in rk5Adaptive.h
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : July 2003
 * Revision     : $Id: rk5Adaptive.cc,v 1.17 2006/03/18 00:05:14 henrik Exp $
 */
#include"rk5Adaptive.h"

Rk5Adaptive::Rk5Adaptive() {
}

Rk5Adaptive::Rk5Adaptive(const Rk5Adaptive & rk5AdaptiveCopy) {
  
  y_.resize( rk5AdaptiveCopy.N() );
  dydt_.resize( rk5AdaptiveCopy.N() );
  int Mtmp=rk5AdaptiveCopy.M();
  for( size_t i=0 ; i<N() ; i++ ) {
    y_[i].resize( Mtmp );
    dydt_[i].resize( Mtmp );
  }    
  for( size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ ) {
      y_[i][j] = rk5AdaptiveCopy.y(i,j);
      dydt_[i][j] = rk5AdaptiveCopy.dydt(i,j);
    }
  std::cerr << "Rk5Adaptive::Rk5Adaptive(const Rk5Adaptive & rk5AdaptiveCopy)"
	    << " \nWarning, parameters not copied...\n";
}

//!The main creator from an Organism object
/*! Creates a rk5Adaptive from an organism by copying variable values
  and defining simulation parameters.*/
Rk5Adaptive::Rk5Adaptive( Organism *OValue, std::string paraFile ) {
    O_ = OValue;
  // Resize and define parameters
  //////////////////////////////////////////////////////////////////////
  size_t N = O_->numCompartment();
  size_t M=O_->numVariable();
  
  // Copy values from the Organism into the y matrix and resize dydt
  //////////////////////////////////////////////////////////////////////
  y_.resize(N);
  dydt_.resize(N);
  for( size_t i=0 ; i<N ; i++ ) {
    y_[i].resize(M);
    dydt_[i].resize(M);
    for( size_t j=0 ; j<M ; j++ )
      y_[i][j] = O_->compartment(i).variable(j); 
  }
  // Read parameter values from file, or use default values
  //////////////////////////////////////////////////////////////////////
  if( paraFile=="" )
    parameterDefault();
  else
    parameterFromFile(paraFile);
  
  std::cerr << "*** Rk5Adaptive ***\n";
  std::cerr << "Simulation time: " << startTime_ << " - " << endTime_ << "\n";
  std::cerr << "Printing " << numPrint_ << " times (flag=" << printFlag_ 
	    << ")\n";
  std::cerr << "h1=" << h1_ << "\t" << "eps=" << eps_ << "\n\n";
}

Rk5Adaptive::~Rk5Adaptive() {
  //std::cerr << "Rk5Adaptive::~Rk5Adaptive()\n";
}

//!Reads simulation parameters from file
void Rk5Adaptive::parameterFromFile( std::string file ) {
  
  const char *tmp = file.c_str();
  std::ifstream IN( tmp );
  if( !IN ) {
    std::cerr << "Rk5Adaptive::parameterFromFile() - "
	      << "Cannot open file " << file << "\n\n\7";exit(-1);}
  //Read in the needed parameters
  IN >> startTime_;
  t_=startTime_;
  IN >> endTime_;
  
  IN >> printFlag_;
  IN >> numPrint_;
  
  IN >> h1_;
  IN >> eps_;
  
  IN.close();
}

//!Sets default values for simulation parameters
void Rk5Adaptive::parameterDefault() {
  
  startTime_=t_=0.;
  endTime_=100;
  
  printFlag_=0;
  numPrint_=0;
  
  h1_=0.01;
  eps_=1e-6;
}

//!Runs a simulation of an organism model
void Rk5Adaptive::simulate() {
  
  //double tiny=1e-30;//Using NR definition
  double tiny=1e-9*eps_;//Caveat! Using new definition
  double  h,hNext,hDid;
  
  //Check that h1 and endTime-startTime are > 0
  //////////////////////////////////////////////////////////////////////
  if( h1_>0. && (endTime_-startTime_)>0. )
    h=h1_;
  else {//either h or (endTime-startTime) <=0
    std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
	      << "simulation. No simulation performed.\n";
    return;
  }
  
  //Introduce the neighborhood
  //////////////////////////////////////////////////////////////////////
  if( O_->numNeighborhood() )
    O_->neighborhoodCreate(y_,t_);
  if( y_.size() && y_.size() != dydt_.size() ) {
    dydt_.resize( y_.size(),y_[0]);
  }
  assert( y_.size() == O_->numCompartment() && y_.size()==dydt_.size());
  
  //Create all vectors that will be needed here and by rkqs and rkck!
  //////////////////////////////////////////////////////////////////////
  //Used here
  std::vector< std::vector<double> > yScal( N() );
  //Used by rkqs
  std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
  //used by rkck
  std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
    ak5( N() ),ak6( N() ),yTempRkck( N() );
  //Resize each vector
  for( size_t i=0 ; i<N() ; i++ ) {
    yScal[i].resize(M());
    yErr[i].resize( M() );
    yTemp[i].resize( M() );
    ak2[i].resize(M());
    ak3[i].resize(M());
    ak4[i].resize(M());
    ak5[i].resize(M());
    ak6[i].resize(M());
    yTempRkck[i].resize(M());
  }
  assert( y_.size()==yScal.size() && y_.size()==yErr.size() );
  assert( y_.size()==yTemp.size() && y_.size()==ak2.size() );
  assert( y_.size()==ak3.size() && y_.size()==ak4.size() );
  assert( y_.size()==ak5.size() && y_.size()==ak6.size() );
  assert( y_.size()==yTempRkck.size() );
  
  // Initiate print times
  //////////////////////////////////////////////////////////////////////
  double printTime=endTime_+tiny;
  double printDeltaTime=endTime_+2.*tiny;
  if( numPrint_<=0 )//No printing
    printFlag_=0;
  else if( numPrint_==1 ) {//Print last point (default)
  }
  else if( numPrint_==2 ) {//Print first/last point
    printTime=startTime_-tiny;
  } 
  else {//Print first/last points and spread the rest uniformly
    printTime=startTime_-tiny;
    printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
  }

  // Go
  //////////////////////////////////////////////////////////////////////
  t_=startTime_;
  numOk_ = numBad_ = 0;
  for( unsigned int nstp=0 ; ; nstp++ ) {
    //Update the derivatives
    O_->derivs(y_,dydt_);

    //Calculate 'scaling' for error measure
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t j=0 ; j<M() ; j++ ) {
        yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
	//Try this modification to see whether it fasten up things
	//if( yScal[i][j]<1.0 )
	//yScal[i][j] = 1.0;
      }

    //Print if applicable 
    if( printFlag_ && t_ >= printTime ) {
      printTime += printDeltaTime;
      print();
    }

    //Check if step is larger than max allowed
    //max step end is min of endTime_ and printTime
    double tMin= endTime_<printTime ? endTime_ : printTime;
    if( t_+h>tMin ) h=tMin-t_;

    //Update
    rkqs(h,hDid,hNext,yScal,
	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
    if (hDid == h) ++numOk_; else ++numBad_;


    //////////////////////////////////////////////////////////////////////
    //UGLY TEMPORARY SOLUTION!
    //Update elastic wall
    if( O_->numTopology() ) {
      for( size_t rCount=0 ; rCount<O_->topology().numReaction() ; rCount++ )
	if( O_->topology().reaction(rCount)->id()
	    =="springElasticWallEllipse" || 
	    O_->topology().reaction(rCount)->id()
	    =="springElasticWallCigar2" )
	  O_->topology().reaction(rCount)->update(hDid);
    }
    //UGLY TEMPORARY SOLUTION!
    //////////////////////////////////////////////////////////////////////

    // Check for cell divisions, compartmental removal etc
    O_->compartmentChange(y_,dydt_,t_);

    //Rescale all temporary vectors as well
    if(y_.size() != yScal.size() ) {
      yScal.resize( N(), yScal[0] );
      yTemp.resize( N(), yTemp[0] );
      yErr.resize( N(), yErr[0] );
      ak2.resize( N(), ak2[0] );
      ak3.resize( N(), ak3[0] );
      ak4.resize( N(), ak4[0] );
      ak5.resize( N(), ak5[0] );
      ak6.resize( N(), ak6[0] );
      yTempRkck.resize( N(), yTempRkck[0] );
    }

    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);

    //If the end t is passed return (print if applicable)
    if ( t_>=endTime_ ) {
      if( printFlag_ ) {
	//Update the derivatives
	O_->derivs(y_,dydt_);
	print();
      }
      std::cerr << "Simulation done.\n"; 
      return;
    }
    //Warn for small step sizes...
    //  if (fabs(hNext) <= hMin) {
    //        std::cerr << "Warning: Step size small (" << hNext
    //  		<< ") in rk5Adaptive::simulate at time " 
    //  		<< t << "\n"; /*exit(-1);*/ }
    h=hNext;
    //Do not take larger steps than h1
    if( h>h1_ )
      h=h1_;
  }
}

//!Runs a simulation of an organism model where mechanics is assumed fast
/*! Runs a simulation where the mechanics equations are treated
  specifically to always be in equilibrium. At each step the
  mechanical equations are run until convergence.
*/
void Rk5Adaptive::
simulateWithConvergingMechanics(double cEps,double cTdiff) {
  
  //double tiny=1e-30;//Using NR definition
  double tiny=1e-9*eps_;//Caveat! Using new definition
  double  h,hNext,hDid;
  
  //Check that h1 and endTime-startTime are > 0
  //////////////////////////////////////////////////////////////////////
  if( h1_>0. && (endTime_-startTime_)>0. )
    h=h1_;
  else {//either h or (endTime-startTime) <=0
    std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
	      << "simulation. No simulation performed.\n";
    return;
  }
  
  //Create all vectors that will be needed here and by rkqs and rkck!
  //////////////////////////////////////////////////////////////////////
  //Used here
  std::vector< std::vector<double> > yScal( N() );
  //Used by rkqs
  std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
  //used by rkck
  std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
    ak5( N() ),ak6( N() ),yTempRkck( N() );
  //Resize each vector
  for( size_t i=0 ; i<N() ; i++ ) {
    yScal[i].resize(M());
    yErr[i].resize( M() );
    yTemp[i].resize( M() );
    ak2[i].resize(M());
    ak3[i].resize(M());
    ak4[i].resize(M());
    ak5[i].resize(M());
    ak6[i].resize(M());
    yTempRkck[i].resize(M());
  }
  
  // Initiate print times
  //////////////////////////////////////////////////////////////////////
  double printTime=endTime_+tiny;
  double printDeltaTime=endTime_+2.*tiny;
  if( numPrint_<=0 )//No printing
    printFlag_=0;
  else if( numPrint_==1 ) {//Print last point (default)
  }
  else if( numPrint_==2 ) {//Print first/last point
    printTime=startTime_-tiny;
  } 
  else {//Print first/last points and spread the rest uniformly
    printTime=startTime_-tiny;
    printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
  }
  
  //Introduce the neighborhood
  //////////////////////////////////////////////////////////////////////
  if( O_->numNeighborhood() )
    O_->neighborhoodCreate(y_,t_);
  
  // Go
  //////////////////////////////////////////////////////////////////////
  t_=startTime_;
  numOk_ = numBad_ = 0;
  for( unsigned int nstp=0 ; ; nstp++ ) {
    
    //Update the derivatives
    O_->derivs(y_,dydt_);
    
    //Calculate 'scaling' for error measure
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t j=0 ; j<M() ; j++ ) {
        yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
	//Try this modification to see whether it fasten up things
	//if( yScal[i][j]<1.0 )
	//yScal[i][j] = 1.0;
      }

    //Print if applicable 
    if( printFlag_ && t_ >= printTime ) {
      printTime += printDeltaTime;
      print();
    }
    
    //Check if step is larger than max allowed
    //max step end is min of endTime_ and printTime
    double tMin= endTime_<printTime ? endTime_ : printTime;
    if( t_+h>tMin ) h=tMin-t_;
    
    //Update
    rkqs(h,hDid,hNext,yScal,
	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
    if (hDid == h) ++numOk_; else ++numBad_;
    
    
    //////////////////////////////////////////////////////////////////////
    //UGLY TEMPORARY SOLUTION!
    //Update elastic wall
    if( O_->numTopology() ) {
      for( size_t rCount=0 ; rCount<O_->topology().numReaction() ; rCount++ )
	if( O_->topology().reaction(rCount)->id()
	    =="springElasticWallEllipse" || 
	    O_->topology().reaction(rCount)->id()
	    =="springElasticWallCigar2" )
	  O_->topology().reaction(rCount)->update(hDid);
    }
    //UGLY TEMPORARY SOLUTION!
    //////////////////////////////////////////////////////////////////////
    
    //Converge mechanics by simulating mech eq 
    updateMechanicsUntilConvergence(cEps,cTdiff,yScal,
				    yTemp,yErr,ak2,ak3,ak4,ak5,ak6,
				    yTempRkck);

    // Check for cell divisions, compartmental removal etc
    O_->compartmentChange(y_,dydt_,t_);
    
    //Rescale all temporary vectors as well
    if(y_.size() != yScal.size() ) {
      yScal.resize( N(), yScal[0] );
      yTemp.resize( N(), yTemp[0] );
      yErr.resize( N(), yErr[0] );
      ak2.resize( N(), ak2[0] );
      ak3.resize( N(), ak3[0] );
      ak4.resize( N(), ak4[0] );
      ak5.resize( N(), ak5[0] );
      ak6.resize( N(), ak6[0] );
      yTempRkck.resize( N(), yTempRkck[0] );
    }
    
    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);

    //If the end t is passed return (print if applicable)
    if ( t_>=endTime_ ) {
      if( printFlag_ ) {
	//Update the derivatives
	O_->derivs(y_,dydt_);
	print();
      }
      std::cerr << "Simulation done.\n"; 
      return;
    }
    //Warn for small step sizes...
    //  if (fabs(hNext) <= hMin) {
    //        std::cerr << "Warning: Step size small (" << hNext
    //  		<< ") in rk5Adaptive::simulate at time " 
    //  		<< t << "\n"; /*exit(-1);*/ }
    h=hNext;
    //Do not take larger steps than h1
    if( h>h1_ )
      h=h1_;
  }  
}

//!Updates mechanical equations until convergance 
/*! Runs a simulation on mechanical equations only until it
  converges. It takes a system state as input and updates only
  positional variables using all mechanical equations until this
  system converges. During the process also the neighborhood is
  allowed to be updated, but no compartmentChanges are allowed for.
*/
void Rk5Adaptive::
updateMechanicsUntilConvergence(double cEps,double cTdiff,
				std::vector< std::vector<double> > &yScal,
				std::vector< std::vector<double> > &yTemp,
				std::vector< std::vector<double> > &yErr,
				std::vector< std::vector<double> > &ak2,
				std::vector< std::vector<double> > &ak3,
				std::vector< std::vector<double> > &ak4,
				std::vector< std::vector<double> > &ak5,
				std::vector< std::vector<double> > &ak6,
				std::vector< std::vector<double> > &yTempRkck ) {
  
  //double tiny=1e-30;//Using NR definition
  double tiny=1e-9*eps_;//Caveat! Using new definition
  double h=h1_,hNext,hDid;
  double finalTime=10000.0; 

  //Temporary addition of mech eq and pos var information
  //////////////////////////////////////////////////////////////////////
  std::vector<size_t> mechEq(5);
  for( size_t i=0 ; i<mechEq.size() ; i++ )
    mechEq[i] = i+1;
  std::vector<size_t> posVar(4);
  for( size_t i=0 ; i<posVar.size() ; i++ )
    posVar[i] = i;
  //////////////////////////////////////////////////////////////////////
  
  // Go
  //////////////////////////////////////////////////////////////////////
  double t=0.0;
  //numOk_ = numBad_ = 0;
  for( unsigned int nstp=0 ; ; nstp++ ) {

    //Update the derivatives
    O_->derivsMechanical(y_,dydt_,mechEq,posVar);
    
    //Calculate 'scaling' for error measure
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<posVar.size() ; k++ ) {
	size_t j=posVar[k];
        yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
	//Try this modification to see whether it fasten up things
	//if( yScal[i][j]<1.0 )
	//yScal[i][j] = 1.0;
      }
    
    //Update
    rkqsMechanical(t,h,hDid,hNext,yScal,
		   yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck,
		   mechEq,posVar);
    //if (hDid == h) ++numOk_; else ++numBad_;
    
    //////////////////////////////////////////////////////////////////////
    //UGLY TEMPORARY SOLUTION!
    //Update elastic wall
    if( O_->numTopology() ) {
      for( size_t rCount=0 ; rCount<O_->topology().numReaction() ; rCount++ )
	if( O_->topology().reaction(rCount)->id()
	    =="springElasticWallEllipse" || 
	    O_->topology().reaction(rCount)->id()
	    =="springElasticWallCigar2" )
	  O_->topology().reaction(rCount)->update(hDid);
    }
    //UGLY TEMPORARY SOLUTION!
    //////////////////////////////////////////////////////////////////////
    
    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);

    //If the positional variables have converged 
    //or maximal T is reached, return
    double maxDydt=0.0;
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<posVar.size() ; k++ ) {
	size_t j=posVar[k];
	if( std::fabs(dydt_[i][j])>maxDydt )
	  maxDydt = std::fabs(dydt_[i][j]); 
      }
    if( maxDydt<cEps ) {
      //std::cerr << "Rk5Adaptive::updateMechanicsUntilConvergance "
      //	<< "t=" << t << " maxDydt=" << maxDydt << "\n";
      return;
    }
    else if ( t>=finalTime ) {
      std::cerr << "Rk5Adaptive::updateMechanicsUntilConvergance\n"
		<< "Warning, final time " << finalTime << " reached with"
		<< " Max dy/dt " << maxDydt << ".\n"; 
      return;
    }
    //Warn for small step sizes...
    //  if (fabs(hNext) <= hMin) {
    //        std::cerr << "Warning: Step size small (" << hNext
    //  		<< ") in rk5Adaptive::simulate at time " 
    //  		<< t << "\n"; /*exit(-1);*/ }
    h=hNext;
    //Do not take larger steps than h1
    if( h>h1_ )
      h=h1_;
  }  
}

// //!Runs a simulation of an organism model updating neighbor areas
// void Rk5Adaptive::simulateWithNeighborArea() {

//   //double tiny=1e-30;//NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;

//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }

//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( size_t i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }
  
//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   if( O_->numNeighborhood() )
//     O_->neighborhoodCreate(y_,t_);
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Find the neighbors for all compartments
//     //Only first time (for static simulations)
//     //O_->findNeighbors(y_,1.2);
//     //O_->findNeighborAreas(y_);
//     //Every step (for growing simulations)
//     //O_->findNeighbors(y_);
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( size_t i=0 ; i<N() ; i++ )
//       for( size_t j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;

//     // Check for cell division
// //     O_->cellDivision(y_,dydt_,t_);
    
// //     //Rescale all temporary vectors as well
// //     yScal.resize( N(), yScal[0] );
// //     yTemp.resize( N(), yTemp[0] );
// //     yErr.resize( N(), yErr[0] );
// //     ak2.resize( N(), ak2[0] );
// //     ak3.resize( N(), ak3[0] );
// //     ak4.resize( N(), ak4[0] );
// //     ak5.resize( N(), ak5[0] );
// //     ak6.resize( N(), ak6[0] );
// //     yTempRkck.resize( N(), yTempRkck[0] );

//     if( O_->numNeighborhood() )
//       O_->neighborhoodUpdate(y_,t_);
    
//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
// }

// //!Runs a simulation of an organism model without any neighborhood update
// void Rk5Adaptive::simulateStaticNeighborhood() {
  
//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid,hMin=1e-6;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( size_t i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }
  
//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( unsigned int nstp=0 ; ; nstp++ ) {
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( size_t i=0 ; i<N() ; i++ )
//       for( size_t j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;
    
//     // Check for cell division (not for static environment)
//     //O_->cellDivision(y_,dydt_,t_);
    
//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );
    
//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     if (fabs(hNext) <= hMin) {
//       std::cerr << "Warning: Step size small (" << hNext 
//       		<< ") in rk5Adaptive::simulateStaticNeighborhood at time " 
//       		<< t_ << "\n"; /*exit(-1);*/ 
//       hNext=hMin;
//     }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
// }

// //!Runs a simulation of an organism model until convergence
// void Rk5Adaptive::simulateUntilConvergence(double cEps,double cTdiff ) {
  
//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
//   double cT=0.0,cFirstMaxDer;
//   int cFlag=0;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( size_t i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }

//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( double nstp=0 ; ; nstp++ ) {
//     //Find the neighbors for all compartments
//     //Only first time (for static simulations)
//     if( nstp==0 ) O_->findNeighbors(y_);
//     //Every step (for growing simulations)
//     //O_->findNeighbors(y_);
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( size_t i=0 ; i<N() ; i++ )
//       for( size_t j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;

//     //Check for convergence by maximal derivative
//     O_->derivs(y_,dydt_);//Calculate the derivatives (again?)
//     double maxDer = maxDerivative();
//     if( maxDer<cEps ) {//Converged at this update
//       if( !cFlag ) {//new convergence time
// 	cT=t_;
// 	cFlag=1;
// 	cFirstMaxDer=maxDer;
//       }
//       std::cerr << "Converged at " << t_ << " (" << maxDer << ")\n";
//       //Converged longer time than cTdiff and decreasing maxDer
//       if( t_>cT+cTdiff && maxDer<=cFirstMaxDer)
// 	return;
//     }
//     else
//       cFlag=0;//reset convergence flag
    
//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);

    
//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );
    
//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
//   std::cerr << "Too many steps in rk5Adaptive::simulateUntilConvergence\n"; 
//   exit(-1);
// }

// //!Runs a simulation of an organism model with temporary adding of cells
// void Rk5Adaptive::simulateGrowthTmp() {

//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( size_t i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }
  
//   // Go
//   //////////////////////////////////////////////////////////////////////
//   double maxZ=2.0;//For growing cylinder
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Find the neighbors for all compartments
//     //Only first time (for static simulations)
//     //if( nstp==0 ) O_->findNeighbors(y_);
//     //Every step (for growing simulations)
//     O_->findNeighbors(y_);
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( size_t i=0 ; i<N() ; i++ )
//       for( size_t j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     int addFlag=0;
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//       //Also use print times to define cell addition times
//       addFlag=1;
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;
    
//     //Set boundary values for auxin
//     y_[0][6] = 0.0;
//     y_[y_.size()-1][6] = 0.0;
    
//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);

    
//     //Also use print times to define cell addition times
//     if( addFlag ) {
//       int Nold=O_->numCompartment();
//       double zMove=0.0;
//       std::vector<int> addedIndex; 
//       std::vector<int> fromIndex; 
//       for( int i=0 ; i<Nold ; i++ ) {	
// 	if( y_[i][2]+y_[i][3]>=maxZ ) {
// 	  int NN=O_->numCompartment();
// 	  int N=NN+1;
// 	  int M=O_->compartment(i).numVariable();
// 	  O_->addCompartment( O_->compartment(i) );
// 	  O_->compartment(NN).setIndex(NN);
// 	  y_.resize( N );
// 	  dydt_.resize( N );
// 	  y_[NN].resize( M );
// 	  dydt_[NN].resize( M );
// 	  for( int j=0 ; j<M ; j++ ) {
// 	    y_[NN][j] = y_[i][j];
// 	    dydt_[NN][j] = dydt_[i][j];
// 	  } 
// 	  //y_[NN][7] = 1.0;
// 	  zMove = 2*y_[i][3];
// 	  y_[i][2] += zMove;
// 	  fromIndex.push_back(i);
// 	  addedIndex.push_back(NN);
// 	}
//       }
//       //Add neighbors from and to new compartments
//       int newN=addedIndex.size();
//       for( int i=0 ; i<newN ; i++ ) {
// 	O_->compartment( fromIndex[i] ).addNeighbor( addedIndex[i] );
// 	O_->compartment( addedIndex[i] ).addNeighbor( fromIndex[i] );
// 	O_->compartment(addedIndex[i]).addNeighbor( addedIndex[(i+1)%newN] );
// 	O_->compartment(addedIndex[(i+1)%newN]).addNeighbor( addedIndex[i] );
//       }
//       maxZ+=zMove;
//     }
    
//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );

//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//   }
//   std::cerr << "Too many steps in rk5Adaptive::simulate\n"; exit(-1);
// }

// //!Runs a simulation of an organism model where cells outside R are removed
// void Rk5Adaptive::simulateGrowthOnSphere(double R) {
  
//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( int i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }
  
//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Find the neighbors for all compartments
//     //Every step (for growing simulations)
//     O_->findNeighbors(y_);
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( int i=0 ; i<N() ; i++ )
//       for( int j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;
    
//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);
    
//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );

//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
// }

// //!Runs a simulation of an organism model where cells below zMin are removed
// void Rk5Adaptive::simulateGrowthOnSphereSurface(double zMin) {
  
//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( int i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }
  
//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Find the neighbors for all compartments
//     //Every step (for growing simulations)
//     O_->findNeighbors(y_);
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( int i=0 ; i<N() ; i++ )
//       for( int j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;

//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);
    
//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );

//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
// }

// //!Runs a simulation of an organism model where cells below zMin are removed
// void Rk5Adaptive::simulateGrowthOnSphereCylinderSurface(double zMin) {
  
//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( int i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }
  
//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Find the neighbors for all compartments
//     //Every step (for growing simulations)
//     O_->findNeighbors(y_);

//     //Update the derivatives
//     O_->derivs(y_,dydt_);
//     for( int i=0 ; i<N() ; i++ )
//       for( int j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;

//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);

//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );

//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
// }

// //!Runs a simulation of an organism model where cells below zMin are removed
// void Rk5Adaptive::
// simulateGrowthOnSphereCylinderSurface(double RMin, double RMax,
// 				      double zMinFac) {
  
//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( int i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }
  
//   //Check that sphere-cylinder radius-parameter is at the correct position
//   if( O_->topology().numReaction() < 1 ||
//       O_->topology().reaction(0)->id() != "springAsymmetricSphereCylinder" ) {
//     std::cerr << "Rk5Adaptive::simulateGrowthOnSphereCylinderSurface "
// 	      << "Need springAsymmetricSphereCylinder as first topology "
// 	      << "reaction! Not " << O_->topology().reaction(0)->id() << "\n";
//     exit(-1);
//   }
  
//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Define and set growing radius
//     double R = RMin + (t_-startTime_) * (RMax-RMin) / (endTime_-startTime_);
//     O_->topology().reaction(0)->setParameter(3,R);
    
//     //Find the neighbors for all compartments
//     //Every step (for growing simulations)
//     O_->findNeighbors(y_,0.9);
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( int i=0 ; i<N() ; i++ )
//       for( int j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;
    
//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);

//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );

//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
// }

//!Runs a simulation of an organism model with Mitchison parameters/update
// void Rk5Adaptive::simulateMitchison() {

//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }
  
//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( int i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }
  
//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }

//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Find the neighbors for all compartments
//     //Only first time (for static simulations)
//     if( nstp==0 ) O_->findNeighbors(y_);
//     //Every step (for growing simulations)
//     //O_->findNeighbors(y_);
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( int i=0 ; i<N() ; i++ )
//       for( int j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;
    
//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);
    
//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );
    
//     // Update compartment's wall parameters (right now only for mitchison)
//     //////////////////////////////////////////////////////////////////////
//     if( O_->numCompartment() && O_->compartment(0).numParameter() ) {
//       for( int i=0 ; i<O_->numCompartment() ; i++ )
// 	for( int k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
// 	  int j = O_->compartment(i).neighbor(k);
// 	  if( i<j ) { //Only for "lower indexed" neighbors here
// 	    O_->compartment(i).updateParametersMitchison(
// 							 y_,O_->compartment(j)
// 							 ,h);
// 	  }
// 	}
//     }
//     //////////////////////////////////////////////////////////////////////

//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
//   std::cerr << "Too many steps in rk5Adaptive::simulate\n"; exit(-1);
// }

// //!Runs a simulation of an organism model with Mitchison parameters/update
// void Rk5Adaptive::simulateMitchisonStaticNeighborhood() {
  
  
//   //double tiny=1e-30;//Using NR definition
//   double tiny=1e-9*eps_;//Caveat! Using new definition
//   double  h,hNext,hDid;
  
//   //Check that h1 and endTime-startTime are > 0
//   //////////////////////////////////////////////////////////////////////
//   if( h1_>0. && (endTime_-startTime_)>0. )
//     h=h1_;
//   else {//either h or (endTime-startTime) <=0
//     std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
// 	      << "simulation. No simulation performed.\n";
//     return;
//   }

//   //Create all vectors that will be needed here and by rkqs and rkck!
//   //////////////////////////////////////////////////////////////////////
//   //Used here
//   std::vector< std::vector<double> > yScal( N() );
//   //Used by rkqs
//   std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
//   //used by rkck
//   std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
//     ak5( N() ),ak6( N() ),yTempRkck( N() );
//   //Resize each vector
//   for( int i=0 ; i<N() ; i++ ) {
//     yScal[i].resize(M());
//     yErr[i].resize( M() );
//     yTemp[i].resize( M() );
//     ak2[i].resize(M());
//     ak3[i].resize(M());
//     ak4[i].resize(M());
//     ak5[i].resize(M());
//     ak6[i].resize(M());
//     yTempRkck[i].resize(M());
//   }

//   // Initiate print times
//   //////////////////////////////////////////////////////////////////////
//   double printTime=endTime_+tiny;
//   double printDeltaTime=endTime_+2.*tiny;
//   if( numPrint_<=0 )//No printing
//     printFlag_=0;
//   else if( numPrint_==1 ) {//Print last point (default)
//   }
//   else if( numPrint_==2 ) {//Print first/last point
//     printTime=startTime_-tiny;
//   } 
//   else {//Print first/last points and spread the rest uniformly
//     printTime=startTime_-tiny;
//     printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
//   }

//   // Go
//   //////////////////////////////////////////////////////////////////////
//   t_=startTime_;
//   numOk_ = numBad_ = 0;
//   for( int nstp=0 ; ; nstp++ ) {
    
//     //Update the derivatives
//     O_->derivs(y_,dydt_);
    
//     for( int i=0 ; i<N() ; i++ )
//       for( int j=0 ; j<M() ; j++ )
//         yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
    
//     //Print if applicable 
//     if( printFlag_ && t_ >= printTime ) {
//       printTime += printDeltaTime;
//       print();
//     }
    
//     //Check if step is larger than max allowed
//     //max step end is min of endTime_ and printTime
//     double tMin= endTime_<printTime ? endTime_ : printTime;
//     if( t_+h>tMin ) h=tMin-t_;
    
//     //Update
//     rkqs(h,hDid,hNext,yScal,
// 	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
//     if (hDid == h) ++numOk_; else ++numBad_;

//     // Check for cell divisions, compartmental removal etc
//     O_->compartmentChange(y_,dydt_,t_);

//     //Rescale all temporary vectors as well
//     yScal.resize( N(), yScal[0] );
//     yTemp.resize( N(), yTemp[0] );
//     yErr.resize( N(), yErr[0] );
//     ak2.resize( N(), ak2[0] );
//     ak3.resize( N(), ak3[0] );
//     ak4.resize( N(), ak4[0] );
//     ak5.resize( N(), ak5[0] );
//     ak6.resize( N(), ak6[0] );
//     yTempRkck.resize( N(), yTempRkck[0] );
    
    
//     // Update compartment parameters (if used ) (right now only for mitchison)
//     //////////////////////////////////////////////////////////////////////
//     if( O_->numCompartment() && O_->compartment(0).numParameter() ) {
//       for( int i=0 ; i<O_->numCompartment() ; i++ )
// 	for( int k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
// 	  int j = O_->compartment(i).neighbor(k);
// 	  if( i<j ) { //Only for "lower indexed" neighbors here
// 	    O_->compartment(i).updateParametersMitchison(
// 							 y_,O_->compartment(j)
// 							 ,h);
// 	  }
// 	}
//     }
//     //////////////////////////////////////////////////////////////////////
    
//     //If the end t is passed return (print if applicable)
//     if ( t_>=endTime_ ) {
//       if( printFlag_ ) print();
//       std::cerr << "Simulation done.\n"; 
//       return;
//     }
//     //Warn for small step sizes...
//     //  if (fabs(hNext) <= hMin) {
//     //        std::cerr << "Warning: Step size small (" << hNext
//     //  		<< ") in rk5Adaptive::simulate at time " 
//     //  		<< t << "\n"; /*exit(-1);*/ }
//     h=hNext;
//     //Do not take larger steps than h1
//     if( h>h1_ )
//       h=h1_;
//   }
//   std::cerr << "Too many steps in rk5Adaptive::simulate\n"; exit(-1);
// }

//!Fifth order Runge-Kutta adaptive stepper.
#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4
void Rk5Adaptive::rkqs(double hTry,double &hDid,double &hNext,
		       std::vector< std::vector<double> > &yScal,
		       std::vector< std::vector<double> > &yTemp,
		       std::vector< std::vector<double> > &yErr,
		       std::vector< std::vector<double> > &ak2,
		       std::vector< std::vector<double> > &ak3,
		       std::vector< std::vector<double> > &ak4,
		       std::vector< std::vector<double> > &ak5,
		       std::vector< std::vector<double> > &ak6,
		       std::vector< std::vector<double> > &yTempRkck ) {
  
  double errMax,h,hTemp,tNew,aux;
  h=hTry;
  for (;;) {
    rkck(h,yTemp,yErr,
	 ak2,ak3,ak4,ak5,ak6,yTempRkck);
    errMax=0.;
    for(size_t i=0 ; i<N() ; i++ )
      for( size_t j=0 ; j<M() ; j++ ) {
        aux = fabs(yErr[i][j]/yScal[i][j]);
        if( aux>errMax )
          errMax = aux;
      }
    errMax /= eps_;
    if (errMax <= 1.0 ) break;
    hTemp=SAFETY*h*pow(errMax,PSHRNK);
    if( h >= 0. )
      h = hTemp > 0.1*h ? hTemp : 0.1*h;
    else
      h = hTemp > 0.1*h ? 0.1*h : hTemp;
    tNew=t_+h;
    if( tNew==t_ ) { 
      std::cerr << "Warning stepsize underflow in Rk5Adaptive::rkqs\n"; 
      exit(-1); }
  }
  if (errMax > ERRCON) hNext=SAFETY*h*pow(errMax,PGROW);
  else hNext=5.0*h;
  t_ += (hDid=h);
  
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      y_[i][j]=yTemp[i][j];
}
#undef SAFETY
#undef PGROW
#undef PSHRNK
#undef ERRCON

//!Fifth order Runge-Kutta adaptive stepper for mechanical update.
#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4
void Rk5Adaptive::
rkqsMechanical(double &t,double hTry,double &hDid,double &hNext,
	       std::vector< std::vector<double> > &yScal,
	       std::vector< std::vector<double> > &yTemp,
	       std::vector< std::vector<double> > &yErr,
	       std::vector< std::vector<double> > &ak2,
	       std::vector< std::vector<double> > &ak3,
	       std::vector< std::vector<double> > &ak4,
	       std::vector< std::vector<double> > &ak5,
	       std::vector< std::vector<double> > &ak6,
	       std::vector< std::vector<double> > &yTempRkck,
	       std::vector<size_t> &mechEq,std::vector<size_t> &posVar) {
  
  double errMax,h,hTemp,tNew,aux;
  h=hTry;
  for (;;) {
    rkckMechanical(h,yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck,
		   mechEq,posVar);
    errMax=0.;
    for(size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<posVar.size() ; k++ ) {
	size_t j=posVar[k];
        aux = fabs(yErr[i][j]/yScal[i][j]);
        if( aux>errMax )
          errMax = aux;
      }
    errMax /= eps_;
    if (errMax <= 1.0 ) break;
    hTemp=SAFETY*h*pow(errMax,PSHRNK);
    if( h >= 0. )
      h = hTemp > 0.1*h ? hTemp : 0.1*h;
    else
      h = hTemp > 0.1*h ? 0.1*h : hTemp;
    tNew=t+h;
    if( tNew==t ) { 
      std::cerr << "Warning stepsize underflow in Rk5Adaptive::rkqs\n"; 
      exit(-1); }
  }
  if (errMax > ERRCON) hNext=SAFETY*h*pow(errMax,PGROW);
  else hNext=5.0*h;
  t += (hDid=h);
  
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      y_[i][j]=yTemp[i][j];
    }
}
#undef SAFETY
#undef PGROW
#undef PSHRNK
#undef ERRCON

//! One fifth order Runge-Kutta step
/*! A fifth order Runge-Kutta step. It uses y,dydt as initial
  conditions at time t and takes a step of size h, returning the new
  function values in yOut. yErr is the estimated error in each
  variable (calculated as the difference of a fourth and fifth order
  step). The function and all of the parameters are taken from
  Numerical Recipes.
*/
void Rk5Adaptive::rkck(double h,
		       std::vector< std::vector<double> > &yOut,
		       std::vector< std::vector<double> > &yErr,
		       std::vector< std::vector<double> > &ak2,
		       std::vector< std::vector<double> > &ak3,
		       std::vector< std::vector<double> > &ak4,
		       std::vector< std::vector<double> > &ak5,
		       std::vector< std::vector<double> > &ak6,
		       std::vector< std::vector<double> > &yTempRkck ) {
  
  //static double a2=0.2,a3=0.3,a4=0.6,a5=1.0,a6=0.875;
  static double b21=0.2,
    b31=3.0/40.0,b32=9.0/40.0,b41=0.3,b42 = -0.9,b43=1.2,
    b51 = -11.0/54.0, b52=2.5,b53 = -70.0/27.0,b54=35.0/27.0,
    b61=1631.0/55296.0,b62=175.0/512.0,b63=575.0/13824.0,
    b64=44275.0/110592.0,b65=253.0/4096.0,c1=37.0/378.0,
    c3=250.0/621.0,c4=125.0/594.0,c6=512.0/1771.0,
    dc5 = -277.00/14336.0;
  double dc1=c1-2825.0/27648.0,dc3=c3-18575.0/48384.0,
    dc4=c4-13525.0/55296.0,dc6=c6-0.25;
  
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+b21*h*dydt_[i][j];
  
  O_->derivs(yTempRkck,ak2);//t+a2h
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*(b31*dydt_[i][j]+b32*ak2[i][j]);
  
  O_->derivs(yTempRkck,ak3);//t+a3h
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*(b41*dydt_[i][j]+b42*ak2[i][j]+b43*ak3[i][j]);
  
  O_->derivs(yTempRkck,ak4);//t+a4*h
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*
        (b51*dydt_[i][j]+b52*ak2[i][j]+b53*ak3[i][j]+b54*ak4[i][j]);
  
  O_->derivs(yTempRkck,ak5);//t+a5*h
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*
        (b61*dydt_[i][j]+b62*ak2[i][j]+b63*ak3[i][j]+b64*ak4[i][j]+
         b65*ak5[i][j]);
  
  O_->derivs(yTempRkck,ak6);//t+a6*h
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      yOut[i][j]=y_[i][j]+h*
        (c1*dydt_[i][j]+c3*ak3[i][j]+c4*ak4[i][j]+c6*ak6[i][j]);
  
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t j=0 ; j<M() ; j++ )
      yErr[i][j]=h*
        (dc1*dydt_[i][j]+dc3*ak3[i][j]+dc4*ak4[i][j]+
         dc5*ak5[i][j]+dc6*ak6[i][j]);
}

//! One fifth order Runge-Kutta step for positional variables
/*! A fifth order Runge-Kutta step. Equivalent to the normal rkck, but
  only updates according to the equations provided in mechEq and
  updates only variables provided in posVar. Caveat: The check that
  mechEq only update positional variables are left to the user!
*/
void Rk5Adaptive::
rkckMechanical(double h,
	       std::vector< std::vector<double> > &yOut,
	       std::vector< std::vector<double> > &yErr,
	       std::vector< std::vector<double> > &ak2,
	       std::vector< std::vector<double> > &ak3,
	       std::vector< std::vector<double> > &ak4,
	       std::vector< std::vector<double> > &ak5,
	       std::vector< std::vector<double> > &ak6,
	       std::vector< std::vector<double> > &yTempRkck,
	       std::vector<size_t> &mechEq,std::vector<size_t> &posVar ) {
  
  //static double a2=0.2,a3=0.3,a4=0.6,a5=1.0,a6=0.875;
  static double b21=0.2,
    b31=3.0/40.0,b32=9.0/40.0,b41=0.3,b42 = -0.9,b43=1.2,
    b51 = -11.0/54.0, b52=2.5,b53 = -70.0/27.0,b54=35.0/27.0,
    b61=1631.0/55296.0,b62=175.0/512.0,b63=575.0/13824.0,
    b64=44275.0/110592.0,b65=253.0/4096.0,c1=37.0/378.0,
    c3=250.0/621.0,c4=125.0/594.0,c6=512.0/1771.0,
    dc5 = -277.00/14336.0;
  double dc1=c1-2825.0/27648.0,dc3=c3-18575.0/48384.0,
    dc4=c4-13525.0/55296.0,dc6=c6-0.25;
  
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+b21*h*dydt_[i][j];
    }
  O_->derivsMechanical(yTempRkck,ak2,mechEq,posVar);//t+a2h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*(b31*dydt_[i][j]+b32*ak2[i][j]);
    }
  O_->derivsMechanical(yTempRkck,ak3,mechEq,posVar);//t+a3h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*(b41*dydt_[i][j]+b42*ak2[i][j]+b43*ak3[i][j]);
    }
  O_->derivsMechanical(yTempRkck,ak4,mechEq,posVar);//t+a4*h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*
        (b51*dydt_[i][j]+b52*ak2[i][j]+b53*ak3[i][j]+b54*ak4[i][j]);
    }
  O_->derivsMechanical(yTempRkck,ak5,mechEq,posVar);//t+a5*h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*
        (b61*dydt_[i][j]+b62*ak2[i][j]+b63*ak3[i][j]+b64*ak4[i][j]+
         b65*ak5[i][j]);
    }  
  O_->derivsMechanical(yTempRkck,ak6,mechEq,posVar);//t+a6*h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yOut[i][j]=y_[i][j]+h*
        (c1*dydt_[i][j]+c3*ak3[i][j]+c4*ak4[i][j]+c6*ak6[i][j]);
    }
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yErr[i][j]=h*
        (dc1*dydt_[i][j]+dc3*ak3[i][j]+dc4*ak4[i][j]+
         dc5*ak5[i][j]+dc6*ak6[i][j]);
    }
}

//!General printing function
/*!Caveat: Not yet general, but will be...*/
void Rk5Adaptive::print(std::ostream &os) const {
  
  static int tCount=0;
  static int NOld=0,okOld=0,badOld=0;
  static double tOld=0.0;
//  int numOkSave = numOk_;
  std::cerr << tCount << " " << t_ << " " << N() << " " << M() << " "
	    << numOk_ << " " << numBad_ << "  " << t_-tOld << " " << N()-NOld 
	    << " " << numOk_-okOld << " " << numBad_-badOld << "\n"; 
  tOld = t_;
  NOld = N();
  okOld = numOk_;
  badOld = numBad_;

  if( printFlag_==1 ) {//Matlab style
    for( size_t i=0 ; i<N() ; i++ ) {
      //os << tCount << " " << i << " "; 
      //os << O_->compartment(i).numNeighbor() << " ";
      os << tCount << " " << t_ << " "; 
      for( size_t j=0 ; j<M() ; j++ )
	os << y_[i][j] << " ";
      os << "\n";
    }
    os << "\n";
  }
  else if( printFlag_==2 ) {//Java3D style
    os << N() << " " << M() << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	if( j==2 && O_->numDimension()==2 )//Not very nice...:) 
	  os << "0 ";
	os << y_[i][j] << " ";
      }
      os << "\n";
    }
    os << "\n";
  }
  else if( printFlag_==3 ) {//template style
    static size_t numDivOld=0;
    if( tCount ) {
      os << O_->numDivision()-numDivOld << "\n";;
      for( size_t i=numDivOld ; i<O_->numDivision() ; i++ )
	os << O_->divisionIndex(i) << " " << O_->divisionTime(i) << "\n";
      os << "\n";
      numDivOld = O_->numDivision();
    }    
    os << t_ << " " << N() << " " << M() << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << "\n";
    }
    os << "\n";
  }
  else if( printFlag_==4 ) {//init style
    os << N() << " " << M() << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << "\n";
    }
    os << "\n";
  }
  else if( printFlag_==5 ) {//polarization
    size_t signalCol=6;
    if( y_[0].size()<=signalCol ) {
      std::cerr << "rk5Adaptive::print() No polarization calculated.\n";
      return;
    }
    for( size_t i=0 ; i<N() ; i++ ) {
      os << y_[i][0] << " " << y_[i][1] << " ";
      double polX=0.0,polY=0.0,pol=0.0;
      for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	size_t j=O_->compartment(i).neighbor(k);
	double dX,dY,polTmp,norm;
	dX = y_[j][0]-y_[i][0];
	dY = y_[j][1]-y_[i][1];
	norm = sqrt( dX*dX+dY*dY );
	dX /= norm;
	dY /= norm;
	
	polTmp = y_[j][signalCol] - y_[i][signalCol];  
	pol += polTmp;
	polX += polTmp*dX;
	polY += polTmp*dY;
      }
      os << polX << " " << polY << " " << pol << " " << pol << "\n";
    }
  }
  else if( printFlag_==6 ) {//gnuplot style
    double z=0;
    size_t firstZindex=0;
    for( size_t i=0 ; i<N() ; i++ ) {
      if( y_[i][2]!=z ) {
	//Print first z point again to connect in gnuplot...
	os << tCount << " " << firstZindex << " ";
	for( size_t j=0 ; j<M() ; j++ ) {
	  os << y_[firstZindex][j] << " ";
	}
	os << "\n\n";
	//set new z and remember first index
	z=y_[i][2];
	firstZindex=i;
      }
      os << tCount << " " << i << " ";
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << "\n";
    }
    //Print first z point again to connect in gnuplot...
    os << tCount << " " << firstZindex << " ";
    for( size_t j=0 ; j<M() ; j++ ) {
      os << y_[firstZindex][j] << " ";
    }    
    os << "\n";
  }
  else if( printFlag_==7 ) {//For gnuplot to be able to follow ind cells
    for( size_t i=0 ; i<N() ; i++ ) {
      os << tCount << " " << i << " ";
      for( size_t j=0 ; j<M() ; j++ )
	os << y_[i][j] << " ";      
      os << O_->compartment(i).numNeighbor() << "\n";	  
    }
    os << "\n";
  }
  else if( printFlag_==8 ) {//Mitchison Flux
    size_t signalCol=3;
    if( y_[0].size()<=signalCol ) {
      std::cerr << "rk5Adaptive::print() No polarization calculated.\n";
      return;
    }
    for( size_t i=0 ; i<N() ; i++ ) {
      double polX=0.0,polY=0.0,pol=0.0;
      for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	size_t j=O_->compartment(i).neighbor(k);
	double dX,dY,polTmp,norm;
	dX = y_[j][0]-y_[i][0];
	dY = y_[j][1]-y_[i][1];
	norm = sqrt( dX*dX+dY*dY );
	dX /= norm;
	dY /= norm;
	
	polTmp = y_[i][signalCol]*O_->compartment(i).parameter(k);  
	pol = polTmp;
	polX = polTmp*dX;
	polY = polTmp*dY;
	os << y_[i][0] << " " << y_[i][1] << " ";
	os << polX << " " << polY << " " << pol << " " << pol << "\n";
      }
    }
  }
  else if( printFlag_==9 ) {//Mitchison p parameters
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ )
	os <<i<<" "<< k << " "<<O_->compartment(i).parameter(k) << "\n";
    os << "\n";
  }
  else if( printFlag_==10 ) {//For most home-made plotting
    int overlapFlag=0;
    if( tCount==0 )
      os << numPrint_ << "\n";
    os << N() << " " << M()+1+overlapFlag << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor();
      if( overlapFlag ) {
	//Print summed overlap larger than k_fac
	double k_fac = 0.75, overlap = 0.0;
	for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	  size_t ii = O_->compartment(i).neighbor(k);
	  double r1 = y_[i][O_->compartment(i).numDimension()]; 
	  double r2 = y_[ii][O_->compartment(i).numDimension()]; 
	  double dist=0.0;
	  for( size_t d=0 ; d<O_->compartment(i).numDimension() ; d++ )
	    dist += (y_[i][d]-y_[ii][d])*(y_[i][d]-y_[ii][d]);
	  dist = std::sqrt( dist);
	  if( dist < k_fac*(r1+r2) )
	    overlap += k_fac*(r1+r2)-dist;
	}
	os << " " << overlap/O_->compartment(i).numNeighbor();
      }
      os << "\n";
    }
    os << "\n";
  }
  else if( printFlag_==11 ) {//For plotPostscript with Mitchison arrows
    if( tCount==0 )
      os << numPrint_ << "\n";
    
    os << N() << " " << M()+1 << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor();
      os << "\n";
    }
    os << "\n";
    //Printing the arrows in a specific file
    static std::ofstream OF;
    if( tCount==0 ) {
      OF.open("/tmp/arrows.data");
      if( !OF ) {
	std::cerr << "rk5Adaptive::print() Cannot open arrow file\n";
	exit(-1);
      }
      OF << numPrint_ << "\n";
    }
    size_t signalCol=3;
    if( y_[0].size()<=signalCol ) {
      std::cerr << "rk5Adaptive::print() No polarization calculated.\n";
      return;
    }
    unsigned int totalNumNeighbor=0;
    for( size_t i=0 ; i<N() ; i++ )
      totalNumNeighbor += O_->compartment(i).numNeighbor();
    OF << totalNumNeighbor << " 4\n";
    
    for( size_t i=0 ; i<N() ; i++ ) {
      double polX=0.0,polY=0.0,pol=0.0;
      for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	size_t j=O_->compartment(i).neighbor(k);
	double dX,dY,polTmp,norm;
	dX = y_[j][0]-y_[i][0];
	dY = y_[j][1]-y_[i][1];
	norm = sqrt( dX*dX+dY*dY );
	dX /= norm;
	dY /= norm;
	
	//polTmp = y_[i][signalCol]*O_->compartment(i).parameter(k);  
	polTmp = O_->compartment(i).parameter(k);  
	pol = polTmp;
	polX = polTmp*dX;
	polY = polTmp*dY;
	OF << y_[i][0] << " " << y_[i][1] << " ";
	OF << polX << " " << polY << "\n";
      }
    }
  }
  else if( printFlag_==12 ) {//For plotPostscript with polarization arrows
    if( tCount==0 )
      os << numPrint_ << "\n";
    
    os << N() << " " << M()+1 << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor();
      os << "\n";
    }
    os << "\n";
    //Printing the arrows in a specific file
    static std::ofstream OF;
    if( tCount==0 ) {
      OF.open("/tmp/arrows.data");
      if( !OF ) {
	std::cerr << "rk5Adaptive::print() Cannot open arrow file\n";
	exit(-1);
      }
      OF << numPrint_ << "\n";
    }
    size_t signalCol=5;
    size_t cellMarkerCol=7;
    if( y_[0].size()<=signalCol ) {
      std::cerr << "rk5Adaptive::print() No polarization calculated.\n";
      return;
    }
    unsigned int totalNumNeighbor=0;
    for( size_t i=0 ; i<N() ; i++ )
      if( y_[i][cellMarkerCol]>0.5 )
	totalNumNeighbor += O_->compartment(i).numNeighbor();
    OF << totalNumNeighbor << " 4\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      if( y_[i][cellMarkerCol]>0.5 ) {
	double polX=0.0,polY=0.0,pol=0.0;
	double sum=0.0;
	for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	  size_t j=O_->compartment(i).neighbor(k);
	  sum += y_[j][signalCol];
	}
	for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	  size_t j=O_->compartment(i).neighbor(k);
	  double dX,dY,polTmp,norm;
	  dX = y_[j][0]-y_[i][0];
	  dY = y_[j][1]-y_[i][1];
	  norm = sqrt( dX*dX+dY*dY );
	  dX /= norm;
	  dY /= norm;
	  
	  //polTmp = y_[j][signalCol] - y_[i][signalCol];  ?
	  //polTmp = y_[j][signalCol]/sum;  
	  //polTmp = y_[j][signalCol];  
	  polTmp = y_[j][signalCol]*y_[j][signalCol+1];  
	  pol = polTmp;
	  polX = polTmp*dX;
	  polY = polTmp*dY;
	  OF << y_[i][0] << " " << y_[i][1] << " ";
	  OF << polX << " " << polY << "\n";
	}
      }
    }
  }
  else if( printFlag_==13 ) {//For plotPostscript with polarization arrows
    size_t signalCol=6;
    size_t polarizationMolCol=signalCol-1;
    size_t cellMarkerCol=signalCol+1;
    
    if( tCount==0 )
      os << numPrint_ << "\n";
    
    os << N() << " " << M()+1 << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor();
      os << "\n";
    }
    os << "\n";
    //Printing the arrows in a specific file
    static std::ofstream OF;
    if( tCount==0 ) {
      OF.open("/tmp/arrows.data");
      if( !OF ) {
	std::cerr << "rk5Adaptive::print() Cannot open arrow file\n";
	exit(-1);
      }
      OF << numPrint_ << "\n";
    }

    if( y_[0].size()<=signalCol ) {
      std::cerr << "rk5Adaptive::print() No polarization calculated.\n";
      return;
    }
    unsigned int totalNumNeighbor=0;
    for( size_t i=0 ; i<N() ; i++ )
      if( y_[i][cellMarkerCol]>0.5 )
	totalNumNeighbor += O_->compartment(i).numNeighbor();
    OF << totalNumNeighbor << " 4\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      double polX=0.0,polY=0.0,pol=0.0;
      double sum=0.0;
      for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	size_t j=O_->compartment(i).neighbor(k);
	sum += y_[j][signalCol];
      }
      //sum /= O_->compartment(i).numNeighbor();
      //sum += 0.3;
      for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	size_t j=O_->compartment(i).neighbor(k);
	double dX,dY,polTmp=0.0,norm;
	dX = y_[j][0]-y_[i][0];
	dY = y_[j][1]-y_[i][1];
	norm = sqrt( dX*dX+dY*dY );
	dX /= norm;
	dY /= norm;
	
	//polTmp = y_[j][signalCol] - y_[i][signalCol];  ?
	//polTmp = y_[j][signalCol]/sum;  
	//polTmp = y_[j][signalCol];  
	if( sum >= 0.0 )
	  polTmp = y_[j][signalCol]*y_[i][polarizationMolCol]/sum;  
	pol = polTmp;
	polX = polTmp*dX;
	polY = polTmp*dY;
	OF << y_[i][0] << " " << y_[i][1] << " ";
	OF << polX << " " << polY << "\n";
      }
    }
  }
  else if( printFlag_==14 ) {//For plotPostscript disregarding z...
    std::vector<size_t> flag,max,peakMarker(y_.size());
    std::vector<double> theta(y_.size()),r(y_.size()),phi(y_.size());
    size_t auxinCol=7;
    O_->findPeaksGradientAscent(y_,auxinCol,max,flag);
    for( size_t i=0 ; i<max.size() ; i++ ) {
      size_t c=max[i];
      flag[c] += max.size();
      peakMarker[c] = 1;
//        std::cerr << t_ << " " << c << " " << flag[c] << " " 
// 		<< O_->compartment(c).numNeighbor() << " b1  ";
//       std::cerr << y_[c][0] << " " << y_[c][1] << " " << y_[c][2] << "\n";
      r[c] = std::sqrt( y_[c][0]*y_[c][0]+y_[c][1]*y_[c][1] );  
      theta[c] = std::acos( y_[c][2] /
			    std::sqrt( y_[c][0]*y_[c][0]+y_[c][1]*y_[c][1]+
				       y_[c][2]*y_[c][2] ) );
      phi[c] = std::atan( std::fabs(y_[c][1]/y_[c][0]) );
      double pi=3.141592654;
      if( y_[c][0]<0 && y_[c][1]>0 )
	phi[c] = pi-phi[c];
      else if( y_[c][0]<0 && y_[c][1]<0 )
	phi[c] = pi+phi[c];
      else if( y_[c][0]>0 && y_[c][1]<0 )
	phi[c] = 2*pi-phi[c];
    }
    if( tCount==0 )
      os << numPrint_ << "\n";
    os << N() << " " << M()+7 << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	//if( j != O_->numDimension()-1 ) 
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor() << " ";
      os << flag[i] << " " << peakMarker[i] << " " << r[i] << " " << theta[i]
	 << " " << phi[i] << " " << t_;
      os << "\n";
    }
    os << "\n";
  }
  else if( printFlag_==15 ) {//For gnuplot with peaks
    std::vector<size_t> flag,max,peakMarker(y_.size());
    size_t auxinCol=6;
    O_->findPeaksGradientAscent(y_,auxinCol,max,flag);
    for( size_t i=0 ; i<max.size() ; i++ ) {
      size_t c=max[i];
      flag[c] += max.size();
      peakMarker[c] = 1;
    }
    //os << N() << " " << M()+7 << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor() << " ";
      os << flag[i] << " " << peakMarker[i] << " " << " " << t_ 
	 << " " << tCount << " " << i << "\n";
    }
    os << "\n";
  }
  else if( printFlag_==16 ) {//First version for ellipses in gnuplot
    for( size_t i=0 ; i<N() ; i++ ) {
      os << tCount << " " << i << " " << y_[i][0] << " " << y_[i][1] 
	 << " " << y_[i][4] << "\n";
      os << tCount << " " << i << " " << y_[i][2] << " " << y_[i][3] 
	 << " " << y_[i][4] << "\n\n";
    }
  }
  else if( printFlag_==17 ) {//For plotPostscript + wall plotting
    //Normal printing
    if( tCount==0 )
      os << numPrint_ << "\n";
    os << N() << " " << M()+1 << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor();
      os << "\n";
    }
    os << "\n";
    //Print for plotting walls in gnuplot
    static std::ofstream OF;
    if( tCount==0 ) {
      const char* wallFile="/tmp/wall.data";
      OF.open(wallFile);
      if( !OF ) {
 	std::cerr << "rk5Adaptive::print() Cannot open wall file "
 		  << wallFile << "\n";
 	exit(-1);
      }
      else {
 	std::cerr << "rk5Adaptive::print() Printing wall data in "
 		  << wallFile << ".\n";
      }
    }
    //Print walls
    if( O_->numTopology() ) {
      for( size_t rCount=0 ; rCount<O_->topology().numReaction() ; rCount++ )
	if( O_->topology().reaction(rCount)->id()
	    =="springElasticWallEllipse" )
	  O_->topology().reaction(rCount)->print(OF);
    }  
  }
  else if( printFlag_==18 ) {//For time checking
    //os << myTimes::getDiffTime() << " " << N()  << " " << numOkSave << std::endl; 
  }
  else if( printFlag_==19 ) {//For most home-made plotting
    static std::ofstream OF;
    if( tCount==0 ) {
      OF.open("time.data");
      if( !OF ) {
	std::cerr << "rk5Adaptive::print() Cannot open time file\n";
	exit(-1);
      }
    }
    //OF << myTimes::getDiffTime() << " " << N()  << " " << numOkSave << std::endl; 
    size_t overlapFlag=0;
    if( tCount==0 )
      os << numPrint_ << "\n";
    os << N() << " " << M()+1+overlapFlag << " " << t_ << "\n";
    for( size_t i=0 ; i<N() ; i++ ) {
      for( size_t j=0 ; j<M() ; j++ ) {
	os << y_[i][j] << " ";
      }
      os << O_->compartment(i).numNeighbor();
      if( overlapFlag ) {
	//Print summed overlap larger than k_fac
	double k_fac = 0.75, overlap = 0.0;
	for( size_t k=0 ; k<O_->compartment(i).numNeighbor() ; k++ ) {
	  size_t ii = O_->compartment(i).neighbor(k);
	  double r1 = y_[i][O_->compartment(i).numDimension()]; 
	  double r2 = y_[ii][O_->compartment(i).numDimension()]; 
	  double dist=0.0;
	  for( size_t d=0 ; d<O_->compartment(i).numDimension() ; d++ )
	    dist += (y_[i][d]-y_[ii][d])*(y_[i][d]-y_[ii][d]);
	  dist = std::sqrt( dist);
	  if( dist < k_fac*(r1+r2) )
	    overlap += k_fac*(r1+r2)-dist;
	}
	os << " " << overlap/O_->compartment(i).numNeighbor();
      }
      os << "\n";
    }
    os << "\n";
  }  
  else
    std::cerr << "Rk5Adaptive::print() Wrong printFlag value\n";
  tCount++;
}

void Rk5Adaptive::printParameter(std::ostream &os) const {
  
  os << "tStart tEnd: " << startTime_ << " " << endTime_ << "\n";
  os << "printFlag: " << printFlag_ << "\n";
  os << "printNum: " << numPrint_ << "\n";
  os << "h1 eps: " << h1_ << " " << eps_ << "\n";  
}

//!Finds the maximal |dydt|/|y| for the system
double Rk5Adaptive::maxDerivative() {
  
  double max=0.0,val;
  for( size_t i=0 ; i<N() ; i++ ) {
    for( size_t j=0 ; j<M() ; j++ ) {
      if( y_[i][j]!=0 )
	val = fabs( dydt_[i][j]/y_[i][j] );
      else
	val = fabs( dydt_[i][j] );
      
      if( val>max ) 
	max = val;
    }
  }
  return max;
}
