/**
 * Filename     : rk5Adaptive.h
 * Description  : A class for a forward simulation of a dynamical system
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: rk5Adaptive.h,v 1.8 2006/03/18 00:05:14 henrik Exp $
 */
#ifndef RK5ADAPTIVE_H
#define RK5ADAPTIVE_H

#include<vector>
#include<algorithm>
#include<cmath>
#include<iostream>
#include<fstream>
#include<string>
#include<cassert>
#include"organism.h"

//! Rk5Adaptive handles the simulation of a dynamical system with a RK-algo  
/*! The Rk5Adaptive class handles the dynamical simulation of an
  organism, using a 5th order Runge-Kutta algorithm with adaptive step
  size.*/
class Rk5Adaptive {
  
 private:
  
  std::vector< std::vector<double> > 
    y_,
    dydt_;
  
  Organism *O_;
  
  double t_;
  double startTime_;
  double endTime_;
  int numPrint_;
  int printFlag_;
  double eps_,h1_;
  unsigned int numOk_,numBad_;
  
 public:
  
  Rk5Adaptive();
  Rk5Adaptive( const Rk5Adaptive & rk5AdaptiveCopy );
  Rk5Adaptive( Organism *O, std::string paraFile="" );
  ~Rk5Adaptive();
  
  //Rk5Adaptive & operator=( const Rk5Adaptive & rk5AdaptiveCopy );
  
  // Get values
  inline size_t N() const;
  inline size_t M() const;
  inline std::vector< std::vector<double> > & y();
  inline std::vector< std::vector<double> > & dydt();
  inline std::vector<double> & y(size_t i);
  inline std::vector<double> & dydt(size_t i);
  inline double y(size_t i,size_t j) const;
  inline double dydt(size_t i,size_t j) const;
  
  // Set values
  inline void setY(size_t i,size_t j,double value);
  inline void setDydt(size_t i,size_t j,double value);
  
  //Read values from file
  void parameterFromFile( std::string file );
  void parameterDefault();
  
  //Print
  void print(std::ostream &os=std::cout) const;
  void printParameter(std::ostream &os=std::cout) const;
  
  //Functions for a 5th order Runge Kutta simulation with adaptive step size
  //Based on Numerical Recipes odeint function
  void simulate();
  void simulateWithConvergingMechanics(double cEps,double cTdiff);
  void updateMechanicsUntilConvergence(double cEps,double cTdiff,
				       std::vector< std::vector<double> > 
				       &yScal,
				       std::vector< std::vector<double> > 
				       &yTemp,
				       std::vector< std::vector<double> > 
				       &yErr,
				       std::vector< std::vector<double> > 
				       &ak2,
				       std::vector< std::vector<double> > 
				       &ak3,
				       std::vector< std::vector<double> > 
				       &ak4,
				       std::vector< std::vector<double> > 
				       &ak5,
				       std::vector< std::vector<double> > 
				       &ak6,
				       std::vector< std::vector<double> > 
				       &yTempRkck );

  //void simulateWithNeighborArea();
  //void simulateStaticNeighborhood();//Never updates the neighborhood
  //void simulateUntilConvergence(double cEps,double cTdiff);
  
  //void simulateGrowthTmp();//Used for some ad hoc manipulations
  //void simulateGrowthOnSphere(double R);//Removes cells outside a sphere
  //void simulateGrowthOnSphereSurface(double zMin=0.0);
  //void simulateGrowthOnSphereCylinderSurface(double zMin=0.0);
  //void simulateGrowthOnSphereCylinderSurface(double RMin,double RMax,
  //				     double zMinFac=0.5);
  //void simulateMitchison();//Used for the Mitchison model
  //void simulateMitchisonStaticNeighborhood();//Never updates the neighborhood
  void rkqs(double hTry,double &hDid,double &hNext,
	    std::vector< std::vector<double> > &yScal,
	    std::vector< std::vector<double> > &yTemp,
	    std::vector< std::vector<double> > &yErr,
	    std::vector< std::vector<double> > &ak2,
	    std::vector< std::vector<double> > &ak3,
	    std::vector< std::vector<double> > &ak4,
	    std::vector< std::vector<double> > &ak5,
	    std::vector< std::vector<double> > &ak6,
	    std::vector< std::vector<double> > &yTempRkck );
  void rkqsMechanical(double &t,double hTry,double &hDid,double &hNext,
		      std::vector< std::vector<double> > &yScal,
		      std::vector< std::vector<double> > &yTemp,
		      std::vector< std::vector<double> > &yErr,
		      std::vector< std::vector<double> > &ak2,
		      std::vector< std::vector<double> > &ak3,
		      std::vector< std::vector<double> > &ak4,
		      std::vector< std::vector<double> > &ak5,
		      std::vector< std::vector<double> > &ak6,
		      std::vector< std::vector<double> > &yTempRkck,
		      std::vector<size_t> &mechEq,
		      std::vector<size_t> &posVar);  
  void rkck(double h,
	    std::vector< std::vector<double> > &yOut,
	    std::vector< std::vector<double> > &yErr,
	    std::vector< std::vector<double> > &ak2,
	    std::vector< std::vector<double> > &ak3,
	    std::vector< std::vector<double> > &ak4,
	    std::vector< std::vector<double> > &ak5,
	    std::vector< std::vector<double> > &ak6,
	    std::vector< std::vector<double> > &yTempRkck );
  void rkckMechanical(double h,
		      std::vector< std::vector<double> > &yOut,
		      std::vector< std::vector<double> > &yErr,
		      std::vector< std::vector<double> > &ak2,
		      std::vector< std::vector<double> > &ak3,
		      std::vector< std::vector<double> > &ak4,
		      std::vector< std::vector<double> > &ak5,
		      std::vector< std::vector<double> > &ak6,
		      std::vector< std::vector<double> > &yTempRkck,
		      std::vector<size_t> &mechEq,
		      std::vector<size_t> &posVar );
  double maxDerivative();
};

//!Number of cells (rows in the variable matrix)
inline size_t Rk5Adaptive::N() const { return y_.size();}
//!Number of species plus four (cols in the variable matrix)
inline size_t Rk5Adaptive::M() const { return (N() ? y_[0].size() : 0);}
//!Variable values stored cell by cell
inline std::vector< std::vector<double> > & Rk5Adaptive::y() { return y_;}
//!Derivative values stored cell by cell
inline std::vector< std::vector<double> > & Rk5Adaptive::dydt() { 
  return dydt_;}
inline std::vector<double> & Rk5Adaptive::y(size_t i) { return y_[i];}
inline std::vector<double> & Rk5Adaptive::dydt(size_t i) { return dydt_[i];}
inline double Rk5Adaptive::y(size_t i,size_t j) const {return y_[i][j];}
inline double Rk5Adaptive::dydt(size_t i,size_t j) const { return dydt_[i][j];}

inline void Rk5Adaptive::setY(size_t i,size_t j,double value) { y_[i][j]=value;}
inline void Rk5Adaptive::setDydt(size_t i,size_t j,double value) { 
  dydt_[i][j]=value;}
#endif


