/**
 * Filename     : species.h
 * Description  : A class describing a species (typically a molecule)
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: species.h,v 1.4 2006/03/18 12:23:08 henrik Exp $
 */
#ifndef SPECIES_H
#define SPECIES_H

#include<vector>
#include<string>
#include<iostream>
#include<fstream>

#include"compartment.h"
#include"baseReaction.h"

//! A class for describing a species (typically a molecule)
/*! The Species class is describing properties of a
  molecule(species). An organism have one object for each molecule
  type. The update rules comes from the baseReactions included.
*/ 
class Species {
  
 private:
  
  std::string id_;
  size_t index_;
  std::vector<BaseReaction*> reaction_;
  
 public:
  
  Species();
  Species( const Species & speciesCopy );
  Species(std::vector<BaseReaction*> &reactionValue, 
	  std::string idValue, 
	  size_t indexValue=static_cast<size_t>(-1) );
  Species( char *inFile );
  Species( std::string inFile );
  Species( std::ifstream &IN );
  
  ~Species();
  
  //Species & operator=( const Species & speciesCopy );
  
  // Get values
  inline std::string id() const;
  inline size_t index() const;
  inline size_t numReaction() const;  
  
  inline BaseReaction* reaction(size_t i) const;
  inline const std::vector<BaseReaction*> reaction() const;
  
  // Set values
  inline void setId(std::string value);
  inline void setIndex(size_t value);
  inline void setReaction(size_t i,BaseReaction* value);
  inline void setReaction(const std::vector<BaseReaction*> value);
  
  //Other functions
  void readSpecies( std::ifstream &IN );
  int addReaction( std::ifstream &IN );
  void derivs(Compartment &compartment,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Returns the id string
inline std::string Species::id() const {
  return id_;
}

//!Returns the index of the species
inline size_t Species::index() const {
  return index_;
}

//!Returns the number of reactions
inline size_t Species::numReaction() const {
  return reaction_.size();
}

//!Returns a reaction (pointer)
inline BaseReaction* Species::reaction(size_t i) const {
  return reaction_[i];
}

//!Returns the reaction pointers vector
inline const std::vector<BaseReaction*> Species::reaction() const {
  return reaction_;
}

//!Sets the id string
inline void Species::setId(std::string value) {
  id_=value;
}

//!Sets the index of the species
inline void Species::setIndex( size_t value ) {
  index_ = value;
}

//!Sets the reaction(pointer) with index i
inline void Species::setReaction(size_t i,BaseReaction* value) {
  reaction_[i]=value;
}

//!Sets all reaction(pointers) for a species
inline void Species::setReaction( const std::vector<BaseReaction*> value) {
  reaction_ = value;
}

#endif
