/**
 * Filename     : transportation.cc
 * Description  : Classes describing updates due to molecule transports
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: transport.cc,v 1.16 2006/03/18 12:23:08 henrik Exp $
 */
#include"transport.h"
#include"baseReaction.h"


//!Constructor for the DiffusionSimple class
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated. The
  parameter is D in dc/dt = D*(c_n-c) (and dc_n/dt = -D*(c_n-c)). The
  update only applys if the neighbor index is larger.
*/
DiffusionSimple::DiffusionSimple(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "DiffusionSimple::DiffusionSimple() "
	      << "Uses only one parameter D.\n";
    exit(0);
  }
  if( indValue.size() != 0 ) {
    std::cerr << "DiffusionSimple::DiffusionSimple() "
	      << "No variable index used.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("diffusionSimple");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "D";
  setParameterId( tmp );
}

//! Derivative contribution for a degradation proportional to concentration
void DiffusionSimple::derivs(Compartment &compartment,size_t species,
			     std::vector< std::vector<double> > &y,
			     std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only update if compartments index less than neighbors
    if( i<j ) {
      double diff = y[j][species]-y[i][species];
      dydt[i][species] += parameter(0)*diff;
      dydt[j][species] -= parameter(0)*diff;
    }
  }
}

