# include <time.h>
# include <stdio.h>
# include <ctype.h>
# include <math.h>
# include <string.h>
# include <stdlib.h>
# include "logfunc.h"
/****************************************************************************/
/* program bas.c                                                            */
/************* geometry *****************************************************/
# define N 16                      /* # amino acids                         */
# define NTO 244                   /* # atoms in total                      */
# define NDF 32                    /* # backbone DOFs                       */
# define NRT 41                    /* # rotamer DOFs                        */
# define NHAA 5                    /* # hydrophobic amino acids             */
# define NR1 0                     /* first amino acid in rmsd              */
# define NR2 15                    /* last amino acid in rmsd               */
# define DFL 1                     /* rmsd calc, 0 backbone, 1 all atoms    */
# define NDON 3
# define NACC 8
/*--------------------------------------------------------------------------*/
# define NBA (3*N)                 /* # backbone atoms                      */
# define NA 5                      /* # atom types                          */
# define NDI 360                   /* # rotamer states (multiple of 6)      */
/************* interactions *************************************************/
# define POWA 0.5                  /* powers in hbonds                      */
# define POWB 0.5                  /* powers in hbonds                      */
# define ALPHA1 0.75               /* scale factor nonlocal pairs           */
# define ALPHA2 0.9                /* scale factor NCd                      */
# define TMIN 0.41                 /* min temperature                       */
# define TMAX 0.55                 /* max temperature                       */
# define NTMP 8                    /* # temperatures                        */
/************* MC parameters ************************************************/
# define MCCYC 6000                /* # cycles                              */
# define MCCON 100                 /* # conf updates per cycle              */
# define NTHERM 1000               /* # discarded steps                     */
# define ISTART 0                  /* 0 hot, 1 helix, 2 old                 */
# define IREADG 1                  /* 1 read g, else g=0                    */
# define DELPH 6.28                /* step size pivot                       */
# define INORM 100                 /* normalize                             */
# define IRT 100                   /* run time                              */
# define IRAMA 100                 /* Ramachandran data                     */
# define ICONF 1000                /* configurations                        */
# define IBLOCK 1000               /* size data blocks                      */
# define NOBS 10                   /* # observables                         */
# define MAXCELL 250000            /* maximum # cells                       */
# define MAXLOC (100*N)            /* maximum # local pairs                 */
# define NBIN 200                  /* # bins in histograms                  */
/************* FP parameters ************************************************/
# define MIN_DENOM 1e-6            /* used in mirror MC step                */
/************* file names ***************************************************/
# define RT "tmp_rt" 
# define RAMA "tmp_rama" 
# define CONF "tmp_conf" 
# define STARTCONF "old_conf" 
# define NATIVE "hairpin.pdb"
# define DATA "tmp_data" 
# define INPUT "input" 
# define OUTPUT "output" 
/************* functions ****************************************************/
# define max(A,B) ((A)>(B)?(A):(B)) 
# define min(A,B) ((A)<(B)?(A):(B))
/*--------------------------------------------------------------------------*/
 
/************* geometry *****************************************************/
int seq[N];                        /* sequence                              */
int iN[N];                         /* indices of N atoms                    */
int iCa[N];                        /* indices of Ca atoms                   */
int iC[N];                         /* indices of C atoms                    */
int irt[N];                        /* indices of rotamers                   */
int atyp[NTO];                     /* atom types                            */
double E,Ebi,Ehb,Esa,Ehp;          /* energies                              */
double x[NTO],y[NTO],z[NTO];       /* atom coordinates                      */
double xo[NTO],yo[NTO],zo[NTO];    /* backup                                */
double ph[NBA];                    /* all backbone torsional angles         */
short rt[max(1,NRT)];              /* rotamers                              */
const double b[3]={1.33,1.46,1.52};/* backbone CN,NCa,CaC bond lengths      */
const double bNH=1.00;             /* NH bond length                        */
const double bCaHa=1.08;           /* CaHa bond length                      */
const double bCaCb=1.53;           /* CaCb bond length                      */
const double bCO=1.23;             /* CO bond length                        */
/************* interactions *************************************************/
const double kbb=0.0;              /* bias beta                             */
const double kba=0.0;              /* bias alpha                            */
const double epshb=3.1;            /* depth H bond                          */
const double epshb2=2.0;           /* depth H bond                          */
const double sighb=2.0;            /* min H bond                            */
const double cuthb=4.5;            /* cutoff H bond                         */
const double sigsa[NA]={1.71,      /* CS                                    */
	                1.64,      /* N                                     */
                        1.42,      /* O                                     */
                        1.00,      /* H                                     */
			1.90};     /* S                                     */
const double ksa=0.10;             /* strength sa                           */
const double cut=4.3;              /* cutoff sa                             */
const double epshp=1.5;            /* strength hp                           */
const double ahp=3.5;              /* inner width hp                        */
const double bhp=4.5;              /* outer width hp                        */
short aa[NTO][NTO];                /* types of pairs                        */
int ih[N];                         /* hydrophobic amino acids (-1 -> polar) */
/************* misc *********************************************************/
const double abgs=300.0;           /* bgs parameter                         */
const double bbgs=10.0;            /* bgs parameter                         */
double beta[NTMP];                 /* inverse temperatures                  */
double g[NTMP];                    /* sim. temp. parameters                 */
int ind;                           /* index of dynamical parameter          */
double pi,pi2,pid2;                /* pi,2*pi,pi/2                          */
long seed=-11;                     /* random number seed                    */
/*--------------------------------------------------------------------------*/
double ran3n(long *seed);
void int2cart(int ia); 
void ala(int iflag,int ia);
void val(int iflag,int ia,int ir);
void leu(int iflag,int ia,int ir);
void ile(int iflag,int ia,int ir);
void ser(int iflag,int ia,int ir);
void thr(int iflag,int ia,int ir);
void met(int iflag,int ia,int ir);
void pro(int iflag,int ia);
void asp(int iflag,int ia,int ir);
void asn(int iflag,int ia,int ir);
void glu(int iflag,int ia,int ir);
void gln(int iflag,int ia,int ir);
void lys(int iflag,int ia,int ir);
void arg(int iflag,int ia,int ir);
void his(int iflag,int ia,int ir);
void phe(int iflag,int ia,int ir);
void tyr(int iflag,int ia,int ir);
void trp(int iflag,int ia,int ir);
void add_sc1(double alpha,double beta,double gamma,
             int i1,int i2,int i3,int i4);
void add_sc2(double alpha1,double beta1,double gamma1,
	     double alpha2,double beta2,double gamma2,
             int i1,int i2,int i3,int i4,int i5);
void add_sc3(double alpha1,double beta1,double gamma1,
	     double alpha2,double beta2,double gamma2,
	     double alpha3,double beta3,double gamma3,
             int i1,int i2,int i3,int i4,int i5,int i6);
double bias(int iflag);
double hbonds(int iflag);
double sac(int iflag,double emax);
double hp(int iflag);
int bgs(void);
int pivot(int c);
int mirror(int c);
int rot(int c);
int flip(double e); 
void profile(int iflag);
int helix(int iflag);
void histor(int iflag);
void histoe(int iflag,int itmp,double x);
void histoq(int iflag,int itmp,double x);
void histohp(int iflag,int itmp,double x);
double ree2(void);
double gyros(void);
double delta2(int flag);
void printheading(double frbgs[],double frmir[],double frrot[]); 
void write_conf(int iflag,char *fn,long seed);
void restart(char *fn,long *seed);
void plot_rama(int iaa);
void runtime(long it,double o[],int n);
void dumppdb(void);
void init(void); 

int main ()
{
  int i,j,iblock=0;
  long npiv[NTMP],nbgs[NTMP],nmir[NTMP],nrot[NTMP],imc,nnorm=0;
  double accpiv[NTMP],accbgs[NTMP],accmir[NTMP],accrot[NTMP],accflip=0;
  double o[NOBS],po[NTMP][NOBS],so[NTMP][NOBS];
  double frbgs[NTMP]={0.0,0.0,0.0,0.1,0.1,0.2,0.2,0.2};
  double frmir[NTMP]={0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05};
  double frrot[NTMP]={0.7,0.7,0.7,0.7,0.7,0.7,0.7,0.7}; 
  double tmp,roff=0,tmpx,tmpy,tmpz;
  FILE *fp;

  init();
  int2cart(0);
  //  dumppdb();
  E=(Ebi=bias(0))+(Ehb=hbonds(0))+(Esa=sac(1,0.0))+(Ehp=hp(0));
  printf("initial energy %f %f %f %f %f\n",Ebi,Ehb,Esa,Ehp,E);
  //  write_conf(0,CONF,0); 
  //  return;  
  o[1]=E; o[2]=Ebi; o[3]=Ehb; o[4]=Esa; o[5]=Ehp; o[6]=gyros(); o[7]=helix(1); 
  o[8]=E*E; o[9]=exp(-delta2(DFL)/100.); 
  for (i=0;i<NTMP;i++) {
    accpiv[i]=accbgs[i]=accmir[i]=accrot[i]=npiv[i]=nbgs[i]=nmir[i]=nrot[i]=0;
    for (j=0;j<NOBS;j++) po[i][j]=so[i][j]=0;
  }
  for (i=0;i<NTO;i++) {xo[i]=x[i]; yo[i]=y[i]; zo[i]=z[i];}
  printheading(frbgs,frmir,frrot);

  for (imc=0;imc<MCCYC;imc++) { 
    accflip+=flip(E);
    for (i=0;i<MCCON;i++) {
      if ((tmp=ran3n(&seed))<frbgs[ind]) {
	nbgs[ind]++; accbgs[ind]+=bgs(); 
      }
      else if (tmp<frbgs[ind]+frmir[ind]) {
        nmir[ind]++; accmir[ind]+=mirror(0); 
      }
      else if (tmp<frbgs[ind]+frmir[ind]+frrot[ind]) {
        nrot[ind]++; accrot[ind]+=rot(0); 
      }
      else {
        npiv[ind]++; accpiv[ind]+=pivot(0);
      }
      if ((i+1)%INORM==0) {
        int2cart(0);
        nnorm++;
        roff+=(tmp=(tmpx=fabs(xo[NTO-1]-x[NTO-1]))+
                   (tmpy=fabs(yo[NTO-1]-y[NTO-1]))+
	           (tmpz=fabs(zo[NTO-1]-z[NTO-1])));
	for (j=0;j<NTO;j++) {xo[j]=x[j]; yo[j]=y[j]; zo[j]=z[j];}
        if (tmp>0.000001) {
          printf("energy normalization %li %f\n",imc,tmp); 
          printf("%f %f %f\n",tmpx,tmpy,tmpz);
          E=(Ebi=bias(0))+(Ehb=hbonds(0))+(Esa=sac(1,0.0))+(Ehp=hp(0));
	}
      } 
    }
    o[1]=E; o[2]=Ebi; o[3]=Ehb; o[4]=Esa; o[5]=Ehp; o[6]=gyros(); 
    o[7]=helix(1); o[8]=E*E; o[9]=exp(-delta2(DFL)/100.); 

    if ((imc+1)>NTHERM) {
      profile(0); 
      po[ind][0]++; for (i=1;i<NOBS;i++) po[ind][i]+=o[i];
      iblock++;
      if (iblock==IBLOCK) {
        fp=fopen(DATA,"a");
        for (i=0;i<NTMP;i++) {
	  for (j=0;j<NOBS;j++) so[i][j]+=po[i][j]; 
          fwrite(&po[i][0],sizeof(double),NOBS,fp);
	  for (j=0;j<NOBS;j++) po[i][j]=0;
	}
        fclose(fp);
	iblock=0;
      }

      if (ind==NTMP-1) histor(0);
      histoe(0,ind,o[1]);
      histoq(0,ind,o[9]);
      histohp(0,ind,o[5]);
      if ((imc+1)%IRAMA==0 && ind==NTMP-1) {plot_rama(10);}  
      if ((imc+1)%ICONF==0) {
	for (i=0;i<NTO;i++) {xo[i]=x[i]; yo[i]=y[i]; zo[i]=z[i];}
        E=(Ebi=bias(0))+(Ehb=hbonds(0))+(Esa=sac(1,0.0))+(Ehp=hp(0));
	write_conf(0,CONF,0); 
      }
    }
    if ((imc+1)%IRT==0) {dumppdb(); runtime(imc+1,o,NOBS);} 
  }

  /* end run */

  printf("run over\n");
  printf("average error in last atom %e\n\n",roff/nnorm);
  printf("Acceptance\n"); 
  printf("==========\n\n"); 
  printf("nflip %i  acc flip %f\n",
          MCCYC-NTHERM,accflip/(MCCYC-NTHERM));
  for (i=0;i<NTMP;i++) {
    printf("temp %i  ",i);
    printf("npiv %10ld  acc piv ",npiv[i]);
    if (npiv[i]>0) printf("%5.3f       ",accpiv[i]/npiv[i]);
    else printf("            ");
    printf("\n");
  }
  for (i=0;i<NTMP;i++) {
    printf("temp %i  ",i);
    printf("nbgs %10ld  acc bgs ",nbgs[i]);
    if (nbgs[i]>0) printf("%5.3f       ",accbgs[i]/nbgs[i]);
    else printf("            ");
    printf("\n");
  }
  for (i=0;i<NTMP;i++) {
    printf("temp %i  ",i);
    printf("nmir %10ld  acc mir ",nmir[i]);
    if (nmir[i]>0) printf("%5.3f       ",accmir[i]/nmir[i]);
    else printf("            ");
    printf("\n");
  }
  for (i=0;i<NTMP;i++) {
    printf("temp %i  ",i);
    printf("nrot %10ld  acc rot ",nrot[i]);
    if (nrot[i]>0) printf("%5.3f       ",accrot[i]/nrot[i]);
    else printf("            ");
    printf("\n");
  }
  printf("\n");
  printf("Averages\n");
  printf("========\n\n");
  for (i=0;i<NTMP;i++) {
    printf("temp %f  %f\n",1/beta[i],so[i][0]/(MCCYC-NTHERM));
    if (so[i][0]>0) {
      for (j=1;j<NOBS;j++) { 
        if (j!=8)
	  printf("obs %i = %f\n",j,so[i][j]/so[i][0]);
        else
          printf("obs %i = %f\n",j,
            beta[i]*beta[i]*(so[i][j]/so[i][0]-pow(so[i][1]/so[i][0],2.0))/N);
      }
    }
    printf("\n");
  }

  histor(1);
  for (j=0;j<NTMP;j++) histoe(1,j,so[j][0]);
  for (j=0;j<NTMP;j++) histoq(1,j,so[j][0]);
  for (j=0;j<NTMP;j++) histohp(1,j,so[j][0]);
  profile(1);

  fp=fopen(OUTPUT,"a");
  for (i=0;i<NTMP;i++) g[i]+=LOGFUNC(max(so[i][0]/(MCCYC-NTHERM),0.01/NTMP));
  for (i=0;i<NTMP;i++) fprintf(fp,"%i %f\n",i,g[i]-g[NTMP-1]);
  fclose(fp);

  return 0;
}
/****************************************************************************/
/***** CHAIN CONSTRUCTION ***************************************************/
/****************************************************************************/
void int2cart(int ia)
{
  int i,j,k;
  double ux,uy,uz,renorm; 
  double sph,cph,sth,cth;
  double xt[NBA],yt[NBA],zt[NBA];
  double bx[NBA+1],by[NBA+1],bz[NBA+1];
  static double a[3],b1[3],b2[3],g[3];
  static double th[3];  
  static double aH,bH;
  static double aO,bO;
  static double phCb,thCb,aCb,b1Cb,b2Cb,gCb;           
  static double phHa,thHa,aHa,b1Ha,b2Ha,gHa,phHa2;

  if (ia<0) {                           
    th[0]=121.7*pi/180;
    th[1]=111.0*pi/180;
    th[2]=116.6*pi/180;
    thCb=110.0*pi/180;
    phCb=-120.9*pi/180;
    thHa=109.0*pi/180;
    phHa=118.7*pi/180;
    phHa2=-118.7*pi/180;
    for (i=0;i<3;i++) {
      sth=sin(pi-th[(i+2)%3]); cth=cos(pi-th[(i+2)%3]);
      a[i]=b[i%3]*sth/b[(i+1)%3]/sin(pi-th[(i+1)%3]);
      b1[i]=b[i%3]*cth/b[(i+2)%3];
      b2[i]=-b[i%3]*sth/tan(pi-th[(i+1)%3])/b[(i+2)%3];
      g[i]=b[i%3]*sth/b[(i+2)%3]/b[(i+1)%3]/sin(pi-th[(i+1)%3]);
    }
    aH=bNH/b[0]/sqrt(2*(1-cos(pi-th[0])));
    bH=-b[0]/b[1];
    aHa=bCaHa*sin(pi-thHa)/b[0]/sin(pi-th[0]);
    b1Ha=bCaHa*cos(pi-thHa)/b[1];
    b2Ha=-bCaHa*sin(pi-thHa)/b[1]/tan(pi-th[0]);
    gHa=bCaHa*sin(pi-thHa)/b[0]/b[1]/sin(pi-th[0]);
    aCb=bCaCb*sin(pi-thCb)/b[0]/sin(pi-th[0]);
    b1Cb=bCaCb*cos(pi-thCb)/b[1];
    b2Cb=-bCaCb*sin(pi-thCb)/b[1]/tan(pi-th[0]);
    gCb=bCaCb*sin(pi-thCb)/b[0]/b[1]/sin(pi-th[0]);
    aO=bCO/b[2]/sqrt(2*(1-cos(pi-th[2])));
    bO=-b[2]/b[0];
    ala(-1,0);
    val(-1,0,0);
    leu(-1,0,0);
    ile(-1,0,0);
    ser(-1,0,0);
    thr(-1,0,0);
    met(-1,0,0);
    pro(-1,0);
    asp(-1,0,0);
    asn(-1,0,0);
    glu(-1,0,0);
    gln(-1,0,0);
    lys(-1,0,0);
    arg(-1,0,0);
    his(-1,0,0);
    phe(-1,0,0);
    tyr(-1,0,0);
    trp(-1,0,0);
    return;
  }

  if (ia==0) {
    xt[0]=bx[0]=b[0]; yt[0]=by[0]=zt[0]=bz[0]=0;
    bx[1]=cos(pi-th[0])*b[1]; by[1]=0; bz[1]=sin(pi-th[0])*b[1];
    xt[1]=xt[0]+bx[1]; yt[1]=0; zt[1]=bz[1];
  }
  else {
    bx[3*ia]=x[iN[ia]]-x[iC[ia-1]];
    by[3*ia]=y[iN[ia]]-y[iC[ia-1]];
    bz[3*ia]=z[iN[ia]]-z[iC[ia-1]];
    xt[3*ia]=x[iN[ia]]; yt[3*ia]=y[iN[ia]]; zt[3*ia]=z[iN[ia]];
    xt[3*ia+1]=x[iCa[ia]]; yt[3*ia+1]=y[iCa[ia]]; zt[3*ia+1]=z[iCa[ia]];    
    bx[3*ia+1]=xt[3*ia+1]-xt[3*ia];
    by[3*ia+1]=yt[3*ia+1]-yt[3*ia];
    bz[3*ia+1]=zt[3*ia+1]-zt[3*ia];
  }

  for (i=3*ia+2;i<=NBA;i++) { /* backbone */
    cph=cos(ph[i-2]-pid2); sph=sin(ph[i-2]-pid2);
    ux=by[i-2]*bz[i-1]-by[i-1]*bz[i-2]; 
    uy=bz[i-2]*bx[i-1]-bz[i-1]*bx[i-2]; 
    uz=bx[i-2]*by[i-1]-bx[i-1]*by[i-2];
    k=i%3;
    bx[i]=a[k]*sph*bx[i-2]+(b1[k]+b2[k]*sph)*bx[i-1]+g[k]*cph*ux;
    by[i]=a[k]*sph*by[i-2]+(b1[k]+b2[k]*sph)*by[i-1]+g[k]*cph*uy;
    bz[i]=a[k]*sph*bz[i-2]+(b1[k]+b2[k]*sph)*bz[i-1]+g[k]*cph*uz;
    renorm=b[k]/sqrt(bx[i]*bx[i]+by[i]*by[i]+bz[i]*bz[i]);
    bx[i]*=renorm;
    by[i]*=renorm;
    bz[i]*=renorm;
    if (i<NBA) {  
      xt[i]=xt[i-1]+bx[i]; yt[i]=yt[i-1]+by[i]; zt[i]=zt[i-1]+bz[i];
    }
    if (k==2) { /* Ha */
      j=iCa[i/3];
      cph=cos(ph[i-2]+phHa-pid2); sph=sin(ph[i-2]+phHa-pid2);
      x[j+1]=xt[i-1]+aHa*sph*bx[i-2]+(b1Ha+b2Ha*sph)*bx[i-1]+gHa*cph*ux;
      y[j+1]=yt[i-1]+aHa*sph*by[i-2]+(b1Ha+b2Ha*sph)*by[i-1]+gHa*cph*uy;
      z[j+1]=zt[i-1]+aHa*sph*bz[i-2]+(b1Ha+b2Ha*sph)*bz[i-1]+gHa*cph*uz;
      if (seq[i/3]!='G') { /* Cb */
	cph=cos(ph[i-2]+phCb-pid2); sph=sin(ph[i-2]+phCb-pid2);
	x[j+2]=xt[i-1]+aCb*sph*bx[i-2]+(b1Cb+b2Cb*sph)*bx[i-1]+gCb*cph*ux;
	y[j+2]=yt[i-1]+aCb*sph*by[i-2]+(b1Cb+b2Cb*sph)*by[i-1]+gCb*cph*uy;
	z[j+2]=zt[i-1]+aCb*sph*bz[i-2]+(b1Cb+b2Cb*sph)*bz[i-1]+gCb*cph*uz;
      }
      else { /* Ha2 */
	cph=cos(ph[i-2]+phHa2-pid2); sph=sin(ph[i-2]+phHa2-pid2);
	x[j+2]=xt[i-1]+aHa*sph*bx[i-2]+(b1Ha+b2Ha*sph)*bx[i-1]+gHa*cph*ux;
	y[j+2]=yt[i-1]+aHa*sph*by[i-2]+(b1Ha+b2Ha*sph)*by[i-1]+gHa*cph*uy;
	z[j+2]=zt[i-1]+aHa*sph*bz[i-2]+(b1Ha+b2Ha*sph)*bz[i-1]+gHa*cph*uz;
      }
    }
  }

  for (i=ia;i<N;i++) {
    x[iN[i]]=xt[3*i]; x[iCa[i]]=xt[3*i+1]; x[iC[i]]=xt[3*i+2];    
    y[iN[i]]=yt[3*i]; y[iCa[i]]=yt[3*i+1]; y[iC[i]]=yt[3*i+2];     
    z[iN[i]]=zt[3*i]; z[iCa[i]]=zt[3*i+1]; z[iC[i]]=zt[3*i+2];
  }

  for (i=ia;i<N;i++) { /* H */
    if (seq[i]!='P') {
      j=3*i; k=iN[i];
      x[k+1]=x[k]+aH*(bx[j]+bH*bx[j+1]);
      y[k+1]=y[k]+aH*(by[j]+bH*by[j+1]);
      z[k+1]=z[k]+aH*(bz[j]+bH*bz[j+1]);
    }
  }

  for (i=ia;i<N;i++) { /* O */
    j=3*i+2; k=iC[i];
    x[k+1]=x[k]+aO*(bx[j]+bO*bx[j+1]);
    y[k+1]=y[k]+aO*(by[j]+bO*by[j+1]);
    z[k+1]=z[k]+aO*(bz[j]+bO*bz[j+1]);
  }

  for (i=ia;i<N;i++) {
    switch (seq[i]){  
    case 'A' : {ala(0,i); break;}
    case 'V' : {val(0,i,0); break;}
    case 'L' : {leu(0,i,0); break;}
    case 'I' : {ile(0,i,0); break;}
    case 'S' : {ser(0,i,0); break;}
    case 'T' : {thr(0,i,0); break;}
    case 'M' : {met(0,i,0); break;}
    case 'P' : {pro(0,i); break;}
    case 'D' : {asp(0,i,0); break;}
    case 'N' : {asn(0,i,0); break;}
    case 'E' : {glu(0,i,0); break;}
    case 'Q' : {gln(0,i,0); break;}
    case 'K' : {lys(0,i,0); break;}
    case 'R' : {arg(0,i,0); break;}
    case 'H' : {his(0,i,0); break;}
    case 'F' : {phe(0,i,0); break;}
    case 'Y' : {tyr(0,i,0); break;}
    case 'W' : {trp(0,i,0); break;}
    default : {}
    }
  }
}
/****************************************************************************/
void ala(int iflag,int ia)
{
  const double bXH=1.0;
  double thCb,thH,chi;
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  int i,j,k;
  
  if (iflag<0) {
    thCb=110.0*pi/180;
    thH=109.5*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2.0*i*pi/NDI; 
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
    }    
    return;
  }
  i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
  add_sc3(alpha1H[i],beta1H[i],gamma1H[i],alpha1H[j],beta1H[j],gamma1H[j],
	  alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	  iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
}
/****************************************************************************/
void val(int iflag,int ia,int ir)
{
  const double c1=1.52,bXH=1.0;
  double th,thCb,thH,thH2,chi;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  int i,j,k;

  if (iflag<0) {
    th=110.7*pi/180;
    thCb=110.0*pi/180;
    thH=109.1*pi/180;
    thH2=109.5*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2.0*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th)-sin(pi-th)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2H[i]=bXH*sin(pi-thH2)*sin(chi-pid2)/bCaCb/sin(pi-th);
      beta2H[i]=cos(pi-thH2)-sin(pi-thH2)*sin(chi-pid2)/tan(pi-th);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thH2)*cos(chi-pid2)/sin(pi-th)/bCaCb/c1;
    }    
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1[j],beta1[j],gamma1[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2H[i],beta2H[i],gamma2H[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  if ((ir==0) || (ir==2)) {
    i=rt[irt[ia]+2]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2H[i],beta2H[i],gamma2H[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+4,
	    iCa[ia]+9,iCa[ia]+10,iCa[ia]+11);
  }
}
/****************************************************************************/
void leu(int iflag,int ia,int ir)
{
  const double c1=1.53,c2=1.52,bXH=1.0;
  double th1,th2,thHb,thHg,thHd,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  static double alpha3H[NDI],beta3H[NDI],gamma3H[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=117.1*pi/180;
    th2=110.1*pi/180;
    thHb=107.0*pi/180;
    thHg=109.3*pi/180;
    thHd=109.5*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thHb)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thHb)-sin(pi-thHb)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thHb)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th1);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3H[i]=bXH*sin(pi-thHd)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3H[i]=cos(pi-thHd)-sin(pi-thHd)*sin(chi-pid2)/tan(pi-th2);
      beta3H[i]*=bXH/c2;
      gamma3H[i]=bXH*sin(pi-thHd)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
    }    
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+2*NDI/3)%NDI;  k=(i+NDI/3)%NDI; 
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2[i],beta2[i],gamma2[i],alpha2[j],beta2[j],gamma2[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  if (ir<=2) {
    i=rt[irt[ia]+2]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha3H[i],beta3H[i],gamma3H[i],alpha3H[j],beta3H[j],gamma3H[j],
	    alpha3H[k],beta3H[k],gamma3H[k],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,
	    iCa[ia]+9,iCa[ia]+10,iCa[ia]+11);
  }
  if ((ir<=1) || (ir==3)) {
    i=rt[irt[ia]+3]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha3H[i],beta3H[i],gamma3H[i],alpha3H[j],beta3H[j],gamma3H[j],
	    alpha3H[k],beta3H[k],gamma3H[k],iCa[ia]+2,iCa[ia]+3,iCa[ia]+7,
	    iCa[ia]+12,iCa[ia]+13,iCa[ia]+14);
  }
}
/****************************************************************************/
void ile(int iflag,int ia,int ir)
{
  const double c1=1.53,c2=1.52,bXH=1.0;
  double th1,th2,thHb,thHg,thHd,thHg2,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];    
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];    
  static double alpha3H[NDI],beta3H[NDI],gamma3H[NDI];    
  static double alpha4H[NDI],beta4H[NDI],gamma4H[NDI];    
  int i,j,k;

  if (iflag<0) {
    th1=110.4*pi/180;
    th2=113.6*pi/180;
    thHb=109.2*pi/180;
    thHg=108.1*pi/180;
    thHd=109.5*pi/180;
    thHg2=109.5*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thHb)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thHb)-sin(pi-thHb)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thHb)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th1);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3H[i]=bXH*sin(pi-thHd)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3H[i]=cos(pi-thHd)-sin(pi-thHd)*sin(chi-pid2)/tan(pi-th2);
      beta3H[i]*=bXH/c2;
      gamma3H[i]=bXH*sin(pi-thHd)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha4H[i]=bXH*sin(pi-thHg2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta4H[i]=cos(pi-thHg2)-sin(pi-thHg2)*sin(chi-pid2)/tan(pi-th1);
      beta4H[i]*=bXH/c1;
      gamma4H[i]=bXH*sin(pi-thHg2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
    }    
    return;
  }

  if (ir==0) {
    i=rt[irt[ia]]; j=(i+2*NDI/3)%NDI; k=(i+NDI/3)%NDI; 
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1[j],beta1[j],gamma1[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) { 
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI; 
    add_sc3(alpha2[i],beta2[i],gamma2[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  if (ir<=2) {
    i=rt[irt[ia]+2]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI; 
    add_sc3(alpha3H[i],beta3H[i],gamma3H[i],alpha3H[j],beta3H[j],gamma3H[j],
	    alpha3H[k],beta3H[k],gamma3H[k],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,
	    iCa[ia]+9,iCa[ia]+10,iCa[ia]+11);
  }
  if ((ir==3) || (ir==0)) {
    i=rt[irt[ia]+3]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI; 
    add_sc3(alpha4H[i],beta4H[i],gamma4H[i],alpha4H[j],beta4H[j],gamma4H[j],
	    alpha4H[k],beta4H[k],gamma4H[k],iCa[ia],iCa[ia]+2,iCa[ia]+4,
	    iCa[ia]+12,iCa[ia]+13,iCa[ia]+14);
  }    
}
/****************************************************************************/
void ser(int iflag,int ia,int ir)
{
  const double c1=1.42,bXH=1.0;
  double th,thH,thHg,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  int i,j,k;

  if (iflag<0) {
    th=110.6*pi/180;
    thH=109.1*pi/180;
    thHg=108.0*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th)-sin(pi-th)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th)/bCaCb/c1;
    }    
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  i=rt[irt[ia]+1]; 
  add_sc1(alpha2H[i],beta2H[i],gamma2H[i],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	  iCa[ia]+6);
}
/****************************************************************************/
void thr(int iflag,int ia,int ir)
{
  const double c1=1.43,c2=1.52,bXH=1.0;
  double th1,th2,thH,thHg1,thHg2,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alphaH[NDI],betaH[NDI],gammaH[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha3H[NDI],beta3H[NDI],gamma3H[NDI];  
  static double alpha4H[NDI],beta4H[NDI],gamma4H[NDI];  

  int i,j,k;

  if (iflag<0) {
    th1=108.6*pi/180;
    th2=111.5*pi/180;
    thH=109.3*pi/180;
    thHg1=108.0*pi/180;
    thHg2=109.5*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alphaH[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      betaH[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      betaH[i]*=bXH/bCaCb;
      gammaH[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-thCb);
      beta2[i]*=c2/bCaCb;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha3H[i]=bXH*sin(pi-thHg1)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta3H[i]=cos(pi-thHg1)-sin(pi-thHg1)*sin(chi-pid2)/tan(pi-th1);
      beta3H[i]*=bXH/c1;
      gamma3H[i]=bXH*sin(pi-thHg1)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha4H[i]=bXH*sin(pi-thHg2)*sin(chi-pid2)/bCaCb/sin(pi-th2);
      beta4H[i]=cos(pi-thHg2)-sin(pi-thHg2)*sin(chi-pid2)/tan(pi-th2);
      beta4H[i]*=bXH/c2;
      gamma4H[i]=bXH*sin(pi-thHg2)*cos(chi-pid2)/sin(pi-th2)/bCaCb/c2;
    }    
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+2*NDI/3)%NDI; k=(i+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha2[j],beta2[j],gamma2[j],
	    alphaH[k],betaH[k],gammaH[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);  
  }
  if (ir<=1) {
    i=rt[irt[ia]+1];
    add_sc1(alpha3H[i],beta3H[i],gamma3H[i],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6);
  }
  if ((ir==0) || (ir==2)) {
    i=rt[irt[ia]+2]; j=(i+2*NDI/3)%NDI; k=(i+NDI/3)%NDI;
    add_sc3(alpha4H[i],beta4H[i],gamma4H[i],alpha4H[j],beta4H[j],gamma4H[j],
	    alpha4H[k],beta4H[k],gamma4H[k],iCa[ia],iCa[ia]+2,iCa[ia]+4,
	    iCa[ia]+7,iCa[ia]+8,iCa[ia]+9);
  }
}
/****************************************************************************/
void met(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.81,c3=1.79,bXH=1.0;
  double th1,th2,th3,thHb,thHg,thHe,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  static double alpha3[NDI],beta3[NDI],gamma3[NDI];  
  static double alpha4H[NDI],beta4H[NDI],gamma4H[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=113.5*pi/180;
    th2=111.9*pi/180;
    th3=100.5*pi/180;
    thHb=108.1*pi/180;
    thHg=108.7*pi/180;
    thHe=109.5*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thHb)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thHb)-sin(pi-thHb)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thHb)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th1);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3[i]=c3*sin(pi-th3)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3[i]=cos(pi-th3)-sin(pi-th3)*sin(chi-pid2)/tan(pi-th2);
      beta3[i]*=c3/c2;
      gamma3[i]=c3*sin(pi-th3)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha4H[i]=bXH*sin(pi-thHe)*sin(chi-pid2)/c2/sin(pi-th3);
      beta4H[i]=cos(pi-thHe)-sin(pi-thHe)*sin(chi-pid2)/tan(pi-th3);
      beta4H[i]*=bXH/c3;
      gamma4H[i]=bXH*sin(pi-thHe)*cos(chi-pid2)/sin(pi-th3)/c2/c3;

    }
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2[i],beta2[i],gamma2[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  if (ir<=2) {
    i=rt[irt[ia]+2]; 
    add_sc1(alpha3[i],beta3[i],gamma3[i],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,
	    iCa[ia]+9);
  }
  i=rt[irt[ia]+3]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
  add_sc3(alpha4H[i],beta4H[i],gamma4H[i],alpha4H[j],beta4H[j],gamma4H[j],
	  alpha4H[k],beta4H[k],gamma4H[k],iCa[ia]+3,iCa[ia]+6,iCa[ia]+9,
	  iCa[ia]+10,iCa[ia]+11,iCa[ia]+12);
}
/****************************************************************************/
void pro(int iflag,int ia)
{
  const double ad=-.596,bd=.910;
  const double ag=.087,bg=1.442;

  if (iflag<0) {
    return;
  }

  x[iCa[ia]+3]=x[iN[ia]]+
               ag*(x[iCa[ia]]-x[iN[ia]])+bg*(x[iCa[ia]+2]-x[iCa[ia]]);  
  y[iCa[ia]+3]=y[iN[ia]]+
               ag*(y[iCa[ia]]-y[iN[ia]])+bg*(y[iCa[ia]+2]-y[iCa[ia]]);  
  z[iCa[ia]+3]=z[iN[ia]]+
               ag*(z[iCa[ia]]-z[iN[ia]])+bg*(z[iCa[ia]+2]-z[iCa[ia]]);  
  x[iCa[ia]+4]=x[iN[ia]]+
               ad*(x[iCa[ia]]-x[iN[ia]])+bd*(x[iCa[ia]+2]-x[iCa[ia]]);  
  y[iCa[ia]+4]=y[iN[ia]]+
               ad*(y[iCa[ia]]-y[iN[ia]])+bd*(y[iCa[ia]+2]-y[iCa[ia]]);  
  z[iCa[ia]+4]=z[iN[ia]]+
               ad*(z[iCa[ia]]-z[iN[ia]])+bd*(z[iCa[ia]+2]-z[iCa[ia]]);  
}
/****************************************************************************/
void asp(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.25,bXH=1.0;
  double th1,th2,thH,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=113.2*pi/180;
    th2=118.6*pi/180;
    thH=108.2*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) 
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
    }
    return;
  }

  if (ir==0) {
      i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
      add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	      alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	      iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  i=rt[irt[ia]+1]; j=(i+NDI/2)%NDI;
  add_sc2(alpha2[i],beta2[i],gamma2[i],alpha2[j],beta2[j],gamma2[j],
	  iCa[ia],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+7);
}
/****************************************************************************/
void asn(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.23,c3=1.33,bXH=1.0;
  double th1,th2,th3,thH,thHd,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha3[NDI],beta3[NDI],gamma3[NDI];
  static double alpha3H[NDI],beta3H[NDI],gamma3H[NDI];
  int i,j,k;

  if (iflag<0) {
    th1=112.6*pi/180;
    th2=120.9*pi/180;
    th3=117.0*pi/180;
    thH=108.4*pi/180;
    thHd=120.0*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3[i]=c3*sin(pi-th3)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta3[i]=cos(pi-th3)-sin(pi-th3)*sin(chi-pid2)/tan(pi-th1);
      beta3[i]*=c3/c1;
      gamma3[i]=c3*sin(pi-th3)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3H[i]=bXH*sin(pi-thHd)*sin(chi-pid2)/c1/sin(pi-th3);
      beta3H[i]=cos(pi-thHd)-sin(pi-thHd)*sin(chi-pid2)/tan(pi-th3);
      beta3H[i]*=bXH/c3;
      gamma3H[i]=bXH*sin(pi-thHd)*cos(chi-pid2)/sin(pi-th3)/c1/c3;
    }
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/2)%NDI;
    add_sc2(alpha2[i],beta2[i],gamma2[i],alpha3[j],beta3[j],gamma3[j],
	    iCa[ia],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+7);
  }
  if (ir<=2) {
    i=rt[irt[ia]+2]; j=(i+NDI/2)%NDI;
    add_sc2(alpha3H[i],beta3H[i],gamma3H[i],alpha3H[j],beta3H[j],gamma3H[j],
	    iCa[ia]+2,iCa[ia]+3,iCa[ia]+7,iCa[ia]+8,iCa[ia]+9);
  }
}
/****************************************************************************/
void glu(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.52,c3=1.25,bXH=1.0;
  double th1,th2,th3,thHb,thHg,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  static double alpha3[NDI],beta3[NDI],gamma3[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=114.1*pi/180;
    th2=113.2*pi/180;
    th3=118.5*pi/180;
    thHb=108.0*pi/180;
    thHg=108.2*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thHb)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thHb)-sin(pi-thHb)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thHb)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th1);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3[i]=c3*sin(pi-th3)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3[i]=cos(pi-th3)-sin(pi-th3)*sin(chi-pid2)/tan(pi-th2);
      beta3[i]*=c3/c2;
      gamma3[i]=c3*sin(pi-th3)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
    }
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2[i],beta2[i],gamma2[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  i=rt[irt[ia]+2]; j=(i+NDI/2)%NDI;
  add_sc2(alpha3[i],beta3[i],gamma3[i],alpha3[j],beta3[j],gamma3[j],
	  iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+9,iCa[ia]+10);
}
/****************************************************************************/
void gln(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.52,c3=1.23,c4=1.33,bXH=1.0;
  double th1,th2,th3,th4,thHb,thHg,thHe,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  static double alpha3[NDI],beta3[NDI],gamma3[NDI];  
  static double alpha4[NDI],beta4[NDI],gamma4[NDI];  
  static double alpha4H[NDI],beta4H[NDI],gamma4H[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=113.7*pi/180;
    th2=112.6*pi/180;
    th3=121.0*pi/180;
    th4=116.9*pi/180;
    thHb=108.1*pi/180;
    thHg=108.4*pi/180;
    thHe=120.0*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thHb)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thHb)-sin(pi-thHb)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thHb)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th1);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3[i]=c3*sin(pi-th3)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3[i]=cos(pi-th3)-sin(pi-th3)*sin(chi-pid2)/tan(pi-th2);
      beta3[i]*=c3/c2;
      gamma3[i]=c3*sin(pi-th3)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha4[i]=c4*sin(pi-th4)*sin(chi-pid2)/c1/sin(pi-th2);
      beta4[i]=cos(pi-th4)-sin(pi-th4)*sin(chi-pid2)/tan(pi-th2);
      beta4[i]*=c4/c2;
      gamma4[i]=c4*sin(pi-th4)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha4H[i]=bXH*sin(pi-thHe)*sin(chi-pid2)/c2/sin(pi-th4);
      beta4H[i]=cos(pi-thHe)-sin(pi-thHe)*sin(chi-pid2)/tan(pi-th4);
      beta4H[i]*=bXH/c4;
      gamma4H[i]=bXH*sin(pi-thHe)*cos(chi-pid2)/sin(pi-th4)/c2/c4;
    }
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2[i],beta2[i],gamma2[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  if (ir<=2) {
    i=rt[irt[ia]+2]; j=(i+NDI/2)%NDI;
    add_sc2(alpha3[i],beta3[i],gamma3[i],alpha4[j],beta4[j],gamma4[j],
	    iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+9,iCa[ia]+10);
  }
  i=rt[irt[ia]+3]; j=(i+NDI/2)%NDI;
  add_sc2(alpha4H[i],beta4H[i],gamma4H[i],alpha4H[j],beta4H[j],gamma4H[j],
	  iCa[ia]+3,iCa[ia]+6,iCa[ia]+10,iCa[ia]+11,iCa[ia]+12);
}
/****************************************************************************/
void lys(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.52,c3=1.52,c4=1.49,bXH=1.0;
  double th1,th2,th3,th4,thHb,thHg,thHz,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  static double alpha3[NDI],beta3[NDI],gamma3[NDI];  
  static double alpha3H[NDI],beta3H[NDI],gamma3H[NDI];  
  static double alpha4[NDI],beta4[NDI],gamma4[NDI];  
  static double alpha4H[NDI],beta4H[NDI],gamma4H[NDI];  
  static double alpha5H[NDI],beta5H[NDI],gamma5H[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=113.8*pi/180;
    th2=th3=th4=111.6*pi/180;
    thHb=108.1*pi/180;
    thHg=108.8*pi/180;
    thHz=109.5*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;

      alpha1H[i]=bXH*sin(pi-thHb)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thHb)-sin(pi-thHb)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thHb)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th1);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3[i]=c3*sin(pi-th3)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3[i]=cos(pi-th3)-sin(pi-th3)*sin(chi-pid2)/tan(pi-th2);
      beta3[i]*=c3/c2;
      gamma3[i]=c3*sin(pi-th3)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha3H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th2);
      beta3H[i]*=bXH/c2;
      gamma3H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha4[i]=c4*sin(pi-th4)*sin(chi-pid2)/c2/sin(pi-th3);
      beta4[i]=cos(pi-th4)-sin(pi-th4)*sin(chi-pid2)/tan(pi-th3);
      beta4[i]*=c4/c3;
      gamma4[i]=c4*sin(pi-th4)*cos(chi-pid2)/sin(pi-th3)/c2/c3;
      alpha4H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/c2/sin(pi-th3);
      beta4H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th3);
      beta4H[i]*=bXH/c3;
      gamma4H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th3)/c2/c3;
      alpha5H[i]=bXH*sin(pi-thHz)*sin(chi-pid2)/c3/sin(pi-th4);
      beta5H[i]=cos(pi-thHz)-sin(pi-thHz)*sin(chi-pid2)/tan(pi-th4);
      beta5H[i]*=bXH/c4;
      gamma5H[i]=bXH*sin(pi-thHz)*cos(chi-pid2)/sin(pi-th4)/c3/c4;
    }
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2[i],beta2[i],gamma2[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  if (ir<=2) {
    i=rt[irt[ia]+2]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha3[i],beta3[i],gamma3[i],alpha3H[j],beta3H[j],gamma3H[j],
	    alpha3H[k],beta3H[k],gamma3H[k],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,
	    iCa[ia]+9,iCa[ia]+10,iCa[ia]+11);
  }
  if (ir<=3) {
    i=rt[irt[ia]+3]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha4[i],beta4[i],gamma4[i],alpha4H[j],beta4H[j],gamma4H[j],
	    alpha4H[k],beta4H[k],gamma4H[k],iCa[ia]+3,iCa[ia]+6,iCa[ia]+9,
	    iCa[ia]+12,iCa[ia]+13,iCa[ia]+14);
  }
  i=rt[irt[ia]+4]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
  add_sc3(alpha5H[i],beta5H[i],gamma5H[i],alpha5H[j],beta5H[j],gamma5H[j],
	  alpha5H[k],beta5H[k],gamma5H[k],iCa[ia]+6,iCa[ia]+9,iCa[ia]+12,
	  iCa[ia]+15,iCa[ia]+16,iCa[ia]+17);
}
/****************************************************************************/
void arg(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.52,c3=1.46,c4=1.33,c5=1.33,bXH=1.0;
  double th1,th2,th3,th4,th5,thHb,thHg,thHd,chi;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha2H[NDI],beta2H[NDI],gamma2H[NDI];  
  static double alpha3[NDI],beta3[NDI],gamma3[NDI];  
  static double alpha3H[NDI],beta3H[NDI],gamma3H[NDI];  
  static double alpha4[NDI],beta4[NDI],gamma4[NDI];  
  static double alpha5,beta5,gamma5;  
  static double alpha6,beta6,gamma6;  
  double ax,ay,az,vx,vy,vz,ux,uy,uz;
  static double hf;
  int i,j,k;

  if (iflag<0) {
    th1=113.7*pi/180;
    th2=th3=111.5*pi/180;
    th4=124.4*pi/180;
    th5=120.0*pi/180;
    thHb=108.1*pi/180;
    thHg=thHd=108.8*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thHb)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thHb)-sin(pi-thHb)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thHb)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha2H[i]=bXH*sin(pi-thHg)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2H[i]=cos(pi-thHg)-sin(pi-thHg)*sin(chi-pid2)/tan(pi-th1);
      beta2H[i]*=bXH/c1;
      gamma2H[i]=bXH*sin(pi-thHg)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3[i]=c3*sin(pi-th3)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3[i]=cos(pi-th3)-sin(pi-th3)*sin(chi-pid2)/tan(pi-th2);
      beta3[i]*=c3/c2;
      gamma3[i]=c3*sin(pi-th3)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha3H[i]=bXH*sin(pi-thHd)*sin(chi-pid2)/c1/sin(pi-th2);
      beta3H[i]=cos(pi-thHd)-sin(pi-thHd)*sin(chi-pid2)/tan(pi-th2);
      beta3H[i]*=bXH/c2;
      gamma3H[i]=bXH*sin(pi-thHd)*cos(chi-pid2)/sin(pi-th2)/c1/c2;
      alpha4[i]=c4*sin(pi-th4)*sin(chi-pid2)/c2/sin(pi-th3);
      beta4[i]=cos(pi-th4)-sin(pi-th4)*sin(chi-pid2)/tan(pi-th3);
      beta4[i]*=c4/c3;
      gamma4[i]=c4*sin(pi-th4)*cos(chi-pid2)/sin(pi-th3)/c2/c3;
    }
    alpha5=c5*sin(pi-th5)*sin(0.0-pid2)/c3/sin(pi-th4);
    beta5=cos(pi-th5)-sin(pi-th5)*sin(0.0-pid2)/tan(pi-th4);
    beta5*=c5/c4;
    gamma5=c5*sin(pi-th5)*cos(0.0-pid2)/sin(pi-th4)/c3/c4;   
    alpha6=c5*sin(pi-th5)*sin(pi-pid2)/c3/sin(pi-th4);
    beta6=cos(pi-th5)-sin(pi-th5)*sin(pi-pid2)/tan(pi-th4);
    beta6*=c5/c4;
    gamma6=c5*sin(pi-th5)*cos(pi-pid2)/sin(pi-th4)/c3/c4;   
    hf=bXH/c5;
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha2[i],beta2[i],gamma2[i],alpha2H[j],beta2H[j],gamma2H[j],
	    alpha2H[k],beta2H[k],gamma2H[k],iCa[ia],iCa[ia]+2,iCa[ia]+3,
	    iCa[ia]+6,iCa[ia]+7,iCa[ia]+8);
  }
  if (ir<=2) {
    i=rt[irt[ia]+2]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha3[i],beta3[i],gamma3[i],alpha3H[j],beta3H[j],gamma3H[j],
	    alpha3H[k],beta3H[k],gamma3H[k],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,
	    iCa[ia]+9,iCa[ia]+10,iCa[ia]+11);
  }
  i=rt[irt[ia]+3]; 
  add_sc1(alpha4[i],beta4[i],gamma4[i],
	  iCa[ia]+3,iCa[ia]+6,iCa[ia]+9,iCa[ia]+12);
  add_sc2(alpha5,beta5,gamma5,alpha6,beta6,gamma6,
	  iCa[ia]+6,iCa[ia]+9,iCa[ia]+12,iCa[ia]+13,iCa[ia]+14);
  
  ax=x[iCa[ia]+12]-x[iCa[ia]+9];
  ay=y[iCa[ia]+12]-y[iCa[ia]+9];
  az=z[iCa[ia]+12]-z[iCa[ia]+9];
  ux=x[iCa[ia]+13]-x[iCa[ia]+12];
  uy=y[iCa[ia]+13]-y[iCa[ia]+12];
  uz=z[iCa[ia]+13]-z[iCa[ia]+12];
  vx=x[iCa[ia]+14]-x[iCa[ia]+12];
  vy=y[iCa[ia]+14]-y[iCa[ia]+12];
  vz=z[iCa[ia]+14]-z[iCa[ia]+12];
  x[iCa[ia]+15]=x[iCa[ia]+9]-hf*ux;
  y[iCa[ia]+15]=y[iCa[ia]+9]-hf*uy;
  z[iCa[ia]+15]=z[iCa[ia]+9]-hf*uz;
  x[iCa[ia]+16]=x[iCa[ia]+14]-hf*ux;
  y[iCa[ia]+16]=y[iCa[ia]+14]-hf*uy;
  z[iCa[ia]+16]=z[iCa[ia]+14]-hf*uz;
  x[iCa[ia]+17]=x[iCa[ia]+13]+hf*ax;
  y[iCa[ia]+17]=y[iCa[ia]+13]+hf*ay;
  z[iCa[ia]+17]=z[iCa[ia]+13]+hf*az;
  x[iCa[ia]+18]=x[iCa[ia]+14]+hf*ax;
  y[iCa[ia]+18]=y[iCa[ia]+14]+hf*ay;
  z[iCa[ia]+18]=z[iCa[ia]+14]+hf*az;
  x[iCa[ia]+19]=x[iCa[ia]+13]-hf*vx;
  y[iCa[ia]+19]=y[iCa[ia]+13]-hf*vy;
  z[iCa[ia]+19]=z[iCa[ia]+13]-hf*vz;
}
/****************************************************************************/
void his(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.35,bXH=1.0;
  double th1,th2,thH,chi;
  double ux,uy,uz;
  double vx,vy,vz;
  double cmx,cmy,cmz;
  static double thCb;
  static double cu,cv,cw,cq;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=113.2*pi/180;
    th2=126.0*pi/180;
    thH=108.2*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
    }
    cu=(cos(0.3*pi)+cos(0.1*pi))/sqrt(2*(1+cos(0.6*pi)));
    cv=0.5/sqrt(2*(1-cos(0.6*pi)));
    cw=0.4*(1+cos(0.3*pi)/(cos(0.3*pi)+cos(0.1*pi)));
    ux=0.4*c2*(cos(0.1*pi)+2*cos(0.3*pi));
    cq=(bXH+ux)/ux;
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  i=rt[irt[ia]+1]; j=(i+NDI/2)%NDI;
  add_sc2(alpha2[i],beta2[i],gamma2[i],alpha2[j],beta2[j],gamma2[j],
	  iCa[ia],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+7);
  ux=cu*(x[iCa[ia]+6]+x[iCa[ia]+7]-2*x[iCa[ia]+3]);
  uy=cu*(y[iCa[ia]+6]+y[iCa[ia]+7]-2*y[iCa[ia]+3]);
  uz=cu*(z[iCa[ia]+6]+z[iCa[ia]+7]-2*z[iCa[ia]+3]);
  vx=cv*(x[iCa[ia]+6]-x[iCa[ia]+7]);
  vy=cv*(y[iCa[ia]+6]-y[iCa[ia]+7]);
  vz=cv*(z[iCa[ia]+6]-z[iCa[ia]+7]);
  x[iCa[ia]+8]=x[iCa[ia]+3]+ux+vx;
  y[iCa[ia]+8]=y[iCa[ia]+3]+uy+vy;
  z[iCa[ia]+8]=z[iCa[ia]+3]+uz+vz;
  x[iCa[ia]+9]=x[iCa[ia]+3]+ux-vx;
  y[iCa[ia]+9]=y[iCa[ia]+3]+uy-vy;
  z[iCa[ia]+9]=z[iCa[ia]+3]+uz-vz;
  cmx=x[iCa[ia]+3]+cw*ux;
  cmy=y[iCa[ia]+3]+cw*uy;
  cmz=z[iCa[ia]+3]+cw*uz;
  x[iCa[ia]+10]=cmx+cq*(x[iCa[ia]+6]-cmx);
  y[iCa[ia]+10]=cmy+cq*(y[iCa[ia]+6]-cmy);
  z[iCa[ia]+10]=cmz+cq*(z[iCa[ia]+6]-cmz);
  x[iCa[ia]+11]=cmx+cq*(x[iCa[ia]+7]-cmx);
  y[iCa[ia]+11]=cmy+cq*(y[iCa[ia]+7]-cmy);
  z[iCa[ia]+11]=cmz+cq*(z[iCa[ia]+7]-cmz);
  x[iCa[ia]+12]=cmx+cq*(x[iCa[ia]+8]-cmx);
  y[iCa[ia]+12]=cmy+cq*(y[iCa[ia]+8]-cmy);
  z[iCa[ia]+12]=cmz+cq*(z[iCa[ia]+8]-cmz);
}
/****************************************************************************/
void phe(int iflag,int ia,int ir)
{
  const double c1=1.52,c2=1.39,bXH=1.0;
  double th1,th2,thH,chi;
  double ex,ey,ez;
  double cmx,cmy,cmz;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double cq;
  int i,j,k;

  if (iflag<0) {
    th1=113.7*pi/180;
    th2=120.0*pi/180;
    thH=108.1*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
    }
    cq=(bXH+c2)/c2;
    return;
  }
  if (ir==0) {
      i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
      add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	      alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	      iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  i=rt[irt[ia]+1]; j=(i+NDI/2)%NDI;
  add_sc2(alpha2[i],beta2[i],gamma2[i],alpha2[j],beta2[j],gamma2[j],
	  iCa[ia],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+7);
  ex=(x[iCa[ia]+3]-x[iCa[ia]+2])*c2/c1;
  ey=(y[iCa[ia]+3]-y[iCa[ia]+2])*c2/c1;
  ez=(z[iCa[ia]+3]-z[iCa[ia]+2])*c2/c1;
  x[iCa[ia]+8]=x[iCa[ia]+6]+ex;
  y[iCa[ia]+8]=y[iCa[ia]+6]+ey;
  z[iCa[ia]+8]=z[iCa[ia]+6]+ez;
  x[iCa[ia]+9]=x[iCa[ia]+7]+ex;
  y[iCa[ia]+9]=y[iCa[ia]+7]+ey;
  z[iCa[ia]+9]=z[iCa[ia]+7]+ez;
  cmx=x[iCa[ia]+3]+ex;
  cmy=y[iCa[ia]+3]+ey;
  cmz=z[iCa[ia]+3]+ez;
  x[iCa[ia]+10]=cmx+ex;
  y[iCa[ia]+10]=cmy+ey;
  z[iCa[ia]+10]=cmz+ez;
  x[iCa[ia]+11]=cmx+cq*(x[iCa[ia]+6]-cmx);
  y[iCa[ia]+11]=cmy+cq*(y[iCa[ia]+6]-cmy);
  z[iCa[ia]+11]=cmz+cq*(z[iCa[ia]+6]-cmz);
  x[iCa[ia]+12]=cmx+cq*(x[iCa[ia]+7]-cmx);
  y[iCa[ia]+12]=cmy+cq*(y[iCa[ia]+7]-cmy);
  z[iCa[ia]+12]=cmz+cq*(z[iCa[ia]+7]-cmz);
  x[iCa[ia]+13]=cmx+cq*(x[iCa[ia]+8]-cmx);
  y[iCa[ia]+13]=cmy+cq*(y[iCa[ia]+8]-cmy);
  z[iCa[ia]+13]=cmz+cq*(z[iCa[ia]+8]-cmz);
  x[iCa[ia]+14]=cmx+cq*(x[iCa[ia]+9]-cmx);
  y[iCa[ia]+14]=cmy+cq*(y[iCa[ia]+9]-cmy);
  z[iCa[ia]+14]=cmz+cq*(z[iCa[ia]+9]-cmz);
  x[iCa[ia]+15]=cmx+cq*(x[iCa[ia]+10]-cmx);
  y[iCa[ia]+15]=cmy+cq*(y[iCa[ia]+10]-cmy);
  z[iCa[ia]+15]=cmz+cq*(z[iCa[ia]+10]-cmz);
}
/****************************************************************************/
void tyr(int iflag,int ia,int ir)
{
  const double c1=1.51,c2=1.39,c3=1.38,bXH=1.0;
  double th1,th2,thH,thHn,chi;
  double ex,ey,ez;
  double cmx,cmy,cmz;
  static double thCb;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  static double alpha3H[NDI],beta3H[NDI],gamma3H[NDI];  
  static double cq;
  int i,j,k;

  if (iflag<0) {
    th1=113.6*pi/180;
    th2=120.0*pi/180;
    thH=108.1*pi/180;
    thHn=108.0*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
      alpha3H[i]=bXH*sin(pi-thHn)*sin(chi-pid2)/c2/sin(pi-2*pi/3.0);
      beta3H[i]=cos(pi-thHn)-sin(pi-thHn)*sin(chi-pid2)/tan(pi-2*pi/3.0);
      beta3H[i]*=bXH/c3;
      gamma3H[i]=bXH*sin(pi-thHn)*cos(chi-pid2)/sin(pi-2*pi/3.0)/c2/c3;
      
    }
    cq=(bXH+c2)/c2;
    return;
  }
  if (ir==0) {
      i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
      add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	      alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	      iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  if (ir<=1) {
    i=rt[irt[ia]+1]; j=(i+NDI/2)%NDI;
    add_sc2(alpha2[i],beta2[i],gamma2[i],alpha2[j],beta2[j],gamma2[j],
	    iCa[ia],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+7);
    ex=(x[iCa[ia]+3]-x[iCa[ia]+2])*c2/c1;
    ey=(y[iCa[ia]+3]-y[iCa[ia]+2])*c2/c1;
    ez=(z[iCa[ia]+3]-z[iCa[ia]+2])*c2/c1;
    x[iCa[ia]+8]=x[iCa[ia]+6]+ex;
    y[iCa[ia]+8]=y[iCa[ia]+6]+ey;
    z[iCa[ia]+8]=z[iCa[ia]+6]+ez;
    x[iCa[ia]+9]=x[iCa[ia]+7]+ex;
    y[iCa[ia]+9]=y[iCa[ia]+7]+ey;
    z[iCa[ia]+9]=z[iCa[ia]+7]+ez;
    cmx=x[iCa[ia]+3]+ex;
    cmy=y[iCa[ia]+3]+ey;
    cmz=z[iCa[ia]+3]+ez;
    x[iCa[ia]+10]=cmx+ex;
    y[iCa[ia]+10]=cmy+ey;
    z[iCa[ia]+10]=cmz+ez;
    x[iCa[ia]+11]=x[iCa[ia]+3]+(2+c3/c2)*ex;
    y[iCa[ia]+11]=y[iCa[ia]+3]+(2+c3/c2)*ey;
    z[iCa[ia]+11]=z[iCa[ia]+3]+(2+c3/c2)*ez;
    x[iCa[ia]+12]=cmx+cq*(x[iCa[ia]+6]-cmx);
    y[iCa[ia]+12]=cmy+cq*(y[iCa[ia]+6]-cmy);
    z[iCa[ia]+12]=cmz+cq*(z[iCa[ia]+6]-cmz);
    x[iCa[ia]+13]=cmx+cq*(x[iCa[ia]+7]-cmx);
    y[iCa[ia]+13]=cmy+cq*(y[iCa[ia]+7]-cmy);
    z[iCa[ia]+13]=cmz+cq*(z[iCa[ia]+7]-cmz);
    x[iCa[ia]+14]=cmx+cq*(x[iCa[ia]+8]-cmx);
    y[iCa[ia]+14]=cmy+cq*(y[iCa[ia]+8]-cmy);
    z[iCa[ia]+14]=cmz+cq*(z[iCa[ia]+8]-cmz);
    x[iCa[ia]+15]=cmx+cq*(x[iCa[ia]+9]-cmx);
    y[iCa[ia]+15]=cmy+cq*(y[iCa[ia]+9]-cmy);
    z[iCa[ia]+15]=cmz+cq*(z[iCa[ia]+9]-cmz);
  }
  i=rt[irt[ia]+2];
  add_sc1(alpha3H[i],beta3H[i],gamma3H[i],iCa[ia]+8,iCa[ia]+10,iCa[ia]+11,
	  iCa[ia]+16);
}
/****************************************************************************/
void trp(int iflag,int ia,int ir)
{
  const double c1=1.50,c2=1.39,bXH=1.0;
  double th1,th2,thH,chi;
  double ux,uy,uz;
  double vx,vy,vz;
  double cmx,cmy,cmz;
  static double thCb;
  static double cu,cv,cw,cw5,cq5,cq6;
  static double alpha1[NDI],beta1[NDI],gamma1[NDI];  
  static double alpha1H[NDI],beta1H[NDI],gamma1H[NDI];  
  static double alpha2[NDI],beta2[NDI],gamma2[NDI];  
  int i,j,k;

  if (iflag<0) {
    th1=113.8*pi/180;
    th2=126.0*pi/180;
    thH=108.1*pi/180;
    thCb=110.0*pi/180;
    for (i=0;i<NDI;i++) {
      chi=2*i*pi/NDI; 
      alpha1[i]=c1*sin(pi-th1)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1[i]=cos(pi-th1)-sin(pi-th1)*sin(chi-pid2)/tan(pi-thCb);
      beta1[i]*=c1/bCaCb;
      gamma1[i]=c1*sin(pi-th1)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha1H[i]=bXH*sin(pi-thH)*sin(chi-pid2)/b[1]/sin(pi-thCb);
      beta1H[i]=cos(pi-thH)-sin(pi-thH)*sin(chi-pid2)/tan(pi-thCb);
      beta1H[i]*=bXH/bCaCb;
      gamma1H[i]=bXH*sin(pi-thH)*cos(chi-pid2)/sin(pi-thCb)/b[1]/bCaCb;
      alpha2[i]=c2*sin(pi-th2)*sin(chi-pid2)/bCaCb/sin(pi-th1);
      beta2[i]=cos(pi-th2)-sin(pi-th2)*sin(chi-pid2)/tan(pi-th1);
      beta2[i]*=c2/c1;
      gamma2[i]=c2*sin(pi-th2)*cos(chi-pid2)/sin(pi-th1)/bCaCb/c1;
    }
    cu=(cos(0.3*pi)+cos(0.1*pi))/sqrt(2*(1+cos(0.6*pi)));
    cv=0.5/sqrt(2*(1-cos(0.6*pi)));
    cw=0.25*sqrt(3.0)/(cos(0.3*pi)+cos(0.1*pi));
    cw5=0.4*(1+cos(0.3*pi)/(cos(0.3*pi)+cos(0.1*pi)));
    ux=0.4*c2*(cos(0.1*pi)+2*cos(0.3*pi));
    cq5=(bXH+ux)/ux;
    cq6=(bXH+c2)/c2;
    return;
  }
  if (ir==0) {
    i=rt[irt[ia]]; j=(i+NDI/3)%NDI; k=(j+NDI/3)%NDI;
    add_sc3(alpha1[i],beta1[i],gamma1[i],alpha1H[j],beta1H[j],gamma1H[j],
	    alpha1H[k],beta1H[k],gamma1H[k],iN[ia],iCa[ia],iCa[ia]+2,
	    iCa[ia]+3,iCa[ia]+4,iCa[ia]+5);
  }
  i=rt[irt[ia]+1]; j=(i+NDI/2)%NDI;
  add_sc2(alpha2[i],beta2[i],gamma2[i],alpha2[j],beta2[j],gamma2[j],
	  iCa[ia],iCa[ia]+2,iCa[ia]+3,iCa[ia]+6,iCa[ia]+7);
  ux=cu*(x[iCa[ia]+6]+x[iCa[ia]+7]-2*x[iCa[ia]+3]);
  uy=cu*(y[iCa[ia]+6]+y[iCa[ia]+7]-2*y[iCa[ia]+3]);
  uz=cu*(z[iCa[ia]+6]+z[iCa[ia]+7]-2*z[iCa[ia]+3]);
  vx=cv*(x[iCa[ia]+6]-x[iCa[ia]+7]);
  vy=cv*(y[iCa[ia]+6]-y[iCa[ia]+7]);
  vz=cv*(z[iCa[ia]+6]-z[iCa[ia]+7]);
  x[iCa[ia]+8]=x[iCa[ia]+3]+ux+vx;
  y[iCa[ia]+8]=y[iCa[ia]+3]+uy+vy;
  z[iCa[ia]+8]=z[iCa[ia]+3]+uz+vz;
  x[iCa[ia]+9]=x[iCa[ia]+3]+ux-vx;
  y[iCa[ia]+9]=y[iCa[ia]+3]+uy-vy;
  z[iCa[ia]+9]=z[iCa[ia]+3]+uz-vz;
  cmx=x[iCa[ia]+3]+cw5*ux;
  cmy=y[iCa[ia]+3]+cw5*uy;
  cmz=z[iCa[ia]+3]+cw5*uz;
  x[iCa[ia]+14]=cmx+cq5*(x[iCa[ia]+6]-cmx);
  y[iCa[ia]+14]=cmy+cq5*(y[iCa[ia]+6]-cmy);
  z[iCa[ia]+14]=cmz+cq5*(z[iCa[ia]+6]-cmz);
  x[iCa[ia]+15]=cmx+cq5*(x[iCa[ia]+8]-cmx);
  y[iCa[ia]+15]=cmy+cq5*(y[iCa[ia]+8]-cmy);
  z[iCa[ia]+15]=cmz+cq5*(z[iCa[ia]+8]-cmz);
  ux=x[iCa[ia]+9]-x[iCa[ia]+7];
  uy=y[iCa[ia]+9]-y[iCa[ia]+7];
  uz=z[iCa[ia]+9]-z[iCa[ia]+7];
  vx=cw*(x[iCa[ia]+9]+x[iCa[ia]+7]-2*x[iCa[ia]+6]);
  vy=cw*(y[iCa[ia]+9]+y[iCa[ia]+7]-2*y[iCa[ia]+6]);
  vz=cw*(z[iCa[ia]+9]+z[iCa[ia]+7]-2*z[iCa[ia]+6]);
  x[iCa[ia]+10]=x[iCa[ia]+7]-0.5*ux+vx;
  y[iCa[ia]+10]=y[iCa[ia]+7]-0.5*uy+vy;
  z[iCa[ia]+10]=z[iCa[ia]+7]-0.5*uz+vz;
  cmx=x[iCa[ia]+7]+0.5*ux+vx;
  cmy=y[iCa[ia]+7]+0.5*uy+vy;
  cmz=z[iCa[ia]+7]+0.5*uz+vz;
  x[iCa[ia]+11]=cmx+ux;
  y[iCa[ia]+11]=cmy+uy;
  z[iCa[ia]+11]=cmz+uz;
  x[iCa[ia]+12]=x[iCa[ia]+7]+2*vx;
  y[iCa[ia]+12]=y[iCa[ia]+7]+2*vy;
  z[iCa[ia]+12]=z[iCa[ia]+7]+2*vz;
  x[iCa[ia]+13]=x[iCa[ia]+7]+ux+2*vx;
  y[iCa[ia]+13]=y[iCa[ia]+7]+uy+2*vy;
  z[iCa[ia]+13]=z[iCa[ia]+7]+uz+2*vz;
  x[iCa[ia]+16]=cmx+cq6*(x[iCa[ia]+10]-cmx);
  y[iCa[ia]+16]=cmy+cq6*(y[iCa[ia]+10]-cmy);
  z[iCa[ia]+16]=cmz+cq6*(z[iCa[ia]+10]-cmz);
  x[iCa[ia]+17]=cmx+cq6*(x[iCa[ia]+11]-cmx);
  y[iCa[ia]+17]=cmy+cq6*(y[iCa[ia]+11]-cmy);
  z[iCa[ia]+17]=cmz+cq6*(z[iCa[ia]+11]-cmz);
  x[iCa[ia]+18]=cmx+cq6*(x[iCa[ia]+12]-cmx);
  y[iCa[ia]+18]=cmy+cq6*(y[iCa[ia]+12]-cmy);
  z[iCa[ia]+18]=cmz+cq6*(z[iCa[ia]+12]-cmz);
  x[iCa[ia]+19]=cmx+cq6*(x[iCa[ia]+13]-cmx);
  y[iCa[ia]+19]=cmy+cq6*(y[iCa[ia]+13]-cmy);
  z[iCa[ia]+19]=cmz+cq6*(z[iCa[ia]+13]-cmz);
}
/****************************************************************************/
void add_sc1(double alpha,double beta,double gamma,
             int i1,int i2,int i3,int i4)
{
  double ax,ay,az;
  double bx,by,bz;
  double ux,uy,uz;
  ax=x[i2]-x[i1]; ay=y[i2]-y[i1]; az=z[i2]-z[i1];
  bx=x[i3]-x[i2]; by=y[i3]-y[i2]; bz=z[i3]-z[i2];
  ux=ay*bz-az*by; uy=az*bx-ax*bz; uz=ax*by-ay*bx;
  x[i4]=x[i3]+alpha*ax+beta*bx+gamma*ux;
  y[i4]=y[i3]+alpha*ay+beta*by+gamma*uy;
  z[i4]=z[i3]+alpha*az+beta*bz+gamma*uz;
}
/****************************************************************************/
void add_sc2(double alpha1,double beta1,double gamma1,
	     double alpha2,double beta2,double gamma2,
             int i1,int i2,int i3,int i4,int i5)
{
  double ax,ay,az;
  double bx,by,bz;
  double ux,uy,uz;
  ax=x[i2]-x[i1]; ay=y[i2]-y[i1]; az=z[i2]-z[i1];
  bx=x[i3]-x[i2]; by=y[i3]-y[i2]; bz=z[i3]-z[i2];
  ux=ay*bz-az*by; uy=az*bx-ax*bz; uz=ax*by-ay*bx;
  x[i4]=x[i3]+alpha1*ax+beta1*bx+gamma1*ux;
  y[i4]=y[i3]+alpha1*ay+beta1*by+gamma1*uy;
  z[i4]=z[i3]+alpha1*az+beta1*bz+gamma1*uz;
  x[i5]=x[i3]+alpha2*ax+beta2*bx+gamma2*ux;
  y[i5]=y[i3]+alpha2*ay+beta2*by+gamma2*uy;
  z[i5]=z[i3]+alpha2*az+beta2*bz+gamma2*uz;
}
/****************************************************************************/
void add_sc3(double alpha1,double beta1,double gamma1,
	     double alpha2,double beta2,double gamma2,
	     double alpha3,double beta3,double gamma3,
             int i1,int i2,int i3,int i4,int i5,int i6)
{
  double ax,ay,az;
  double bx,by,bz;
  double ux,uy,uz;
  ax=x[i2]-x[i1]; ay=y[i2]-y[i1]; az=z[i2]-z[i1];
  bx=x[i3]-x[i2]; by=y[i3]-y[i2]; bz=z[i3]-z[i2];
  ux=ay*bz-az*by; uy=az*bx-ax*bz; uz=ax*by-ay*bx;
  x[i4]=x[i3]+alpha1*ax+beta1*bx+gamma1*ux;
  y[i4]=y[i3]+alpha1*ay+beta1*by+gamma1*uy;
  z[i4]=z[i3]+alpha1*az+beta1*bz+gamma1*uz;
  x[i5]=x[i3]+alpha2*ax+beta2*bx+gamma2*ux;
  y[i5]=y[i3]+alpha2*ay+beta2*by+gamma2*uy;
  z[i5]=z[i3]+alpha2*az+beta2*bz+gamma2*uz;
  x[i6]=x[i3]+alpha3*ax+beta3*bx+gamma3*ux;
  y[i6]=y[i3]+alpha3*ay+beta3*by+gamma3*uy;
  z[i6]=z[i3]+alpha3*az+beta3*bz+gamma3*uz;
}
/****************************************************************************/
/***** ENERGIES *************************************************************/
/****************************************************************************/
double bias(int iflag)
{
  int i,j,k;
  double tmp,e=0;
  static double cst;                                      

  if (iflag<0) {
    cst=1.0/bCO/bCO;
    return 0.0;
  }

  for (i=0;i<N-1;i++) {
    j=iC[i]; k=iC[i+1]; 
    tmp=(x[j+1]-x[j])*(x[k+1]-x[k])+
        (y[j+1]-y[j])*(y[k+1]-y[k])+
        (z[j+1]-z[j])*(z[k+1]-z[k]);
    tmp*=cst; 
//    if (i<5 || i>8) {
      if (tmp<0) 
	e+=kbb*tmp;
      else
	e-=kba*tmp;
//    }
//    else if (i==6 || i==8) {
//      if (tmp>0) e-=0.75*tmp;
//    }
  }
  return e;
}
/****************************************************************************/
double hbonds(int iflag)
{
// assumes that all H have bond length 1.00
// assumes that all OC acceptor pairs except in mainchain have bond length 1.25
  extern double hb_pair(int iflag,int d1,int d2,int a1,int a2);
  extern double hb_sidemain(int i);
  extern double hb_sideside(int i);
  int i,j,d1,d2,a1,a2;
  double e=0;

  if (iflag<0) {
    hb_pair(-1,0,0,0,0);
    hb_sideside(-1);
    return 0.0; 
  }

  for (i=0;i<N;i++) {
    if (seq[i]=='P') continue;
    d1=iN[i]; d2=d1+1; 
    for (j=0;j<N;j++) {
      if (j>=i-2 && j<=i+1) continue;
      a2=iC[j]; a1=a2+1;
      e+=hb_pair(0,d1,d2,a1,a2);
    }
  }
  e*=epshb;

  for (i=0;i<N;i++) e+=epshb2*hb_sidemain(i);

  e+=epshb2*hb_sideside(0);

  return e;
}
/****************************************************************************/
double hb_sideside(int iflag)
{
  extern double hb_pair(int iflag,int d1,int d2,int a1,int a2);
  static int d1[max(1,NDON)],d2[max(1,NDON)],daa[max(1,NDON)];
  static int a1[max(1,NACC)],a2[max(1,NACC)],aaa[max(1,NACC)];
  int i,nacc,ndon;
  double e=0;
  
  if (iflag<0) {
    ndon=nacc=0;
    for (i=0;i<N;i++) {
      switch (seq[i]){ 
      case 'D' : {
        a1[nacc]=iCa[i]+6; a2[nacc]=iCa[i]+3; aaa[nacc++]=i;
        a1[nacc]=iCa[i]+7; a2[nacc]=iCa[i]+3; aaa[nacc++]=i;   
	break;
      }
      case 'E' : {
	a1[nacc]=iCa[i]+9; a2[nacc]=iCa[i]+6; aaa[nacc++]=i;
	a1[nacc]=iCa[i]+10; a2[nacc]=iCa[i]+6; aaa[nacc++]=i;   
	break;
      }
      case 'K' : {
	d1[ndon]=iCa[i]+12; d2[ndon]=iCa[i]+15; daa[ndon++]=i;
	d1[ndon]=iCa[i]+12; d2[ndon]=iCa[i]+16; daa[ndon++]=i;
	d1[ndon]=iCa[i]+12; d2[ndon]=iCa[i]+17; daa[ndon++]=i;
	break;
      }
      case 'R' : {
	d1[ndon]=iCa[i]+9; d2[ndon]=iCa[i]+15; daa[ndon++]=i;
	d1[ndon]=iCa[i]+13; d2[ndon]=iCa[i]+17; daa[ndon++]=i;
	d1[ndon]=iCa[i]+13; d2[ndon]=iCa[i]+19; daa[ndon++]=i;
	d1[ndon]=iCa[i]+14; d2[ndon]=iCa[i]+16; daa[ndon++]=i;
	d1[ndon]=iCa[i]+14; d2[ndon]=iCa[i]+18; daa[ndon++]=i;
	break;
      }
      default : {;}
      }
    }
    if (ndon!=NDON) {printf("ndon=%i\n",ndon); exit(-1);}
    if (nacc!=NACC) {printf("nacc=%i\n",nacc); exit(-1);}
    return 0.0;
  }
 
  for (ndon=0;ndon<NDON;ndon++) {
    i=daa[ndon];
    for (nacc=0;nacc<NACC;nacc++) {
      if (abs(i-aaa[nacc])<=1) continue;
      e+=hb_pair(1,d1[ndon],d2[ndon],a1[nacc],a2[nacc]);
    }
  }

  return e;
}
/****************************************************************************/
double hb_sidemain(int i)
{
  extern double acceptormain(int i,int a1,int a2);
  extern double donatormain(int i,int d1,int d2);
  double e=0;

  switch (seq[i]){ 
  case 'G' : {break;}
  case 'A' : {break;}
  case 'V' : {break;}
  case 'L' : {break;}
  case 'I' : {break;}
  case 'S' : {break;}
  case 'T' : {break;}
  case 'M' : {break;}
  case 'P' : {break;}
  case 'D' : {
    e+=acceptormain(i,iCa[i]+6,iCa[i]+3); 
    e+=acceptormain(i,iCa[i]+7,iCa[i]+3);   
    break;
  }
  case 'N' : {break;}
  case 'E' : {
    e+=acceptormain(i,iCa[i]+9,iCa[i]+6); 
    e+=acceptormain(i,iCa[i]+10,iCa[i]+6);   
    break;
  }
  case 'Q' : {break;}
  case 'K' : {
    e+=donatormain(i,iCa[i]+12,iCa[i]+15); 
    e+=donatormain(i,iCa[i]+12,iCa[i]+16);
    e+=donatormain(i,iCa[i]+12,iCa[i]+17);  
    break;
  }
  case 'R' : {
    e+=donatormain(i,iCa[i]+9,iCa[i]+15); 
    e+=donatormain(i,iCa[i]+13,iCa[i]+17);
    e+=donatormain(i,iCa[i]+13,iCa[i]+19);  
    e+=donatormain(i,iCa[i]+14,iCa[i]+16);
    e+=donatormain(i,iCa[i]+14,iCa[i]+18);    
    break;
  }
  case 'H' : {break;}
  case 'F' : {break;}
  case 'Y' : {break;}
  case 'W' : {break;}
  default : {printf("error in sidemain\n");exit(-1);}
  }

  return e;
}
/****************************************************************************/
double acceptormain(int j,int a1,int a2)
{
  extern double hb_pair(int iflag,int d1,int d2,int a1,int a2);
  int i,d1,d2;
  double e=0;
  for (i=0;i<N;i++) {
    if (seq[i]=='P') continue;
    if (abs(i-j)<=1) continue;
    d1=iN[i]; d2=d1+1; 
    e+=hb_pair(1,d1,d2,a1,a2);
  }
  return e;
}
/****************************************************************************/
double donatormain(int j,int d1,int d2)
{
  extern double hb_pair(int iflag,int d1,int d2,int a1,int a2);
  int i,a1,a2;
  double e=0;
  for (i=0;i<N;i++) {
    if (abs(i-j)<=1) continue;
    a2=iC[i]; a1=a2+1; 
    e+=hb_pair(0,d1,d2,a1,a2); 
  }
  return e;
}
/****************************************************************************/
double hb_pair(int iflag,int d1,int d2,int a1,int a2)
{
  double e;
  double r2,r4,r6;
  double dx,dy,dz,ca,cb;
  static double cut2,cst[2];                                      
  static double ahb,bhb,sighb2;  

  if (iflag<0) {
    cut2=cuthb*cuthb;
    bhb=-30*(pow(sighb/cuthb,10.0)-pow(sighb/cuthb,12.0))/cut2;
    ahb=-(5*pow(sighb/cuthb,12.0)-6*pow(sighb/cuthb,10.0))-bhb*cut2;
    sighb2=sighb*sighb;
    cst[0]=pow(1.0/bNH,POWA)*pow(1.0/bCO,POWB);
    cst[1]=pow(1.0/bNH,POWA)*pow(1.0/1.25,POWB);
    return 0.0;
  }

  if ((r2=(x[a1]-x[d2])*(x[a1]-x[d2])+
          (y[a1]-y[d2])*(y[a1]-y[d2])+
          (z[a1]-z[d2])*(z[a1]-z[d2]))>cut2) return 0.0;
  dx=x[a1]-x[d2]; dy=y[a1]-y[d2]; dz=z[a1]-z[d2];
  if (POWA>0 &&
      (ca=(x[d2]-x[d1])*dx+(y[d2]-y[d1])*dy+(z[d2]-z[d1])*dz)<0) return 0.0; 
  if (POWB>0 && 
      (cb=(x[a2]-x[a1])*dx+(y[a2]-y[a1])*dy+(z[a2]-z[a1])*dz)<0) return 0.0;
  r6=sighb2/r2; r4=r6*r6; r6=r6*r4;   
  e=cst[iflag]*pow(ca*ca/r2,0.5*POWA)*pow(cb*cb/r2,0.5*POWB)*
    (r6*(5*r6-6*r4)+ahb+bhb*r2);

  return e;
}
/****************************************************************************/
double sac(int iflag,double emax)
{
  /* iflag<0 initialize                               */
  /* iflag>0 calculate the full energy                */
  /* iflag=0 calculate the energy but stop if e>emax  */
  extern double cellcalc(int inc,int nl,short list[]);
  extern double localsa(int iflag);
  int nec=0;
  int nx,ny,nz,nc;
  int i,j,ix,iy,iz,ic;
  int h3,h4,h5,h6,h7,h8,h9,h10,h11,h12,h13;
  int lx[NTO],ly[NTO],lz[NTO],lc[NTO];
  int in_cell,nl;
  double xmin,ymin,zmin;
  double xmax,ymax,zmax;
  double e,etest;
  short a,list[NTO];
  short pnt[NTO];                  /* pointers to atoms                     */
  static short cell[MAXCELL];      /* division into cells                   */
  static double cutg;

   if (iflag<0) {
    for (i=0;i<MAXCELL;i++) cell[i]=-1;
    cutg=ALPHA1*cut;
    cellcalc(-1,0,list);
    localsa(-1);
    return 0.0;
  }

  etest=emax/ksa;
  e=localsa(0);
  if (e>=etest && iflag==0) return ksa*e;

  xmin=xmax=x[0]; ymin=ymax=y[0]; zmin=zmax=z[0];
  for (i=1;i<NTO;i++) {
    if (x[i]<xmin) xmin=x[i]; if (x[i]>xmax) xmax=x[i];
    if (y[i]<ymin) ymin=y[i]; if (y[i]>ymax) ymax=y[i];
    if (z[i]<zmin) zmin=z[i]; if (z[i]>zmax) zmax=z[i];
  }
  nx=(xmax-xmin)/cutg+1; ny=(ymax-ymin)/cutg+1; nz=(zmax-zmin)/cutg+1;
  nc=nx*ny*nz;
  if (nc>MAXCELL) {printf("nc=%i TOO BIG, nx=%i ny=%i nz=%i\n",nc,nx,ny,nz);
                   write_conf(0,CONF,0); exit(-1);}
  for (i=NTO-1;i>=0;i--) {
    ix=(x[i]-xmin)/cutg; iy=(y[i]-ymin)/cutg; iz=(z[i]-zmin)/cutg;
    ic=ix+nx*(iy+ny*iz);
    pnt[i]=cell[ic];
    if (cell[ic]<0) {lx[nec]=ix; ly[nec]=iy; lz[nec]=iz; lc[nec++]=ic;}
    cell[ic]=i;
  }

  h3=nx*ny; h4=1+nx; h5=h3+1; h6=h3+nx; h7=1-nx; h8=1-h3; h9=nx-h3;
  h10=h4+h3; h11=h4-h3; h12=h3+h7; h13=h3-h7;

  i=0; 
  while (i<nec && (e<etest || iflag>0)) {
    nl=0;
    list[nl++]=a=cell[(j=lc[i])]; 
    while ((a=pnt[a])>=0) {list[nl++]=a;}
    in_cell=nl;
    if (lx[i]+1<nx && (a=cell[j+1])>=0) { 
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (ly[i]+1<ny && (a=cell[j+nx])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lz[i]+1<nz && (a=cell[j+h3])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]+1<nx && ly[i]+1<ny && (a=cell[j+h4])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]+1<nx && lz[i]+1<nz && (a=cell[j+h5])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (ly[i]+1<ny && lz[i]+1<nz && (a=cell[j+h6])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]+1<nx && ly[i]>0 && (a=cell[j+h7])>=0) { 
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]+1<nx && lz[i]>0 && (a=cell[j+h8])>=0) {  
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (ly[i]+1<ny && lz[i]>0 && (a=cell[j+h9])>=0) {  
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]+1<nx && ly[i]+1<ny && lz[i]+1<nz && (a=cell[j+h10])>=0) { 
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]+1<nx && ly[i]+1<ny && lz[i]>0 && (a=cell[j+h11])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]+1<nx && ly[i]>0 && lz[i]+1<nz && (a=cell[j+h12])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    if (lx[i]>0 && ly[i]+1<ny && lz[i]+1<nz && (a=cell[j+h13])>=0) {
      list[nl++]=a; while ((a=pnt[a])>=0) list[nl++]=a;
    }
    e+=cellcalc(in_cell,nl,list);
    i++;
  }

  for (i=0;i<nec;i++) cell[lc[i]]=-1;
  return ksa*e;
}
/****************************************************************************/
double localsa(int iflag)
{
  int a,i,j,k;
  double r2,r6,e=0;
  static int npair;
  static int i1[MAXLOC],i2[MAXLOC],id[MAXLOC];
  static double cut2;
  static double sig2[3*NA*NA],asa[3*NA*NA],bsa[3*NA*NA];

  if (iflag<0) {
    npair=0;
    for (i=0;i<NTO;i++) {
      for (j=i+1;j<NTO;j++) {
        if ((a=aa[i][j])>=NA*NA) {
          i1[npair]=i;
          i2[npair]=j;
          id[npair++]=a;
	}
      }
    }
    if (npair>MAXLOC) {printf("%i -- too many local pairs\n",npair);exit(-1);}
    cut2=cut*cut;
    for (i=0;i<NA;i++) {
      for (j=0;j<NA;j++) {
	sig2[i+j*NA+NA*NA]=pow(sigsa[i]+sigsa[j],2.0);
	sig2[i+j*NA+2*NA*NA]=pow(ALPHA2*(sigsa[i]+sigsa[j]),2.0);
      }
    }
    for (i=NA*NA;i<3*NA*NA;i++) {
      asa[i]=-7*pow(sig2[i]/cut2,6.0);
      bsa[i]=6*pow(sig2[i]/cut2,6.0)/cut2;
    } 
    return 0.0;
  }
 
  for (k=0;k<npair;k++) {
    i=i1[k]; j=i2[k];
    if ((r2=(x[i]-x[j])*(x[i]-x[j])+
	    (y[i]-y[j])*(y[i]-y[j])+
	    (z[i]-z[j])*(z[i]-z[j]))>cut2) continue;
    r6=sig2[(a=id[k])]/r2; r6=r6*r6*r6;
    e+=r6*r6+asa[a]+bsa[a]*r2;
  }
  return e;
}
/****************************************************************************/
double cellcalc(int inc,int nl,short list[])
{
  int i,j,n,m,a;
  double e=0;
  double r2,r6;
  static double cutg2;
  static double sig2[NA*NA],asa[NA*NA],bsa[NA*NA];

  if (inc<0) {
    cutg2=ALPHA1*ALPHA1*cut*cut;
    for (i=0;i<NA;i++) {
      for (j=0;j<NA;j++) {
	sig2[i+j*NA]=pow(ALPHA1*(sigsa[i]+sigsa[j]),2.0);
      }
    }
    for (i=0;i<NA*NA;i++) {
      asa[i]=-7*pow(sig2[i]/cutg2,6.0);
      bsa[i]=6*pow(sig2[i]/cutg2,6.0)/cutg2;
    } 
    return 0.0;
  }

  for (n=0;n<inc;n++) {
    i=list[n];
    for (m=n+1;m<nl;m++) {
      j=list[m];
      if ((r2=(x[i]-x[j])*(x[i]-x[j])+
              (y[i]-y[j])*(y[i]-y[j])+
              (z[i]-z[j])*(z[i]-z[j]))>cutg2) continue;
      if ((a=aa[i][j])<0 || a>=NA*NA) continue;
      r6=sig2[a]/r2; r6=r6*r6*r6;
      e+=r6*r6+asa[a]+bsa[a]*r2;
    }
  }
  return e;
}
/****************************************************************************/
double hp(int iflag) 
{
  extern double hp_pair(int iflag,int i,int j);
  int i,j;
  double e=0;
  static int omit[max(1,NHAA)];// omit neighbor

  if (iflag<0) {
    hp_pair(-1,0,0); 
    for (i=0;i<NHAA;i++) omit[i]=0;
    for (i=0;i<N-1;i++) {
      if ((j=ih[i])<0) continue;
      if (ih[i+1]==j+1) omit[j]=1;
    }
    return 0;
  }

  for (i=0;i<NHAA;i++) {
    for (j=i+1;j<NHAA;j++) { 
      if (j==i+1 && omit[i]==1) continue;
      e+=hp_pair(0,i,j);
    }
  }
  return epshp*e;

}
/****************************************************************************/
double hp_pair(int iflag,int i,int j)
{
  int n,m,k,l;
  int ni,nj;
  double sum=0,r2min[12];
  double tmp;
  static double a2,b2,d2;
  static double strength[max(1,NHAA)][max(1,NHAA)];
  static int atom[max(1,NHAA)][6];
  static int natom[max(1,NHAA)];
  if (iflag<0) {
    a2=ahp*ahp; b2=bhp*bhp; d2=b2-a2;
    for (k=0;k<N;k++) {
      if ((n=ih[k])<0) continue;
      for (l=k+1;l<N;l++) {
	if ((m=ih[l])<0) continue;
        if (seq[k]=='A' && seq[l]=='A') 
	  strength[n][m]=0.00;
        else if ((seq[k]=='A' && seq[l]=='V') || (seq[k]=='V' && seq[l]=='A'))
          strength[n][m]=0.44;
        else if ((seq[k]=='A' && seq[l]=='L') || (seq[k]=='L' && seq[l]=='A'))
          strength[n][m]=1.31;
        else if ((seq[k]=='A' && seq[l]=='I') || (seq[k]=='I' && seq[l]=='A'))
          strength[n][m]=0.98;
        else if ((seq[k]=='A' && seq[l]=='M') || (seq[k]=='M' && seq[l]=='A'))
          strength[n][m]=0.34;
        else if ((seq[k]=='A' && seq[l]=='F') || (seq[k]=='F' && seq[l]=='A'))
          strength[n][m]=1.21;
        else if ((seq[k]=='A' && seq[l]=='Y') || (seq[k]=='Y' && seq[l]=='A'))
          strength[n][m]=0.00;
        else if ((seq[k]=='A' && seq[l]=='W') || (seq[k]=='W' && seq[l]=='A'))
          strength[n][m]=0.22;

        else if (seq[k]=='V' && seq[l]=='V') 
	  strength[n][m]=1.92;
        else if ((seq[k]=='V' && seq[l]=='L') || (seq[k]=='L' && seq[l]=='V'))
          strength[n][m]=2.88;
        else if ((seq[k]=='V' && seq[l]=='I') || (seq[k]=='I' && seq[l]=='V'))
          strength[n][m]=2.45;
        else if ((seq[k]=='V' && seq[l]=='M') || (seq[k]=='M' && seq[l]=='V'))
          strength[n][m]=1.72;
        else if ((seq[k]=='V' && seq[l]=='F') || (seq[k]=='F' && seq[l]=='V'))
          strength[n][m]=2.69;
        else if ((seq[k]=='V' && seq[l]=='Y') || (seq[k]=='Y' && seq[l]=='V'))
          strength[n][m]=1.02;
        else if ((seq[k]=='V' && seq[l]=='W') || (seq[k]=='W' && seq[l]=='V'))
          strength[n][m]=1.58;

        else if (seq[k]=='L' && seq[l]=='L') 
	  strength[n][m]=3.77;
        else if ((seq[k]=='L' && seq[l]=='I') || (seq[k]=='I' && seq[l]=='L'))
          strength[n][m]=3.44;
        else if ((seq[k]=='L' && seq[l]=='M') || (seq[k]=='M' && seq[l]=='L'))
          strength[n][m]=2.81;
        else if ((seq[k]=='L' && seq[l]=='F') || (seq[k]=='F' && seq[l]=='L'))
          strength[n][m]=3.68;
        else if ((seq[k]=='L' && seq[l]=='Y') || (seq[k]=='Y' && seq[l]=='L'))
          strength[n][m]=2.07;
        else if ((seq[k]=='L' && seq[l]=='W') || (seq[k]=='W' && seq[l]=='L'))
          strength[n][m]=2.54;

        else if (seq[k]=='I' && seq[l]=='I')
	  strength[n][m]=2.94;
        else if ((seq[k]=='I' && seq[l]=='M') || (seq[k]=='M' && seq[l]=='I'))
          strength[n][m]=2.42;
        else if ((seq[k]=='I' && seq[l]=='F') || (seq[k]=='F' && seq[l]=='I'))
          strength[n][m]=3.24;
        else if ((seq[k]=='I' && seq[l]=='Y') || (seq[k]=='Y' && seq[l]=='I'))
          strength[n][m]=1.65;
        else if ((seq[k]=='I' && seq[l]=='W') || (seq[k]=='W' && seq[l]=='I'))
          strength[n][m]=2.18;

        else if (seq[k]=='M' && seq[l]=='M')
	  strength[n][m]=1.86;
        else if ((seq[k]=='M' && seq[l]=='F') || (seq[k]=='F' && seq[l]=='M'))
          strength[n][m]=2.96;
        else if ((seq[k]=='M' && seq[l]=='Y') || (seq[k]=='Y' && seq[l]=='M'))
          strength[n][m]=1.31;
        else if ((seq[k]=='M' && seq[l]=='W') || (seq[k]=='W' && seq[l]=='M'))
          strength[n][m]=1.95;

        else if (seq[k]=='F' && seq[l]=='F')
	  strength[n][m]=3.66;
        else if ((seq[k]=='F' && seq[l]=='Y') || (seq[k]=='Y' && seq[l]=='F'))
          strength[n][m]=2.06;
        else if ((seq[k]=='F' && seq[l]=='W') || (seq[k]=='W' && seq[l]=='F'))
          strength[n][m]=2.56;

        else if (seq[k]=='Y' && seq[l]=='Y')
	  strength[n][m]=0.57;
        else if ((seq[k]=='Y' && seq[l]=='W') || (seq[k]=='W' && seq[l]=='Y'))
          strength[n][m]=1.06;

        else if (seq[k]=='W' && seq[l]=='W')
	  strength[n][m]=1.46;

        else {printf("Error 1 in hp_pair\n"); exit(-1);}
        strength[m][n]=strength[n][m];
      }
    }
    
    for (k=0;k<N;k++) {
      if ((n=ih[k])<0) continue;
      if (seq[k]=='A') {
        atom[n][0]=iCa[k]+2;
        natom[n]=1;
      }
      else if (seq[k]=='V') {
        atom[n][0]=iCa[k]+2;
        atom[n][1]=iCa[k]+3;
        atom[n][2]=iCa[k]+4;
        natom[n]=3;
      }
      else if (seq[k]=='L'){ 
	atom[n][0]=iCa[k]+2;
        atom[n][1]=iCa[k]+3;
        atom[n][2]=iCa[k]+6;
        atom[n][3]=iCa[k]+7;
        natom[n]=4;
      }
      else if (seq[k]=='I'){ 
        atom[n][0]=iCa[k]+2;
        atom[n][1]=iCa[k]+3;
        atom[n][2]=iCa[k]+4;
        atom[n][3]=iCa[k]+6;
        natom[n]=4;
      }
      else if (seq[k]=='M'){ 
        atom[n][0]=iCa[k]+2;
        atom[n][1]=iCa[k]+3;
        atom[n][2]=iCa[k]+6;
        atom[n][3]=iCa[k]+9;
        natom[n]=4;
      }
      else if (seq[k]=='F'){ 
        atom[n][0]=iCa[k]+3; 
        atom[n][1]=iCa[k]+6;
        atom[n][2]=iCa[k]+8;
        atom[n][3]=iCa[k]+10; 
        atom[n][4]=iCa[k]+9;
        atom[n][5]=iCa[k]+7;
        natom[n]=6;
      }
      else if (seq[k]=='Y') {
        atom[n][0]=iCa[k]+3; 
        atom[n][1]=iCa[k]+6;
        atom[n][2]=iCa[k]+7;
        atom[n][3]=iCa[k]+10; 
        atom[n][4]=iCa[k]+9;
        atom[n][5]=iCa[k]+7;
        natom[n]=6;
      }  
      else if (seq[k]=='W') {
        atom[n][0]=iCa[k]+7; 
        atom[n][1]=iCa[k]+10;
        atom[n][2]=iCa[k]+12;
        atom[n][3]=iCa[k]+13; 
        atom[n][4]=iCa[k]+11;
        atom[n][5]=iCa[k]+9;
        natom[n]=6;
      }  
      else
        {printf("Error 2 in hp_pair\n"); exit(-1);}
    }
    return 0;
  }

  ni=natom[i]; nj=natom[j];
  for (n=0;n<ni+nj;n++) r2min[n]=1e10; 
  for (n=0;n<ni;n++) {
    k=atom[i][n];
    for (m=0;m<nj;m++) {
      l=atom[j][m];
      tmp=(x[k]-x[l])*(x[k]-x[l])+
	  (y[k]-y[l])*(y[k]-y[l])+
	  (z[k]-z[l])*(z[k]-z[l]);
      if (tmp<r2min[n]) r2min[n]=tmp;
      if (tmp<r2min[ni+m]) r2min[ni+m]=tmp;
    }
  }
  for (n=0;n<ni+nj;n++) {
    if (r2min[n]>b2) continue;
    if (r2min[n]<a2) 
      sum++;
    else
      sum+=(b2-r2min[n])/d2;
  }
  return -strength[i][j]*sum/(ni+nj);
}
/****************************************************************************/
/***** UPDATES **************************************************************/
/****************************************************************************/
int bgs(void)
{
  int i,j,k,ia,ib,iph[8],nph=0;
  double e1,e2,e3,e4,emax;
  double ix[8],iy[8],iz[8],bx[8],by[8],bz[8],ab[8];
  double rx[3],ry[3],rz[3];
  double dx[3][8],dy[3][8],dz[3][8];
  double A[8][8],p[8],psi[8],dph[8],sum,r1,r2;
  double wfw,wbw;

  ia=(N-4)*ran3n(&seed);
  ib=iCa[ia]+1; 

  /** forward move **/

  for (i=0;i<4;i++){
    if (seq[ia+i]!='P') {
      ix[nph]=x[iCa[ia+i]];
      iy[nph]=y[iCa[ia+i]];
      iz[nph]=z[iCa[ia+i]];
      bx[nph]=x[iCa[ia+i]]-x[iN[ia+i]];
      by[nph]=y[iCa[ia+i]]-y[iN[ia+i]];
      bz[nph]=z[iCa[ia+i]]-z[iN[ia+i]];
      ab[nph]=b[1];
      iph[nph]=3*(ia+i);
      nph++;
    }
    ix[nph]=x[iC[ia+i]];
    iy[nph]=y[iC[ia+i]];
    iz[nph]=z[iC[ia+i]];
    bx[nph]=x[iC[ia+i]]-x[iCa[ia+i]];
    by[nph]=y[iC[ia+i]]-y[iCa[ia+i]];
    bz[nph]=z[iC[ia+i]]-z[iCa[ia+i]];
    ab[nph]=b[2];
    iph[nph]=3*(ia+i)+1;
    nph++;
  } 
  i=iCa[ia+3]; rx[0]=x[i]; ry[0]=y[i]; rz[0]=z[i];
  i=iC[ia+3]; rx[1]=x[i]; ry[1]=y[i]; rz[1]=z[i];
  i++; rx[2]=x[i]; ry[2]=y[i]; rz[2]=z[i];
  for (i=0;i<3;i++) {
    for (j=0;j<nph;j++) {
      dx[i][j]=(by[j]*(rz[i]-iz[j])-bz[j]*(ry[i]-iy[j]))/ab[j];
      dy[i][j]=(bz[j]*(rx[i]-ix[j])-bx[j]*(rz[i]-iz[j]))/ab[j];
      dz[i][j]=(bx[j]*(ry[i]-iy[j])-by[j]*(rx[i]-ix[j]))/ab[j];
    }
  }
  for (i=0;i<nph;i++) {
    for (j=i;j<nph;j++) {
      A[i][j]=0;
      for (k=0;k<3;k++) {
        A[i][j]+=dx[k][i]*dx[k][j]+dy[k][i]*dy[k][j]+dz[k][i]*dz[k][j];
      }
      A[i][j]*=bbgs;
      if (i==j) A[i][j]+=1;
      A[i][j]*=abgs/2;
    }
  }
  for (i=0;i<nph;i++) {
    for (j=i;j<nph;j++) {
      for (sum=A[i][j],k=i-1;k>=0;k--) sum-=A[i][k]*A[j][k];
      if (i==j)
        p[i]=sqrt(sum);
      else
        A[j][i]=sum/p[i];
    }
  }
  for (i=0;i<8;i+=2) {
    r1=sqrt(-LOGFUNC(ran3n(&seed)));
    r2=ran3n(&seed);
    psi[i]=r1*cos(pi2*r2);
    psi[i+1]=r1*sin(pi2*r2);
  }
  for (i=nph-1;i>=0;i--) {
    for (sum=psi[i],k=i+1;k<nph;k++) sum-=A[k][i]*dph[k];
    dph[i]=sum/p[i];
  }
  sum=0; for (i=0;i<nph;i++) sum+=psi[i]*psi[i];
  wfw=exp(-sum); 
  for (i=0;i<nph;i++) wfw*=p[i];

  /** backward probability **/

  for (i=0;i<nph;i++) ph[iph[i]]+=dph[i];
  int2cart(ia);
  nph=0;
  for (i=0;i<4;i++){
    if (seq[ia+i]!='P') {
      ix[nph]=x[iCa[ia+i]];
      iy[nph]=y[iCa[ia+i]];
      iz[nph]=z[iCa[ia+i]];
      bx[nph]=x[iCa[ia+i]]-x[iN[ia+i]];
      by[nph]=y[iCa[ia+i]]-y[iN[ia+i]];
      bz[nph]=z[iCa[ia+i]]-z[iN[ia+i]];
      nph++;
    }
    ix[nph]=x[iC[ia+i]];
    iy[nph]=y[iC[ia+i]];
    iz[nph]=z[iC[ia+i]];
    bx[nph]=x[iC[ia+i]]-x[iCa[ia+i]];
    by[nph]=y[iC[ia+i]]-y[iCa[ia+i]];
    bz[nph]=z[iC[ia+i]]-z[iCa[ia+i]];
    nph++;
  } 
  i=iCa[ia+3]; rx[0]=x[i]; ry[0]=y[i]; rz[0]=z[i];
  i=iC[ia+3]; rx[1]=x[i]; ry[1]=y[i]; rz[1]=z[i];
  i++; rx[2]=x[i]; ry[2]=y[i]; rz[2]=z[i];
  for (i=0;i<3;i++) {
    for (j=0;j<nph;j++) {
      dx[i][j]=(by[j]*(rz[i]-iz[j])-bz[j]*(ry[i]-iy[j]))/ab[j];
      dy[i][j]=(bz[j]*(rx[i]-ix[j])-bx[j]*(rz[i]-iz[j]))/ab[j];
      dz[i][j]=(bx[j]*(ry[i]-iy[j])-by[j]*(rx[i]-ix[j]))/ab[j];
    }
  }
  for (i=0;i<nph;i++) {
    for (j=i;j<nph;j++) {
      A[i][j]=0;
      for (k=0;k<3;k++) {
        A[i][j]+=dx[k][i]*dx[k][j]+dy[k][i]*dy[k][j]+dz[k][i]*dz[k][j];
      }
      A[i][j]*=bbgs;
      if (i==j) A[i][j]+=1;
      A[i][j]*=abgs/2;
    }
  }
  for (i=0;i<nph;i++) {
    for (j=i;j<nph;j++) {
      for (sum=A[i][j],k=i-1;k>=0;k--) sum-=A[i][k]*A[j][k];
      if (i==j)
        p[i]=sqrt(sum);
      else
        A[j][i]=sum/p[i];
    }
  }
  for (i=0;i<nph;i++) {
    psi[i]=p[i]*dph[i]; for (j=i+1;j<nph;j++) psi[i]+=A[j][i]*dph[j];  
  }
  sum=0; for (i=0;i<nph;i++) sum+=psi[i]*psi[i];
  wbw=exp(-sum); 
  for (i=0;i<nph;i++) wbw*=p[i];

  /** accept/reject **/

  emax=Esa-LOGFUNC(wfw*ran3n(&seed)/wbw)/beta[ind]+
       Ebi-(e1=bias(0))+Ehb-(e2=hbonds(0))+Ehp-(e4=hp(0));
  e3=sac(0,emax);
  if (e3<emax) {
    for (i=ib;i<NTO;i++) {xo[i]=x[i]; yo[i]=y[i]; zo[i]=z[i];}
    Ebi=e1; Ehb=e2; Esa=e3; Ehp=e4; E=Ebi+Ehb+Esa+Ehp;
    return 1;
  }
  else {
    for (i=0;i<nph;i++) ph[iph[i]]-=dph[i];
    for (i=ib;i<NTO;i++) {x[i]=xo[i]; y[i]=yo[i]; z[i]=zo[i];}
    return 0;
  }
}
/****************************************************************************/
int pivot(int iflag)
{
  int i,j,ia,i1,i2,iup;
  double phnew;
  double e1,e2,e3,e4,emax;
  double ex,ey,ez,e;
  double dx,dy,dz; 
  double Axx,Axy,Axz,Ayy,Ayz,Azz;
  double Bxx,Bxy,Bxz,Byy,Byz,Bzz;
  double Cxx,Cxy,Cxz,Cyx,Cyy,Cyz,Czx,Czy,Czz;
  double dph,cdph,sdph;
  static int df2a[NDF];                    
  if(iflag<0){
    j=0;
    for (i=0;i<N;i++) {
      if (seq[i]!='P') df2a[j++]=3*i; 
      df2a[j++]=3*i+1;
    }
    if (j!=NDF) {printf("%i -- NDF wrong\n",j);exit(-1);}
    return -1;
  }
  i=NDF*ran3n(&seed); iup=df2a[i]; ia=iup/3;
  if (iup%3==0) {
    i1=iN[ia]; i2=iCa[ia]; e=b[1];
  }
  else if (iup%3==1) {
    i1=iCa[ia]; i2=iC[ia]; e=b[2];
  }
  else {
    printf("%i -- error in pivot\n",iup); exit(-1);
  }
  dph=DELPH*(ran3n(&seed)-0.5); 
  phnew=ph[iup]+dph;
  cdph=cos(dph); sdph=sin(dph);
  ex=(x[i2]-x[i1])/e; 
  ey=(y[i2]-y[i1])/e; 
  ez=(z[i2]-z[i1])/e;
  Axx=ex*ex; Axy=ex*ey; Axz=ex*ez; Ayy=ey*ey; Ayz=ey*ez; Azz=ez*ez;
  Bxx=Byy=Bzz=cdph;
  Bxy=-ez*sdph; Byz=-ex*sdph; Bxz=ey*sdph; 
  Cxx=Bxx+(1-Bxx)*Axx-Bxy*Axy-Bxz*Axz;
  Cxy=Bxy+(1-Bxx)*Axy-Bxy*Ayy-Bxz*Ayz;
  Cxz=Bxz+(1-Bxx)*Axz-Bxy*Ayz-Bxz*Azz;
  Cyx=-Bxy+Bxy*Axx+(1-Byy)*Axy-Byz*Axz;
  Cyy=Byy+Bxy*Axy+(1-Byy)*Ayy-Byz*Ayz;
  Cyz=Byz+Bxy*Axz+(1-Byy)*Ayz-Byz*Azz;
  Czx=-Bxz+Bxz*Axx+Byz*Axy+(1-Bzz)*Axz;
  Czy=-Byz+Bxz*Axy+Byz*Ayy+(1-Bzz)*Ayz;
  Czz=Bzz+Bxz*Axz+Byz*Ayz+(1-Bzz)*Azz;
  for (j=i2+1;j<NTO;j++) {
    dx=x[j]-x[i2]; dy=y[j]-y[i2]; dz=z[j]-z[i2];
    x[j]=x[i2]+Cxx*dx+Cxy*dy+Cxz*dz; 
    y[j]=y[i2]+Cyx*dx+Cyy*dy+Cyz*dz; 
    z[j]=z[i2]+Czx*dx+Czy*dy+Czz*dz; 
  }
  emax=Esa-(LOGFUNC(ran3n(&seed))/beta[ind])+Ebi-(e1=bias(0))+
       Ehb-(e2=hbonds(0))+Ehp-(e4=hp(0));
  e3=sac(0,emax);
  if (e3<emax) {
    ph[iup]=phnew;
    for (i=i2+1;i<NTO;i++) {xo[i]=x[i]; yo[i]=y[i]; zo[i]=z[i];}
    Ebi=e1; Ehb=e2; Esa=e3; Ehp=e4; E=Ebi+Ehb+Esa+Ehp;
    return 1;
  }
  else {
    for (i=i2+1;i<NTO;i++) {x[i]=xo[i]; y[i]=yo[i]; z[i]=zo[i];}
    return 0;
  }
}
/****************************************************************************/
int mirror(int iflag)
{
  int i,j,sep;
  int p,pa,p1,p2,pup;
  int q,qa,q1,q2,qup;
  int r,ra,r1,r2,rup;
  double qlen, len, ulen2, uq, denom0, denom1, scal_p0, scal_p1;
  double px,py,pz;
  double qx,qy,qz;
  double qnx,qny,qnz;
  double qtx,qty,qtz;
  double rx,ry,rz;
  double tx,ty,tz;
  double ux,uy,uz;
  double vx,vy,vz;
  double vnx,vny,vnz;
  double vtx,vty,vtz;
  double wx,wy,wz;
  double cdphp,sdphp;
  double cdphq,sdphq;
  double cdphr,sdphr;
  double cpq, cqr, st;
  double e1,e2,e3,e4,emax;
  double dx,dy,dz; 
  double Axx,Axy,Axz,Ayy,Ayz,Azz;
  double Bxx,Bxy,Bxz,Byy,Byz,Bzz;
  double Cxx,Cxy,Cxz,Cyx,Cyy,Cyz,Czx,Czy,Czz;
  static int df2a[NDF];
  if(iflag<0){
    j=0;
    for (i=0;i<N;i++) {
      if (seq[i]!='P') df2a[j++]=3*i; 
      df2a[j++]=3*i+1;
    }
    if (j!=NDF) {printf("%i -- NDF wrong\n",j);exit(-1);}
    return -1;
  }
  sep=4*ran3n(&seed);
  if(sep==0){
    p=(NDF-2)*ran3n(&seed); 
    q=p+1;
    r=q+1;
  }
  else{
    p=NDF*ran3n(&seed); 
    while((r=NDF*ran3n(&seed))==p);
    if(r<p){
      i=p; p=r; r=i;
    }
    if(sep==3)
      q=p+1+(r-p-1)*ran3n(&seed);
    else
      if(sep==2)
	q=p+1;
      else
	q=r-1;
  }
  pup=df2a[p]; pa=pup/3;
  qup=df2a[q]; qa=qup/3;
  rup=df2a[r]; ra=rup/3;
  if (pup%3==0) {
    p1=iN[pa]; p2=iCa[pa]; len=b[1];
  }
  else if (pup%3==1) {
    p1=iCa[pa]; p2=iC[pa]; len=b[2];
  }
  else {
    printf("%i -- error in mirror\n",pup); exit(-1);
  }
  px=(x[p2]-x[p1])/len; 
  py=(y[p2]-y[p1])/len; 
  pz=(z[p2]-z[p1])/len;
  if (qup%3==0) {
    q1=iN[qa]; q2=iCa[qa]; qlen=b[1];
  }
  else if (qup%3==1) {
    q1=iCa[qa]; q2=iC[qa]; qlen=b[2];
  }
  else {
    printf("%i -- error in mirror\n",qup); exit(-1);
  }
  qx=(x[q2]-x[q1])/qlen; 
  qy=(y[q2]-y[q1])/qlen; 
  qz=(z[q2]-z[q1])/qlen;
  if (rup%3==0) {
    r1=iN[ra]; r2=iCa[ra]; len=b[1];
  }
  else if (rup%3==1) {
    r1=iCa[ra]; r2=iC[ra]; len=b[2];
  }
  else {
    printf("%i -- error in mirror\n",rup); exit(-1);
  }
  rx=(x[r2]-x[r1])/len; 
  ry=(y[r2]-y[r1])/len; 
  rz=(z[r2]-z[r1])/len;
  if(px*px+py*py+pz*pz>1+MIN_DENOM||
     qx*qx+qy*qy+qz*qz>1+MIN_DENOM||
     rx*rx+ry*ry+rz*rz>1+MIN_DENOM) {
    //printf("langd (p,q,r) = (%f,%f,%f)\n",
    //  px*px+py*py+pz*pz,qx*qx+qy*qy+qz*qz,rx*rx+ry*ry+rz*rz);
    //printf("index (p,q,r) = (%i,%i,%i)\n",pup,qup,rup);
    exit(-1);
  }

  ux=py*rz-pz*ry;
  uy=pz*rx-px*rz;
  uz=px*ry-py*rx;
  vx=py*qz-pz*qy;
  vy=pz*qx-px*qz;
  vz=px*qy-py*qx;
  wx=qy*rz-qz*ry;
  wy=qz*rx-qx*rz;
  wz=qx*ry-qy*rx;
  cpq=px*qx+py*qy+pz*qz;
  cqr=qx*rx+qy*ry+qz*rz;
  ulen2=ux*ux+uy*uy+uz*uz;
  uq=ux*qx+uy*qy+uz*qz;

  tx=qx-cpq*px;
  ty=qy-cpq*py;
  tz=qz-cpq*pz;
  denom0=sqrt(ulen2*(tx*tx+ty*ty+tz*tz));
  if(denom0<MIN_DENOM) return 0;
  st=uq/denom0;
  scal_p0 = ux*vx+uy*vy+uz*vz;
  if(fabs(scal_p0)<MIN_DENOM) return 0;
  if(scal_p0>0.) st=-st;
  cdphp=1-2*st*st;
  sdphp=2*st*sqrt(1-st*st);

  tx=qx-cqr*rx;
  ty=qy-cqr*ry;
  tz=qz-cqr*rz;
  denom1=sqrt(ulen2*(tx*tx+ty*ty+tz*tz));
  if(denom1<MIN_DENOM) return 0;
  st=uq/denom1;
  scal_p1 = ux*wx+uy*wy+uz*wz;
  if(fabs(scal_p1)<MIN_DENOM) return 0;
  if(scal_p1>0.) st=-st;
  cdphr=1-2*st*st;
  sdphr=2*st*sqrt(1-st*st);

  qnx=(qx-2*ux*uq/ulen2)*qlen;
  qny=(qy-2*uy*uq/ulen2)*qlen;
  qnz=(qz-2*uz*uq/ulen2)*qlen;

  Axx=px*px; Axy=px*py; Axz=px*pz; Ayy=py*py; Ayz=py*pz; Azz=pz*pz;
  Bxx=Byy=Bzz=cdphp;
  Bxy=-pz*sdphp; Byz=-px*sdphp; Bxz=py*sdphp; 
  Cxx=Bxx+(1-Bxx)*Axx-Bxy*Axy-Bxz*Axz;
  Cxy=Bxy+(1-Bxx)*Axy-Bxy*Ayy-Bxz*Ayz;
  Cxz=Bxz+(1-Bxx)*Axz-Bxy*Ayz-Bxz*Azz;
  Cyx=-Bxy+Bxy*Axx+(1-Byy)*Axy-Byz*Axz;
  Cyy=Byy+Bxy*Axy+(1-Byy)*Ayy-Byz*Ayz;
  Cyz=Byz+Bxy*Axz+(1-Byy)*Ayz-Byz*Azz;
  Czx=-Bxz+Bxz*Axx+Byz*Axy+(1-Bzz)*Axz;
  Czy=-Byz+Bxz*Axy+Byz*Ayy+(1-Bzz)*Ayz;
  Czz=Bzz+Bxz*Axz+Byz*Ayz+(1-Bzz)*Azz;
  for (j=p2+1;j<q2;j++) {
    dx=x[j]-x[p2]; dy=y[j]-y[p2]; dz=z[j]-z[p2];
    x[j]=x[p2]+Cxx*dx+Cxy*dy+Cxz*dz; 
    y[j]=y[p2]+Cyx*dx+Cyy*dy+Cyz*dz; 
    z[j]=z[p2]+Czx*dx+Czy*dy+Czz*dz; 
  }
  vnx=Cxx*vx+Cxy*vy+Cxz*vz;
  vny=Cyx*vx+Cyy*vy+Cyz*vz; 
  vnz=Czx*vx+Czy*vy+Czz*vz; 

  Axx=rx*rx; Axy=rx*ry; Axz=rx*rz; Ayy=ry*ry; Ayz=ry*rz; Azz=rz*rz;
  Bxx=Byy=Bzz=cdphr;
  Bxy=rz*sdphr; Byz=rx*sdphr; Bxz=-ry*sdphr; 
  Cxx=Bxx+(1-Bxx)*Axx-Bxy*Axy-Bxz*Axz;
  Cxy=Bxy+(1-Bxx)*Axy-Bxy*Ayy-Bxz*Ayz;
  Cxz=Bxz+(1-Bxx)*Axz-Bxy*Ayz-Bxz*Azz;
  Cyx=-Bxy+Bxy*Axx+(1-Byy)*Axy-Byz*Axz;
  Cyy=Byy+Bxy*Axy+(1-Byy)*Ayy-Byz*Ayz;
  Cyz=Byz+Bxy*Axz+(1-Byy)*Ayz-Byz*Azz;
  Czx=-Bxz+Bxz*Axx+Byz*Axy+(1-Bzz)*Axz;
  Czy=-Byz+Bxz*Axy+Byz*Ayy+(1-Bzz)*Ayz;
  Czz=Bzz+Bxz*Axz+Byz*Ayz+(1-Bzz)*Azz;
  for (j=q2;j<r2;j++) {
    dx=x[j]-x[r2]; dy=y[j]-y[r2]; dz=z[j]-z[r2];
    x[j]=x[r2]+Cxx*dx+Cxy*dy+Cxz*dz; 
    y[j]=y[r2]+Cyx*dx+Cyy*dy+Cyz*dz; 
    z[j]=z[r2]+Czx*dx+Czy*dy+Czz*dz; 
  }
  vtx=Cxx*vx+Cxy*vy+Cxz*vz;
  vty=Cyx*vx+Cyy*vy+Cyz*vz; 
  vtz=Czx*vx+Czy*vy+Czz*vz; 
  qtx=Cxx*qx+Cxy*qy+Cxz*qz;
  qty=Cyx*qx+Cyy*qy+Cyz*qz; 
  qtz=Czx*qx+Czy*qy+Czz*qz; 
  if(fabs(qny*qtz-qnz*qty) > 0.00001){
    printf("Bug 0 in mirror %f %f %f %f %f\n",fabs(qny*qtz-qnz*qty),
	   denom0, denom1, scal_p0, scal_p1);
    exit(-1);
  }

  dx=qnx+x[q1]-x[q2];
  dy=qny+y[q1]-y[q2];
  dz=qnz+z[q1]-z[q2];
  for (j=q2;j<NTO;j++) {
    x[j]+=dx; 
    y[j]+=dy; 
    z[j]+=dz; 
  }

  //len=vx*vx+vy*vy+vz*vz;
  len=sqrt((vnx*vnx+vny*vny+vnz*vnz)*(vtx*vtx+vty*vty+vtz*vtz));
  cdphq=(vnx*vtx+vny*vty+vnz*vtz)/len;
  tx=vny*vtz-vnz*vty;
  ty=vnz*vtx-vnx*vtz;
  tz=vnx*vty-vny*vtx;
 
  len*=qlen;
  sdphq=(tx*qnx+ty*qny+tz*qnz)/len;
  if(fabs(qny*tz-qnz*ty) > 0.00001){
    printf("Bug 1 in mirror %f %f %f %f %f\n",fabs(qny*tz-qnz*ty),
	   denom0, denom1, scal_p0, scal_p1);
    exit(-1);
  }
  emax=Esa-(LOGFUNC(ran3n(&seed))/beta[ind])+
       Ebi-(e1=bias(0))+Ehb-(e2=hbonds(0))+Ehp-(e4=hp(0));
  e3=sac(0,emax);
  if (e3<emax) {
    ph[pup]+=atan2(sdphp,cdphp);
    ph[qup]+=atan2(sdphq,cdphq);
    ph[rup]+=atan2(sdphr,cdphr);
    for (i=p2+1;i<NTO;i++) {xo[i]=x[i]; yo[i]=y[i]; zo[i]=z[i];}
    Ebi=e1; Ehb=e2; Esa=e3; Ehp=e4; E=Ebi+Ehb+Esa+Ehp;
    return 1;
  }
  else {
    for (i=p2+1;i<NTO;i++) {x[i]=xo[i]; y[i]=yo[i]; z[i]=zo[i];}
    return 0;
  }
}
/****************************************************************************/
int rot(int iflag)
{
  extern double hb_sidemain(int i);
  extern double hb_sideside(int i);
  extern double hp_pair(int iflag,int i,int j);
  extern void init_rot(int i,int rta[],int rtb[],int rtc[],int rtd[]);
  int i,j,a,ir,iaa;
  short oldrt;
  double oldhb,newhb,oldsa=0,newsa=0,oldhp=0,newhp=0;
  double r2,r6;
  static double cut2[3];                                      
  static double sig2[3*NA*NA],asa[3*NA*NA],bsa[3*NA*NA];
  static int rta[max(1,NRT)];   // first atom affected 
  static int rtb[max(1,NRT)];   // last atom affected  
  static int rtc[max(1,NRT)];   // amino acid index    
  static int rtd[max(1,NRT)];   // index within amino acid 
  static int omitp[max(1,NHAA)];// omit hydrophobic neighbors
  static int omitm[max(1,NHAA)];// omit hydrophobic neighbors

  if (iflag<0) {
    cut2[0]=ALPHA1*ALPHA1*cut*cut;
    cut2[1]=cut2[2]=cut*cut;
    for (i=0;i<NA;i++) {
      for (j=0;j<NA;j++) {
	sig2[i+j*NA]=pow(ALPHA1*(sigsa[i]+sigsa[j]),2.0);
	sig2[i+j*NA+NA*NA]=pow(sigsa[i]+sigsa[j],2.0);
	sig2[i+j*NA+2*NA*NA]=pow(ALPHA2*(sigsa[i]+sigsa[j]),2.0);
      }
    }
    for (i=0;i<3*NA*NA;i++) {
      asa[i]=-7*pow(sig2[i]/cut2[i/(NA*NA)],6.0);
      bsa[i]=6*pow(sig2[i]/cut2[i/(NA*NA)],6.0)/cut2[i/(NA*NA)];
    } 
    for (i=0;i<N;i++) init_rot(i,rta,rtb,rtc,rtd);        
    for (i=0;i<NHAA;i++) omitp[i]=omitm[i]=0;
    for (i=0;i<N-1;i++) {
      if ((a=ih[i])<0) continue;
      if (ih[i+1]==a+1) omitp[a]=1;
    }
    for (i=1;i<N;i++) {
      if ((a=ih[i])<0) continue;
      if (ih[i-1]==a-1) omitm[a]=1;
    }
    return 0;
  }

  ir=NRT*ran3n(&seed); oldrt=rt[ir]; 
  while ((rt[ir]=NDI*ran3n(&seed))==oldrt) {};

  oldhb=hb_sidemain((iaa=rtc[ir]));
  oldhb+=hb_sideside(0);

  for (i=rta[ir];i<=rtb[ir];i++) {
    for (j=0;j<NTO;j++) {
      if ((a=aa[i][j])<0) continue;
      if ((r2=(x[i]-x[j])*(x[i]-x[j])+
              (y[i]-y[j])*(y[i]-y[j])+
              (z[i]-z[j])*(z[i]-z[j]))>cut2[a/(NA*NA)]) continue;
      r6=sig2[a]/r2; r6=r6*r6*r6;
      oldsa+=r6*r6+asa[a]+bsa[a]*r2;
    }
  }

  if ((i=ih[iaa])>=0) {
    for (j=0;j<NHAA;j++) {
      if (j==i) continue;
      if (j==i+1 && omitp[i]==1) continue;
      if (j==i-1 && omitm[i]==1) continue;
      oldhp+=hp_pair(0,i,j);
    }
  }

  switch (seq[(i=iaa)]){ 
  case 'A' : {ala(0,i); break;}
  case 'V' : {val(0,i,rtd[ir]); break;}
  case 'L' : {leu(0,i,rtd[ir]); break;}
  case 'I' : {ile(0,i,rtd[ir]); break;}
  case 'S' : {ser(0,i,rtd[ir]); break;}
  case 'T' : {thr(0,i,rtd[ir]); break;}
  case 'M' : {met(0,i,rtd[ir]); break;}
  case 'P' : {pro(0,i); break;}
  case 'D' : {asp(0,i,rtd[ir]); break;}
  case 'N' : {asn(0,i,rtd[ir]); break;}
  case 'E' : {glu(0,i,rtd[ir]); break;}
  case 'Q' : {gln(0,i,rtd[ir]); break;}
  case 'K' : {lys(0,i,rtd[ir]); break;}
  case 'R' : {arg(0,i,rtd[ir]); break;}
  case 'H' : {his(0,i,rtd[ir]); break;}
  case 'F' : {phe(0,i,rtd[ir]); break;}
  case 'Y' : {tyr(0,i,rtd[ir]); break;}
  case 'W' : {trp(0,i,rtd[ir]); break;}
  default : {printf("error in rot\n");exit(-1);}
  }

  newhb=hb_sidemain(iaa);
  newhb+=hb_sideside(0);

  for (i=rta[ir];i<=rtb[ir];i++) {
    for (j=0;j<NTO;j++) {
      if ((a=aa[i][j])<0) continue;
      if ((r2=(x[i]-x[j])*(x[i]-x[j])+
              (y[i]-y[j])*(y[i]-y[j])+
              (z[i]-z[j])*(z[i]-z[j]))>cut2[a/(NA*NA)]) continue;
      r6=sig2[a]/r2; r6=r6*r6*r6;
      newsa+=r6*r6+asa[a]+bsa[a]*r2;
    }
  }

  if ((i=ih[iaa])>=0) {
    for (j=0;j<NHAA;j++) {
      if (j==i) continue;
      if (j==i+1 && omitp[i]==1) continue;
      if (j==i-1 && omitm[i]==1) continue;
      newhp+=hp_pair(0,i,j);
    }
  }

  if (beta[ind]*(epshb2*(newhb-oldhb)+ksa*(newsa-oldsa)+epshp*(newhp-oldhp))
      <-LOGFUNC(ran3n(&seed))) {
    Ehb+=epshb2*(newhb-oldhb); 
    Esa+=ksa*(newsa-oldsa); 
    Ehp+=epshp*(newhp-oldhp); 
    E=Ebi+Ehb+Esa+Ehp;
    for (i=rta[ir];i<=rtb[ir];i++) {xo[i]=x[i]; yo[i]=y[i]; zo[i]=z[i];}
    return 1;
  }
  else {
    rt[ir]=oldrt;
    for (i=rta[ir];i<=rtb[ir];i++) {x[i]=xo[i]; y[i]=yo[i]; z[i]=zo[i];}
    return 0;
  }
}
/****************************************************************************/
void init_rot(int i,int rta[],int rtb[],int rtc[],int rtd[])        
{
  int iCb=iCa[i]+2;
  switch (seq[i]){  
  case 'G' : {
    break;
  }
  case 'A' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+3;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    break;
  } 
  case 'V' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+9;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+6;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+9;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    break;
  }
  case 'L' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+12;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+12;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+9;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    rta[irt[i]+3]=iCb+10; rtb[irt[i]+3]=iCb+12;
    rtc[irt[i]+3]=i; rtd[irt[i]+3]=3;
    break;
  }
  case 'I' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+12;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+9;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+9;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    rta[irt[i]+3]=iCb+10; rtb[irt[i]+3]=iCb+12;
    rtc[irt[i]+3]=i; rtd[irt[i]+3]=3;
    break;
  }
  case 'S' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+4;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+4;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    break;
  }
  case 'T' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+7;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+4;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+5; rtb[irt[i]+2]=iCb+7;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    break;
  }
  case 'M' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+10;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+10;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+10;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    rta[irt[i]+3]=iCb+8; rtb[irt[i]+3]=iCb+10;
    rtc[irt[i]+3]=i; rtd[irt[i]+3]=3;
    break;
  }
  case 'P' : {
    break;
  }
  case 'D' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+5;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+5;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    break;
  }
  case 'N' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+7;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+7;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+6; rtb[irt[i]+2]=iCb+7;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    break;
  }
  case 'E' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+8;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+8;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+8;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    break;
  }
  case 'Q' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+10;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+10;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+10;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    rta[irt[i]+3]=iCb+9; rtb[irt[i]+3]=iCb+10;
    rtc[irt[i]+3]=i; rtd[irt[i]+3]=3;
    break;
  }
  case 'K' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+15;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+15;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+15;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    rta[irt[i]+3]=iCb+10; rtb[irt[i]+3]=iCb+15;
    rtc[irt[i]+3]=i; rtd[irt[i]+3]=3;
    rta[irt[i]+4]=iCb+13; rtb[irt[i]+4]=iCb+15;
    rtc[irt[i]+4]=i; rtd[irt[i]+4]=4;
    break;
  }
  case 'R' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+17;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+17;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+7; rtb[irt[i]+2]=iCb+17;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    rta[irt[i]+3]=iCb+10; rtb[irt[i]+3]=iCb+17;
    rtc[irt[i]+3]=i; rtd[irt[i]+3]=3;
    break;
  }
  case 'H' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+10;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+10;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    break;
  }
  case 'F' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+13;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+13;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    break;
  }
  case 'Y' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+14;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+14;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    rta[irt[i]+2]=iCb+14; rtb[irt[i]+2]=iCb+14;
    rtc[irt[i]+2]=i; rtd[irt[i]+2]=2;
    break;
  }
  case 'W' : {
    rta[irt[i]]=iCb+1; rtb[irt[i]]=iCb+17;
    rtc[irt[i]]=i; rtd[irt[i]]=0;
    rta[irt[i]+1]=iCb+4; rtb[irt[i]+1]=iCb+17;
    rtc[irt[i]+1]=i; rtd[irt[i]+1]=1;
    break;
  }
  default : {printf("WARNING: unknown sequence\n");}
  }        
}
/****************************************************************************/
int flip(double e)
{
  int nind; 
  nind=(ran3n(&seed)>0.5) ? ind+1:ind-1; 
  if (nind==NTMP || nind==-1) return 0.0;
  if (ran3n(&seed)<exp(-((beta[nind]-beta[ind])*e+g[nind]-g[ind]))) { 
    ind=nind; return 1;
  }
  else 
    return 0;
}
/****************************************************************************/
/***** MEASUREMENTS *********************************************************/
/****************************************************************************/
void histor(int iflag)
{
  int i,j;
  static double cnt,his[max(1,NRT)][NDI];
  FILE *fp;
  if (iflag<0) {
    cnt=0;
    for (i=0;i<NRT;i++) {for (j=0;j<NDI;j++) his[i][j]=0;}
    fp=fopen("tmp_hisr","w");
    fclose(fp);
    return;
  }
  if (iflag==0) {
    cnt++;
    for (i=0;i<NRT;i++) {
      his[i][rt[i]]++;
    }
    return;
  }
  if (iflag>0 && cnt>0) {
    fp=fopen("tmp_hisr","a");
    for (i=0;i<NRT;i++) {
      for (j=0;j<NDI;j++) fprintf(fp,"%i %f %f\n",
                                     i,j*360.0/NDI,his[i][j]*NDI/360.0/cnt);
      fprintf(fp,"\n");
    }
    fclose(fp);
    return;
  }
}
/****************************************************************************/
void profile(int iflag)
{
  int i,ic,in,ih,io;
  double r2,r4,r6;
  double dx,dy,dz,ca,cb;
  static double cut2,cst;                                      
  static double ahb,bhb,sighb2;  
  static double prof[NTMP][N-4];
  static int nmeas[NTMP];
  FILE *fp;

  if (iflag<0) {
    for (i=0;i<NTMP;i++) {
      nmeas[i]=0;
      for (io=0;io<N-4;io++) prof[i][io]=0;
    }
    cut2=cuthb*cuthb;
    bhb=-30*(pow(sighb/cuthb,10.0)-pow(sighb/cuthb,12.0))/cut2;
    ahb=-(5*pow(sighb/cuthb,12.0)-6*pow(sighb/cuthb,10.0))-bhb*cut2;
    sighb2=sighb*sighb; 
    cst=pow(1.0/bNH,POWA)*pow(1.0/bCO,POWB);
    return;
  }

  if (iflag==0) {
    nmeas[ind]++;
    for (i=0;i<N-4;i++) {
      if (seq[i+4]=='P') continue;
      in=iN[i+4]; ih=in+1; 
      ic=iC[i]; io=ic+1;
      if ((r2=(x[io]-x[ih])*(x[io]-x[ih])+
	   (y[io]-y[ih])*(y[io]-y[ih])+
	   (z[io]-z[ih])*(z[io]-z[ih]))>cut2) continue;
      dx=x[io]-x[ih]; dy=y[io]-y[ih]; dz=z[io]-z[ih];
      if (POWA>0 &&
          (ca=(x[ih]-x[in])*dx+(y[ih]-y[in])*dy+(z[ih]-z[in])*dz)<0) continue;
      if (POWB>0 &&	
          (cb=(x[ic]-x[io])*dx+(y[ic]-y[io])*dy+(z[ic]-z[io])*dz)<0) continue;
      r6=sighb2/r2; r4=r6*r6; r6=r6*r4;
      prof[ind][i]+=cst*pow(ca*ca/r2,0.5*POWA)*pow(cb*cb/r2,0.5*POWB)*
                   (r6*(5*r6-6*r4)+ahb+bhb*r2);
    }
    return;
  }

  if (iflag>0) {
    fp=fopen("tmp_prof","w");
    for (i=0;i<NTMP;i++) {
      if (nmeas[i]>0) {
	for (io=0;io<N-4;io++) fprintf(fp,"%i %i %f\n",i,io,
                                       prof[i][io]/nmeas[i]);
        fprintf(fp,"\n");
      }
    } 
    fclose(fp);
    return;
  }
  
}
/****************************************************************************/
void histoe(int iflag,int itmp,double x)
{ 
  static double low=0.0;      
  static double high=80.0;
  static double his[NTMP][NBIN],out[NTMP];
  static double eps;
  int i,k;
  FILE *fp;

  if (iflag<0) {
    for (i=0;i<NTMP;i++) {
      out[i]=0;
      for (k=0;k<NBIN;k++) his[i][k]=0;
    }
    eps=(high-low)/NBIN;
    fp=fopen("tmp_hise","w");
    fclose(fp);
    return;
  }

  if (iflag==0) {
    if (x>low && x<high) { 
      k=(x-low)/eps; his[itmp][k]++;
    }
    else out[itmp]++;
    return;
  }

  if (iflag>0) {
    fp=fopen("tmp_hise","a");
    if (x>0) {
      for (k=0;k<NBIN;k++) {
	fprintf(fp,"%i %f %f\n",
		itmp,low+eps*(k+0.5),his[itmp][k]/x/eps);
      }
    }
    fprintf(fp,"\n");
    if (out[itmp]>0) printf("temp %i energy out %f\n",itmp,out[itmp]);
    fclose(fp);
    return;
  }
}
/****************************************************************************/
void histoq(int iflag,int itmp,double x)
{ 
  static double low=0.0;      
  static double high=1.0;
  static double his[NTMP][NBIN],out[NTMP];
  static double eps;
  int i,k;
  FILE *fp;

  if (iflag<0) {
    for (i=0;i<NTMP;i++) {
      out[i]=0;
      for (k=0;k<NBIN;k++) his[i][k]=0;
    }
    eps=(high-low)/NBIN;
    fp=fopen("tmp_hisq","w");
    fclose(fp);
    return;
  }

  if (iflag==0) {
    if (x>low && x<high) { 
      k=(x-low)/eps; his[itmp][k]++;
    }
    else out[itmp]++;
    return;
  }

  if (iflag>0) {
    fp=fopen("tmp_hisq","a");
    if (x>0) {
      for (k=0;k<NBIN;k++) {
	fprintf(fp,"%i %f %f\n",
		itmp,low+eps*(k+0.5),his[itmp][k]/x/eps);
      }
    }
    fprintf(fp,"\n");
    if (out[itmp]>0) printf("temp %i q out %f\n",itmp,out[itmp]);
    fclose(fp);
    return;
  }
}
/****************************************************************************/
void histohp(int iflag,int itmp,double x)
{ 
  static double low=-15.0;      
  static double high=1.0;
  static double his[NTMP][NBIN],out[NTMP];
  static double eps;
  int i,k;
  FILE *fp;

  if (iflag<0) {
    for (i=0;i<NTMP;i++) {
      out[i]=0;
      for (k=0;k<NBIN;k++) his[i][k]=0;
    }
    eps=(high-low)/NBIN;
    fp=fopen("tmp_hishp","w");
    fclose(fp);
    return;
  }

  if (iflag==0) {
    if (x>low && x<high) { 
      k=(x-low)/eps; his[itmp][k]++;
    }
    else out[itmp]++;
    return;
  }

  if (iflag>0) {
    fp=fopen("tmp_hishp","a");
    if (x>0) {
      for (k=0;k<NBIN;k++) {
	fprintf(fp,"%i %f %f\n",
		itmp,low+eps*(k+0.5),his[itmp][k]/x/eps);
      }
    }
    fprintf(fp,"\n");
    if (out[itmp]>0) printf("temp %i hp out %f\n",itmp,out[itmp]);
    fclose(fp);
    return;
  }
}
/****************************************************************************/
double ree2(void)
{
  return (x[iN[0]]-x[iC[N-1]])*(x[iN[0]]-x[iC[N-1]])+
         (y[iN[0]]-y[iC[N-1]])*(y[iN[0]]-y[iC[N-1]])+
         (z[iN[0]]-z[iC[N-1]])*(z[iN[0]]-z[iC[N-1]]);
}
/****************************************************************************/
double gyros(void)
{
  double rg2=0,xcm=0,ycm=0,zcm=0;
  int i;
  for (i=0;i<N;i++) {
    xcm+=x[iN[i]]; ycm+=y[iN[i]]; zcm+=z[iN[i]];
    xcm+=x[iCa[i]]; ycm+=y[iCa[i]]; zcm+=z[iCa[i]];
    xcm+=x[iC[i]]; ycm+=y[iC[i]]; zcm+=z[iC[i]];
  }
  xcm/=NBA; ycm/=NBA; zcm/=NBA;
  for (i=0;i<N;i++) rg2+=(x[iN[i]]-xcm)*(x[iN[i]]-xcm)+
                         (x[iCa[i]]-xcm)*(x[iCa[i]]-xcm)+
                         (x[iC[i]]-xcm)*(x[iC[i]]-xcm)+
                         (y[iN[i]]-ycm)*(y[iN[i]]-ycm)+
                         (y[iCa[i]]-ycm)*(y[iCa[i]]-ycm)+
                         (y[iC[i]]-ycm)*(y[iC[i]]-ycm)+
                         (z[iN[i]]-zcm)*(z[iN[i]]-zcm)+
                         (z[iCa[i]]-zcm)*(z[iCa[i]]-zcm)+
                         (z[iC[i]]-zcm)*(z[iC[i]]-zcm);
  return rg2/NBA;
}
/****************************************************************************/
int helix(int iflag){
  int i,j;
  int ic,in,ih,io,nhb=0;
  double e=0;
  double r2,r4,r6;
  double dx,dy,dz,ca,cb;
  static double cut2,cst;                                      
  static double ahb,bhb,sighb2;  

  if (iflag<0) {
    cut2=cuthb*cuthb;
    bhb=-30*(pow(sighb/cuthb,10.0)-pow(sighb/cuthb,12.0))/cut2;
    ahb=-(5*pow(sighb/cuthb,12.0)-6*pow(sighb/cuthb,10.0))-bhb*cut2;
    sighb2=sighb*sighb;
    cst=pow(1.0/bNH,POWA)*pow(1.0/bCO,POWB);
    return 0.0;
  }

  for (i=4;i<N;i++) {
    if (seq[i]=='P') continue;
    in=iN[i]; ih=in+1; 
    j=i-4;
    ic=iC[j]; io=ic+1;
    if ((r2=(x[io]-x[ih])*(x[io]-x[ih])+
	 (y[io]-y[ih])*(y[io]-y[ih])+
	 (z[io]-z[ih])*(z[io]-z[ih]))>cut2) continue;
    dx=x[io]-x[ih]; dy=y[io]-y[ih]; dz=z[io]-z[ih];
    if (POWA>0 &&
	(ca=(x[ih]-x[in])*dx+(y[ih]-y[in])*dy+(z[ih]-z[in])*dz)<0) continue; 
    if (POWB>0 && 
	(cb=(x[ic]-x[io])*dx+(y[ic]-y[io])*dy+(z[ic]-z[io])*dz)<0) continue;
    r6=sighb2/r2; r4=r6*r6; r6=r6*r4;   
    e=cst*pow(ca*ca/r2,0.5*POWA)*pow(cb*cb/r2,0.5*POWB)*
      (r6*(5*r6-6*r4)+ahb+bhb*r2);
    if (e<-0.5) nhb++;
  }
  return nhb;
}
/****************************************************************************/
/***** RMSD *****************************************************************/
/****************************************************************************/
double delta2(int flag)
{
  extern void rmsd_init(char *fn,int *seq,int nr1,int nr2);
  extern double rmsd2(int flag,double x[],double y[], double z[],int natom);
  double xx[NTO],yy[NTO],zz[NTO];
  int i=0,j,k,ke;

  if(flag<0){
    if (NR2>=N) {printf("NR2=%i too large\n",NR2);exit(-1);}
    rmsd_init(NATIVE,seq,NR1,NR2);
    return 0;
  }
  
  for(j=NR1;j<=NR2;j++){
    xx[i]=x[iN[j]];   yy[i]=y[iN[j]];   zz[i]=z[iN[j]]; i++;
    xx[i]=x[iCa[j]];  yy[i]=y[iCa[j]];  zz[i]=z[iCa[j]]; i++;
    xx[i]=x[iC[j]];   yy[i]=y[iC[j]];   zz[i]=z[iC[j]]; i++;
    if(flag==1){
      xx[i]=x[iC[j]+1];   yy[i]=y[iC[j]+1];   zz[i]=z[iC[j]+1]; i++;
      ke = (j!=N-1 ? iN[j+1] : NTO);
      for(k=iCa[j]+2;k<ke-2;k++){
        if(atyp[k]==3) continue;
        xx[i]=x[k]; yy[i]=y[k]; zz[i]=z[k]; i++;
      }
    }
  }
  return rmsd2(flag,xx,yy,zz,i);
}
/****************************************************************************/
/***** INPUT/OUTPUT *********************************************************/
/****************************************************************************/
void printheading(double frbgs[],double frmir[],double frrot[])
{
  int i;
  printf("Program bas_W.c (020509)\n");
  printf("Sequence: ");
  for (i=0;i<N;i++) {putchar(seq[i]); if ((i+1)%4==0) printf(" ");}
  printf("\n");
  printf("T, g, fraction bgs, fraction mir, fraction rot:\n");
  for (i=0;i<NTMP;i++) {
    printf("%i %10.6f %10.6f %10.6f %10.6f %10.6f\n",
            i,1./beta[i],g[i],frbgs[i],frmir[i],frrot[i]);
  }
  printf("\n");
  printf("Bond lengths: NCa %f  CaC %f  CN %f\n",b[1],b[2],b[0]);
  printf("              NH %f  CaHa %f  CaCb %f  CO %f\n",bNH,bCaHa,bCaCb,bCO);
  printf("Bias alpha/beta: %f / %f\n",kba,kbb);
  printf("H bonds: powa %f  powb %f  epshb %f  epshb2 %f\n",
          POWA,POWB,epshb,epshb2);
  printf("         sighb %f  cuthb %f\n",
          sighb,cuthb);
  printf("Hydrophobicity: epshp %f  ahp %f  bhp %f\n",epshp,ahp,bhp);
  printf("Self-avoidance: ksa %f  cut %f\n",ksa,cut);
  printf("Radii: C %f  N %f  O %f  H %f\n",
	  sigsa[0],sigsa[1],sigsa[2],sigsa[3]);
  printf("       ALPHA1 %f  ALPHA2 %f\n",ALPHA1,ALPHA2);
  printf("Native structure from file %s\n",NATIVE);  
  printf("Rmsd: atoms %i - %i\n",NR1,NR2);
  printf("MC parameters:\n");
  printf("  MCCYC %i  NTHERM %i  MCCON %i  DELPH %f  NDI %i\n",
	 MCCYC,NTHERM,MCCON,DELPH,NDI);
  printf("  abgs %f  bbgs %f\n",abgs,bbgs);
  printf("  INORM %i  IRT %i  IRAMA %i  ICONF %i  IBLOCK %i\n\n",
          INORM,IRT,IRAMA,ICONF,IBLOCK);
  printf("-----------------------------\n\n");
  fflush(0);
} 
/****************************************************************************/
 void write_conf(int iflag,char *fn,long orig_seed)
{
  int i;
  double phw[2*N];
  FILE *fp;

  if (iflag<0) {
    fp=fopen(fn,"w");
    fwrite(&orig_seed,sizeof(long),1,fp); 
    fclose(fp);
    return;
  }
  fp=fopen(fn,"a");
  for (i=0;i<N;i++) {phw[2*i]=ph[3*i]; phw[2*i+1]=ph[3*i+1];} 
  fwrite(&ind,sizeof(int),1,fp);
  fwrite(&seed,sizeof(long),1,fp);
  fwrite(phw,sizeof(double),2*N,fp);
  fwrite(rt,sizeof(short),NRT,fp);
  fwrite(&E,sizeof(double),1,fp);
  fclose(fp);
}
/****************************************************************************/
void restart(char *fn,long *orig_seed)
{
  FILE *fp;
  int i,j=0;
  long cnt_seed;
  double e,phr[2*N];
  fp=fopen(fn,"r");
  fread(orig_seed,sizeof(long),1,fp);
  while (fread(&ind,sizeof(int),1,fp)==1) {
    fread(&cnt_seed,sizeof(long),1,fp);
    fread(phr,sizeof(double),2*N,fp);
    fread(rt,sizeof(short),NRT,fp);
    fread(&e,sizeof(double),1,fp);
  }
  for(i=0;i<3*N;i++){if(i%3==2)ph[i]=pi;else ph[i]=phr[j++];}
  seed=(*orig_seed);
  for(i=0;i<cnt_seed;i++) ran3n(&seed);
  printf("temp index=%i energy=%f\n",ind,e);
  printf("no of previous ran3n calls %li\n",cnt_seed);
  fclose(fp);
}
/****************************************************************************/
void plot_rama(int iaa)
{
  FILE *fp;
  while (ph[3*iaa]>pi) ph[3*iaa]-=pi2;
  while (ph[3*iaa]<-pi) ph[3*iaa]+=pi2;
  while (ph[3*iaa+1]>pi) ph[3*iaa+1]-=pi2;
  while (ph[3*iaa+1]<-pi) ph[3*iaa+1]+=pi2;
  fp=fopen(RAMA,"a"); 
  fprintf(fp,"%f %f\n",ph[3*iaa]*180/pi,ph[3*iaa+1]*180/pi);
  fclose(fp); 
}
/****************************************************************************/
void runtime(long it,double o[],int n)
{
  extern double sidemain(int i);
  int i;
  FILE *fp;
  fp=fopen(RT,"a");
  fprintf(fp,"%li %i ",it,ind);
  for (i=1;i<n;i++) {if (i!=8) fprintf(fp,"%f ",o[i]);}
  fprintf(fp,"\n");
  fclose(fp);
}
/****************************************************************************/
void dumppdb(void)
{
  int i,j,iatom,iaa,n=0;
  const char atom_name[11][4]={"C  ","N  ","O  ","H  ","CA ","CB ",
			      "CX ","NX ","OX ","HX ","SX "};  
  const char aminoacid[19][4]={"GLY","ALA","VAL","LEU","ILE","SER","THR",
                               "MET","PRO","ASP","ASN","GLU","GLN","LYS",
			       "ARG","HIS","PHE","TYR","TRP"};

  FILE *fp;
  fp=fopen("tmp_pdb","w");
  for (i=0;i<N;i++) {
    for (j=iN[i];j<=iC[i]+1;j++) {
      n++;
      iatom=0;
      if (atyp[j]==0) {
	  if(j==iC[i])iatom=0; else
	  if(j==iCa[i])iatom=4; else
          if((seq[i]!='G')&&(j==iCa[i]+2))iatom=5; 
	  else
	    iatom=6;
      }
      if (atyp[j]==1) {
	  if(j==iN[i])
	      iatom=1;
	  else
	      iatom=7;
      }
      if (atyp[j]==2) {
	  if(j==iC[i]+1)
	      iatom=2;
	  else
	      iatom=8;
      }
      if (atyp[j]==3) {
	  if(j==iN[i]+1)
	      iatom=3;
	  else
	      iatom=9;
      }
      if (atyp[j]==4) {
	iatom=10;
      }
      switch (seq[i]){  
      case 'G' : {iaa=0; break;}
      case 'A' : {iaa=1; break;}
      case 'V' : {iaa=2; break;}
      case 'L' : {iaa=3; break;}
      case 'I' : {iaa=4; break;}
      case 'S' : {iaa=5; break;}
      case 'T' : {iaa=6; break;}
      case 'M' : {iaa=7; break;}
      case 'P' : {iaa=8; break;}
      case 'D' : {iaa=9; break;}
      case 'N' : {iaa=10; break;}
      case 'E' : {iaa=11; break;}
      case 'Q' : {iaa=12; break;}
      case 'K' : {iaa=13; break;}
      case 'R' : {iaa=14; break;}
      case 'H' : {iaa=15; break;}
      case 'F' : {iaa=16; break;}
      case 'Y' : {iaa=17; break;}
      case 'W' : {iaa=18; break;}
      default : {printf("%i -- aa\n",i);exit(-1);}
      }
      if(n<100)
        fprintf(fp,"ATOM     %2u  %s %s    %2u     %7.3f %7.3f %7.3f  1.00\n",
                n,atom_name[iatom],aminoacid[iaa],i+1,x[j],y[j],z[j]);
      else
        fprintf(fp,"ATOM    %2u  %s %s    %2u     %7.3f %7.3f %7.3f  1.00\n",
                n,atom_name[iatom],aminoacid[iaa],i+1,x[j],y[j],z[j]); 
    }
  }
  fclose(fp);
}
/****************************************************************************/
/***** INITIALIZATION *******************************************************/
/****************************************************************************/
void init(void)
{
  extern void sidechain(int i,int *nsa,int *nhaa,int *nrt);
  extern void init_aa(void);
  int i,c;
  int nsa,nrt,nhaa;
  long orig_seed;
  FILE *fp;

  /* read sequence and g parameters */

  fp=fopen(INPUT,"r"); 
  for (i=0;i<N;i++) {                       
    while ((c=getc(fp))==' ' || c==',' || c=='\n') {}
    seq[i]=c;
  }
  if (NTMP>1) {
    for (i=0;i<NTMP;i++) {   
      beta[i]=1.0/TMAX*pow(TMAX/TMIN,(double)i/(NTMP-1));
      if (IREADG==1) fscanf(fp,"%i %lf",&i,&g[i]); else g[i]=0;
    }
  }
  else {beta[0]=1/TMAX; g[0]=0;}
  fclose(fp);

  fp=fopen(OUTPUT,"w");
  for (i=0;i<N;i++) putc(seq[i],fp);
  fprintf(fp,"\n");
  fclose(fp);

  /* calculate atom indices */

  nhaa=nrt=0;
  for (i=0;i<N;i++) {  
    ih[i]=-1;  // NEW
    irt[i]=nrt;
    iN[i]=(i==0) ? 0:iC[i-1]+2;
    iCa[i]=(seq[i]=='P') ? iN[i]+1:iN[i]+2;    
    sidechain(i,&nsa,&nhaa,&nrt); 
    iC[i]=iCa[i]+nsa+2;
    if (seq[i]=='G') iC[i]++;
    atyp[iN[i]]=1;    
    if (seq[i]!='P') atyp[iN[i]+1]=3;
    atyp[iCa[i]]=0;
    atyp[iCa[i]+1]=3;
    if (seq[i]=='G') atyp[iCa[i]+2]=3;
    atyp[iC[i]]=0;
    atyp[iC[i]+1]=2;
  }
  if (iC[N-1]+2!=NTO) {printf("in init: %i -- NTO\n",iC[N-1]+2);exit(-1);}
  if (nhaa!=NHAA) {printf("in init:%i -- NHAA=%i\n",nhaa,NHAA);exit(-1);}
  if (nrt!=NRT) {printf("in init: %i -- NRT\n",nrt);exit(-1);}

  /* various constants */   

  pi=acos(-1.);
  pi2=2*pi;
  pid2=pi/2;

  /* initialize various subroutines */

  init_aa();
  int2cart(-1);
  bias(-1);
  hbonds(-1);
  sac(-1,0.0);
  hp(-1);
  pivot(-1);
  mirror(-1);
  rot(-1);
  delta2(-1);
  profile(-1);
  helix(-1);
  histor(-1);
  histoe(-1,ind,E);
  histoq(-1,ind,E);
  histohp(-1,ind,E);

  /* initialize dynamical variables */

  orig_seed=seed;

  if (ISTART==0) { 
    ind=NTMP*ran3n(&seed);
    for (i=0;i<N;i++) {
      ph[3*i]=pi2*ran3n(&seed); ph[3*i+1]=pi2*ran3n(&seed); ph[3*i+2]=pi;
      if (seq[i]=='P') ph[3*i]=-pi*65./180.;
    }
    for (i=0;i<NRT;i++) rt[i]=NDI*ran3n(&seed);
    printf("disordered start, seed=%li\n",orig_seed);
  }
  else if (ISTART==1) {
    ind=NTMP*ran3n(&seed);
    for (i=0;i<N;i++) {
      ph[3*i]=-pi/3; ph[3*i+1]=-pi/4; ph[3*i+2]=pi;
      if (seq[i]=='P') ph[3*i]=-pi*65./180.;
    }
    for (i=0;i<NRT;i++) rt[i]=0;
    printf("ordered start, seed=%li\n",orig_seed);
  }
  else if (ISTART==2) {
    restart(STARTCONF,&orig_seed);
    for (i=0;i<N;i++) {
      if (seq[i]=='P' && fabs(ph[3*i]+pi*65./180)>.00001)
        {printf("%f -- Pro phi angle wrong\n",ph[3*i]);exit(-1);}
    }
    printf("restart from file, original seed=%li\n",orig_seed);
  }
  else
    printf("wrong ISTART %i\n",ISTART);

  write_conf(-1,CONF,orig_seed);

}
/****************************************************************************/
void sidechain(int i,int *nsa,int *nhaa,int *nrt)
{
  int iCb=iCa[i]+2;
  switch (seq[i]){  
  case 'G' : {
    *nsa=0;
    break;
  }
  case 'A' : {
    *nsa=4;
    atyp[iCb]=0;
    atyp[iCb+1]=atyp[iCb+2]=atyp[iCb+3]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)++;
    break;
  } 
  case 'V' : {
    *nsa=10;
    atyp[iCb]=atyp[iCb+1]=atyp[iCb+2]=0; atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+5]=atyp[iCb+6]=3;
    atyp[iCb+7]=atyp[iCb+8]=atyp[iCb+9]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)+=3;
    break;
  }
  case 'L' : {
    *nsa=13;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+5]=0; atyp[iCb+6]=3;
    atyp[iCb+7]=atyp[iCb+8]=atyp[iCb+9]=3;
    atyp[iCb+10]=atyp[iCb+11]=atyp[iCb+12]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)+=4;
    break;
  }
  case 'I' : {
    *nsa=13;
    atyp[iCb]=atyp[iCb+1]=atyp[iCb+2]=0; atyp[iCb+3]=3;
    atyp[iCb+4]=0; atyp[iCb+5]=atyp[iCb+6]=3;
    atyp[iCb+7]=atyp[iCb+8]=atyp[iCb+9]=3;
    atyp[iCb+10]=atyp[iCb+11]=atyp[iCb+12]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)+=4;
    break;
  }
  case 'S' : {
    *nsa=5;
    atyp[iCb]=0; atyp[iCb+1]=2; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=3;
    (*nrt)+=2;
    break;
  }
  case 'T' : {
    *nsa=8;
    atyp[iCb]=atyp[iCb+2]=0; atyp[iCb+1]=2; atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+5]=atyp[iCb+6]=atyp[iCb+7]=3;
    (*nrt)+=3;
    break;
  }
  case 'M' : {
    *nsa=11;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+4]=4; atyp[iCb+7]=0;
    atyp[iCb+2]=atyp[iCb+3]=atyp[iCb+5]=atyp[iCb+6]=3;
    atyp[iCb+8]=atyp[iCb+9]=atyp[iCb+10]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)+=4;
    break;
  }
  case 'P' : {
    *nsa=3;
    atyp[iCb]=atyp[iCb+1]=atyp[iCb+2]=0;
    break;
  }
  case 'D' : {
    *nsa=6;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+5]=2;
    (*nrt)+=2;
    break;
  }
  case 'N' : {
    *nsa=8;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=2; atyp[iCb+5]=1;
    atyp[iCb+6]=atyp[iCb+7]=3;
    (*nrt)+=3;
    break;
  }
  case 'E' : {
    *nsa=9;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=0;
    atyp[iCb+5]=atyp[iCb+6]=3; atyp[iCb+7]=atyp[iCb+8]=2;
    (*nrt)+=3;
    break;
  }
  case 'Q' : {
    *nsa=11;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=0;
    atyp[iCb+5]=atyp[iCb+6]=3; atyp[iCb+7]=2; atyp[iCb+8]=1;
    atyp[iCb+9]=atyp[iCb+10]=3;
    (*nrt)+=4;
    break;
  }
  case 'K' : {
    *nsa=16;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+7]=0; atyp[iCb+5]=atyp[iCb+6]=3; 
    atyp[iCb+8]=atyp[iCb+9]=atyp[iCb+11]=atyp[iCb+12]=3;
    atyp[iCb+10]=1;
    atyp[iCb+13]=atyp[iCb+14]=atyp[iCb+15]=3;
    (*nrt)+=5;
    break;
  }
  case 'R' : {
    *nsa=18;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=0; atyp[iCb+5]=atyp[iCb+6]=3; 
    atyp[iCb+7]=atyp[iCb+11]=atyp[iCb+12]=1;
    atyp[iCb+8]=atyp[iCb+9]=3; atyp[iCb+10]=0;
    atyp[iCb+13]=atyp[iCb+14]=atyp[iCb+15]=atyp[iCb+16]=atyp[iCb+17]=3;
    (*nrt)+=4;
    break;
  }
  case 'H' : {
    *nsa=11;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+7]=1;
    atyp[iCb+5]=atyp[iCb+6]=0;
    atyp[iCb+8]=atyp[iCb+9]=atyp[iCb+10]=3; 
    (*nrt)+=2;
    break;
  }
  case 'F' : {
    *nsa=14;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+5]=atyp[iCb+6]=atyp[iCb+7]=atyp[iCb+8]=0;
    atyp[iCb+9]=atyp[iCb+10]=atyp[iCb+11]=atyp[iCb+12]=atyp[iCb+13]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)+=2;
    break;
  }
  case 'Y' : {
    *nsa=15;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+5]=atyp[iCb+6]=atyp[iCb+7]=atyp[iCb+8]=0;
    atyp[iCb+9]=2;
    atyp[iCb+10]=atyp[iCb+11]=atyp[iCb+12]=atyp[iCb+13]=3;
    atyp[iCb+14]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)+=3;
    break;
  }
  case 'W' : {
    *nsa=18;
    atyp[iCb]=atyp[iCb+1]=0; atyp[iCb+2]=atyp[iCb+3]=3;
    atyp[iCb+4]=atyp[iCb+5]=0; atyp[iCb+6]=1;
    atyp[iCb+7]=atyp[iCb+8]=atyp[iCb+9]=atyp[iCb+10]=atyp[iCb+11]=0; 
    atyp[iCb+12]=atyp[iCb+13]=atyp[iCb+14]=3;
    atyp[iCb+15]=atyp[iCb+16]=atyp[iCb+17]=3;
    ih[i]=(*nhaa); (*nhaa)++;
    (*nrt)+=2;
    break;
  }
  default : {printf("WARNING: unknown sequence\n");}
  }        
}
/****************************************************************************/
void init_aa(void)
{
  extern void init_aa_sc(int i);
  int i,j,n;
  for (i=0;i<NTO;i++) {for (j=i+1;j<NTO;j++) aa[i][j]=1;} 
  for (i=0;i<NTO;i++) aa[i][i]=-1;
  for (i=0;i<N;i++) {if (seq[i]!='P') aa[iN[i]][iN[i]+1]=-1;}   /* NH */
  for (i=0;i<N;i++) aa[iN[i]][iCa[i]]=-1;                       /* NCa */
  for (i=0;i<N;i++) aa[iN[i]][iCa[i]+1]=-1;                     /* NHa */
  for (i=0;i<N;i++) aa[iN[i]][iCa[i]+2]=-1;                     /* NCb/Ha2 */
  for (i=0;i<N;i++) aa[iN[i]][iC[i]]=-1;                        /* NC */
  for (i=0;i<N;i++) {if (seq[i]!='P') aa[iN[i]+1][iCa[i]]=-1;}  /* HCa */
  for (i=0;i<N;i++) aa[iCa[i]][iCa[i]+1]=-1;                    /* CaHa */
  for (i=0;i<N;i++) aa[iCa[i]][iCa[i]+2]=-1;                    /* CaCb/Ha2 */
  for (i=0;i<N;i++) aa[iCa[i]][iC[i]]=-1;                       /* CaC */
  for (i=0;i<N;i++) aa[iCa[i]][iC[i]+1]=-1;                     /* CaO */
  for (i=0;i<N-1;i++) aa[iCa[i]][iN[i+1]]=-1;                   /* CaN */
  for (i=1;i<N;i++) {if (seq[i]!='P') aa[iCa[i-1]][iN[i]+1]=-1;}/* CaH */
  for (i=0;i<N-1;i++) aa[iCa[i]][iCa[i+1]]=-1;                  /* CaCa */
  for (i=0;i<N;i++) aa[iCa[i]+1][iCa[i]+2]=-1;                  /* HaCb/Ha2 */
  for (i=0;i<N;i++) aa[iCa[i]+1][iC[i]]=-1;                     /* HaC */
  for (i=0;i<N;i++) aa[iCa[i]+2][iC[i]]=-1;                     /* Cb/Ha2C */
  for (i=0;i<N;i++) aa[iC[i]][iC[i]+1]=-1;                      /* CO */
  for (i=0;i<N-1;i++) aa[iC[i]][iN[i+1]]=-1;                    /* CN */
  for (i=1;i<N;i++) {if (seq[i]!='P') aa[iC[i-1]][iN[i]+1]=-1;} /* CH */
  for (i=0;i<N-1;i++) aa[iC[i]][iCa[i+1]]=-1;                   /* CCa*/
  for (i=0;i<N-1;i++) aa[iC[i]+1][iN[i+1]]=-1;                  /* ON */
  for (i=1;i<N;i++) {if (seq[i]!='P')aa[iC[i-1]+1][iN[i]+1]=-1;}/* OH */
  for (i=0;i<N-1;i++) aa[iC[i]+1][iCa[i+1]]=-1;                 /* OCa */

  /* distances fixed because of fixed Pro phi angle */

  for (i=1;i<N;i++){
    if (seq[i]=='P') {
      aa[iCa[i-1]][iCa[i]+1]=-1;   /* CaHa */
      aa[iCa[i-1]][iCa[i]+2]=-1;   /* CaCb/Ha2 */
      aa[iCa[i-1]][iC[i]]=-1;      /* CaC */
      aa[iC[i-1]][iCa[i]+1]=-1;    /* CHa */
      aa[iC[i-1]][iCa[i]+2]=-1;    /* CCb/Ha2 */
      aa[iC[i-1]][iC[i]]=-1;       /* CC */
      aa[iC[i-1]+1][iCa[i]+1]=-1;  /* OHa */
      aa[iC[i-1]+1][iCa[i]+2]=-1;  /* OCb/Ha2 */
      aa[iC[i-1]+1][iC[i]]=-1;     /* OO */
    }
  }

  /* sidechain atoms */

  for (i=0;i<N;i++) init_aa_sc(i);
  
  /* phi pairs separated by 3 bonds */

  for (i=0;i<N-1;i++) {
    if (seq[i+1]!='P') {
      aa[iC[i]][iCa[i+1]+2]=2;          /* CpCb/Ha2 */
      aa[iC[i]][iC[i+1]]=2;             /* CpCp */
      aa[iC[i]][iCa[i+1]+1]=2;          /* CpHa */
    }
  }
  for (i=0;i<N;i++) {
    if (seq[i]!='P') {
      aa[iN[i]+1][iCa[i]+2]=2;          /* HCb/Ha2 */
      aa[iN[i]+1][iC[i]]=2;             /* HCp */
      aa[iN[i]+1][iCa[i]+1]=2;          /* HHa */
    }
  }

  /* psi pairs separated by 3 bonds */

  for (i=0;i<N;i++) {
    aa[iCa[i]+2][iC[i]+1]=2;            /* Cb/Ha2O */
    aa[iN[i]][iC[i]+1]=2;               /* NO */
    aa[iCa[i]+1][iC[i]+1]=2;            /* HaO */
  }
  for (i=0;i<N-1;i++) {
    aa[iCa[i]+2][iN[i+1]]=2;            /* Cb/Ha2N */
    aa[iN[i]][iN[i+1]]=2;               /* NN */
    aa[iCa[i]+1][iN[i+1]]=2;            /* HaN */
  }

  /* HH and OO repulsions */ 

  for (i=0;i<N-1;i++) {
    if (seq[i]!='P' && seq[i+1]!='P') {
      aa[iN[i]+1][iN[i+1]+1]=2;         /* HH */
    }
    aa[iC[i]+1][iC[i+1]+1]=2;           /* OO */
  }

  /* extra helix breaking term for proline */ 

  for (i=1;i<N;i++) {
    if (seq[i]=='P') {
      aa[iN[i-1]][iCa[i]+4]=3;                 /* NCd */
    }
  }

  n=0;
  for (i=0;i<NTO;i++) {
    for (j=i+1;j<NTO;j++) {
      if (aa[i][j]==1) {
        aa[i][j]=atyp[i]+atyp[j]*NA;
        aa[j][i]=atyp[j]+atyp[i]*NA;
        n++;
      }
      else if (aa[i][j]==2) {
        aa[i][j]=atyp[i]+atyp[j]*NA+NA*NA;
        aa[j][i]=atyp[j]+atyp[i]*NA+NA*NA;
        n++;
      }
      else if (aa[i][j]==3) {
        aa[i][j]=atyp[i]+atyp[j]*NA+2*NA*NA;
        aa[j][i]=atyp[j]+atyp[i]*NA+2*NA*NA;
        n++;
      }
      else if (aa[i][j]==-1) {
        aa[j][i]=-1;
      }
      else {
        printf("aa wrong\n");exit(-1);
      }
    }
  }
  printf("no of atom pairs %i   no of distances %i\n",NTO*(NTO-1)/2,n);
 
}
/****************************************************************************/
void init_aa_sc(int i)
{
  int j,k;
  switch (seq[i]){
  case 'G' : {
    break;
  }
  case 'A' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb  */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb  */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* HbHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* HbHb  */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb  */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    break;
  }
  case 'V' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg1 */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaCg2 */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb  */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg1 */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbCg2 */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb  */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* Cg1Cg2 */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* Cg1Hb  */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* Cg2Hb  */

    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg  */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* Cg1Hg */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* Cg1Hg */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* Cg1Hg  */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* HgHg */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* HgHg  */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg  */
    aa[iCa[i]+2][iCa[i]+9]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+10]=-1;                               /* CbHg */
    aa[iCa[i]+2][iCa[i]+11]=-1;                               /* CbHg  */
    aa[iCa[i]+4][iCa[i]+9]=-1;                                /* Cg2Hg */
    aa[iCa[i]+4][iCa[i]+10]=-1;                               /* Cg2Hg */
    aa[iCa[i]+4][iCa[i]+11]=-1;                               /* Cg2Hg  */
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* HgHg */
    aa[iCa[i]+9][iCa[i]+11]=-1;                               /* HgHg  */
    aa[iCa[i]+10][iCa[i]+11]=-1;                              /* HgHg  */
    
    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;

    aa[iCa[i]][iCa[i]+6]=2;                                    /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;

    aa[iCa[i]][iCa[i]+9]=2;                                    /* chi3 */
    aa[iCa[i]][iCa[i]+10]=2;
    aa[iCa[i]][iCa[i]+11]=2;
    aa[iCa[i]+3][iCa[i]+9]=2;
    aa[iCa[i]+3][iCa[i]+10]=2;
    aa[iCa[i]+3][iCa[i]+11]=2;
    aa[iCa[i]+5][iCa[i]+9]=2;
    aa[iCa[i]+5][iCa[i]+10]=2;
    aa[iCa[i]+5][iCa[i]+11]=2;

    break;
  }
  case 'L' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbCd1 */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbCd2 */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg  */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgCd1 */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgCd2 */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* CgHg */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* Cd1Cd2 */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* Cd1Hg */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* CgHd */
    aa[iCa[i]+3][iCa[i]+10]=-1;                               /* CgHd */
    aa[iCa[i]+3][iCa[i]+11]=-1;                               /* CgHd */
    aa[iCa[i]+6][iCa[i]+9]=-1;                                /* Cd1Hd */
    aa[iCa[i]+6][iCa[i]+10]=-1;                               /* Cd1Hd */
    aa[iCa[i]+6][iCa[i]+11]=-1;                               /* Cd1Hd */
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* HdHd */
    aa[iCa[i]+9][iCa[i]+11]=-1;                               /* HdHd */
    aa[iCa[i]+10][iCa[i]+11]=-1;                              /* HdHd */
    aa[iCa[i]+3][iCa[i]+12]=-1;                               /* CgHd */
    aa[iCa[i]+3][iCa[i]+13]=-1;                               /* CgHd */
    aa[iCa[i]+3][iCa[i]+14]=-1;                               /* CgHd */
    aa[iCa[i]+7][iCa[i]+12]=-1;                               /* Cd2Hd */
    aa[iCa[i]+7][iCa[i]+13]=-1;                               /* Cd2Hd */
    aa[iCa[i]+7][iCa[i]+14]=-1;                               /* Cd2Hd */
    aa[iCa[i]+12][iCa[i]+13]=-1;                              /* HdHd */
    aa[iCa[i]+12][iCa[i]+14]=-1;                              /* HdHd */
    aa[iCa[i]+13][iCa[i]+14]=-1;                              /* HdHd */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+2][iCa[i]+9]=2;                                   /* chi3 */
    aa[iCa[i]+2][iCa[i]+10]=2;
    aa[iCa[i]+2][iCa[i]+11]=2;
    aa[iCa[i]+7][iCa[i]+9]=2;
    aa[iCa[i]+7][iCa[i]+10]=2;
    aa[iCa[i]+7][iCa[i]+11]=2;
    aa[iCa[i]+8][iCa[i]+9]=2;
    aa[iCa[i]+8][iCa[i]+10]=2;
    aa[iCa[i]+8][iCa[i]+11]=2;
    aa[iCa[i]+2][iCa[i]+12]=2;                                   /* chi4 */
    aa[iCa[i]+2][iCa[i]+13]=2;
    aa[iCa[i]+2][iCa[i]+14]=2;
    aa[iCa[i]+6][iCa[i]+12]=2;
    aa[iCa[i]+6][iCa[i]+13]=2;
    aa[iCa[i]+6][iCa[i]+14]=2;
    aa[iCa[i]+8][iCa[i]+12]=2;
    aa[iCa[i]+8][iCa[i]+13]=2;
    aa[iCa[i]+8][iCa[i]+14]=2;
    break;
  }
  case 'I' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg1 */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaCg2 */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg1 */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbCg2 */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* Cg1Cg2 */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* Cg1Hb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* Cg2Hb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbCd */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* Cg1Cd */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* Cg1Hg */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* Cg1Hg */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* CdHg */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* CdHg */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* Cg1Hd */
    aa[iCa[i]+3][iCa[i]+10]=-1;                               /* Cg1Hd */
    aa[iCa[i]+3][iCa[i]+11]=-1;                               /* Cg1Hd */
    aa[iCa[i]+6][iCa[i]+9]=-1;                                /* CdHd */
    aa[iCa[i]+6][iCa[i]+10]=-1;                               /* CdHd */
    aa[iCa[i]+6][iCa[i]+11]=-1;                               /* CdHd */
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* HdHd */
    aa[iCa[i]+9][iCa[i]+11]=-1;                               /* HdHd */
    aa[iCa[i]+10][iCa[i]+11]=-1;                              /* HdHd */
    aa[iCa[i]+2][iCa[i]+12]=-1;                               /* CbHg2 */
    aa[iCa[i]+2][iCa[i]+13]=-1;                               /* CbHg2 */
    aa[iCa[i]+2][iCa[i]+14]=-1;                               /* CbHg2 */
    aa[iCa[i]+4][iCa[i]+12]=-1;                               /* Cg2Hg2 */
    aa[iCa[i]+4][iCa[i]+13]=-1;                               /* Cg2Hg2 */
    aa[iCa[i]+4][iCa[i]+14]=-1;                               /* Cg2Hg2 */
    aa[iCa[i]+12][iCa[i]+13]=-1;                              /* Hg2Hg2 */
    aa[iCa[i]+12][iCa[i]+14]=-1;                              /* Hg2Hg2 */
    aa[iCa[i]+13][iCa[i]+14]=-1;                              /* Hg2Hg2 */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;                                
    aa[iCa[i]][iCa[i]+8]=2;                                
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+2][iCa[i]+9]=2;                                 /* chi3 */
    aa[iCa[i]+2][iCa[i]+10]=2;                                
    aa[iCa[i]+2][iCa[i]+11]=2;                                
    aa[iCa[i]+7][iCa[i]+9]=2;
    aa[iCa[i]+7][iCa[i]+10]=2;
    aa[iCa[i]+7][iCa[i]+11]=2;
    aa[iCa[i]+8][iCa[i]+9]=2;
    aa[iCa[i]+8][iCa[i]+10]=2;
    aa[iCa[i]+8][iCa[i]+11]=2;
    aa[iCa[i]][iCa[i]+12]=2;                                   /* chi4 */
    aa[iCa[i]][iCa[i]+13]=2;                                
    aa[iCa[i]][iCa[i]+14]=2;                                
    aa[iCa[i]+3][iCa[i]+12]=2;
    aa[iCa[i]+3][iCa[i]+13]=2;
    aa[iCa[i]+3][iCa[i]+14]=2;
    aa[iCa[i]+5][iCa[i]+12]=2;
    aa[iCa[i]+5][iCa[i]+13]=2;
    aa[iCa[i]+5][iCa[i]+14]=2;
    break;
  }
  case 'S' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaOg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbOg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* OgHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* OgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbHg */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* OgHg */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                    /* chi2 */
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    break;
  }
  case 'T' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaOg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbOg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* OgCg */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* OgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* CgHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbHg1 */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* OgHg1 */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg2 */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg2 */
    aa[iCa[i]+2][iCa[i]+9]=-1;                                /* CbHg2 */
    aa[iCa[i]+4][iCa[i]+7]=-1;                                /* CgHg2 */
    aa[iCa[i]+4][iCa[i]+8]=-1;                                /* CgHg2 */
    aa[iCa[i]+4][iCa[i]+9]=-1;                                /* CgHg2 */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* Hg2Hg2 */
    aa[iCa[i]+7][iCa[i]+9]=-1;                                /* Hg2Hg2 */
    aa[iCa[i]+8][iCa[i]+9]=-1;                                /* Hg2Hg2 */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]+4][iCa[i]+6]=2;

    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]][iCa[i]+7]=2;                                   /* chi3 */
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]][iCa[i]+9]=2;
    aa[iCa[i]+3][iCa[i]+7]=2;
    aa[iCa[i]+3][iCa[i]+8]=2;
    aa[iCa[i]+3][iCa[i]+9]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+9]=2;
    break;
  }
  case 'M' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbSd */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgSd */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgHg */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* CgHg */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* SdHg */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* SdHg */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* CgCe */
    aa[iCa[i]+6][iCa[i]+9]=-1;                                /* SdCe */   
    aa[iCa[i]+6][iCa[i]+10]=-1;                               /* SdHe */
    aa[iCa[i]+6][iCa[i]+11]=-1;                               /* SdHe */
    aa[iCa[i]+6][iCa[i]+12]=-1;                               /* SdHe */
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* CeHe */
    aa[iCa[i]+9][iCa[i]+11]=-1;                               /* CeHe */
    aa[iCa[i]+9][iCa[i]+12]=-1;                               /* CeHe */
    aa[iCa[i]+10][iCa[i]+11]=-1;                              /* HeHe */
    aa[iCa[i]+10][iCa[i]+12]=-1;                              /* HeHe */
    aa[iCa[i]+11][iCa[i]+12]=-1;                              /* HeHe */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+2][iCa[i]+9]=2;                                 /* chi3 */
    aa[iCa[i]+7][iCa[i]+9]=2;
    aa[iCa[i]+8][iCa[i]+9]=2;
    aa[iCa[i]+6][iCa[i]+10]=2;                                /* chi4 */
    aa[iCa[i]+6][iCa[i]+11]=2;
    aa[iCa[i]+6][iCa[i]+12]=2;
    break;
  }
  case 'P' : {
    if (i>0) aa[iCa[i-1]][iCa[i]+3]=-1; 
    if (i>0) aa[iCa[i-1]][iCa[i]+4]=-1;
    if (i>0) aa[iC[i-1]][iCa[i]+3]=-1;
    if (i>0) aa[iC[i-1]][iCa[i]+4]=-1;
    if (i>0) aa[iC[i-1]+1][iCa[i]+3]=-1;
    if (i>0) aa[iC[i-1]+1][iCa[i]+4]=-1;
    aa[iN[i]][iCa[i]+3]=-1;                                   
    aa[iN[i]][iCa[i]+4]=-1;                                 
    aa[iCa[i]][iCa[i]+3]=-1;
    aa[iCa[i]][iCa[i]+4]=-1;
    aa[iCa[i]+1][iCa[i]+3]=-1;
    aa[iCa[i]+1][iCa[i]+4]=-1;
    aa[iCa[i]+2][iCa[i]+3]=-1;
    aa[iCa[i]+2][iCa[i]+4]=-1;
    aa[iCa[i]+3][iC[i]]=-1;
    aa[iCa[i]+3][iCa[i]+4]=-1;
    aa[iCa[i]+4][iC[i]]=-1;
    break;
  }
  case 'D' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */    
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */    
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */    
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbOd1 */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbOd2 */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgOd1 */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgOd2 */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* Od1Od2 */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    break;
  }
  case 'N' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */    
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */    
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */    
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbOd1 */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbNd2 */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgOd1 */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgNd2 */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* Od1Nd2 */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* CgHd */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* CgHd */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* NdHd */
    aa[iCa[i]+7][iCa[i]+9]=-1;                                /* NdHd */
    aa[iCa[i]+8][iCa[i]+9]=-1;                                /* HdHd */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+2][iCa[i]+8]=2;                                 /* chi3 */
    aa[iCa[i]+2][iCa[i]+9]=2;
    aa[iCa[i]+6][iCa[i]+8]=2;
    aa[iCa[i]+6][iCa[i]+9]=2;
    break;
  }
  case 'E' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbCd */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgCd */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgHg */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* CgHg */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* CdHg */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* CdHg */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* CgO1 */
    aa[iCa[i]+3][iCa[i]+10]=-1;                               /* CgO2 */
    aa[iCa[i]+6][iCa[i]+9]=-1;                                /* CdO1 */
    aa[iCa[i]+6][iCa[i]+10]=-1;                               /* CdO2 */
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* O1O2 */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+2][iCa[i]+9]=2;                                 /* chi3 */
    aa[iCa[i]+2][iCa[i]+10]=2;
    aa[iCa[i]+7][iCa[i]+9]=2;
    aa[iCa[i]+7][iCa[i]+10]=2;
    aa[iCa[i]+8][iCa[i]+9]=2;
    aa[iCa[i]+8][iCa[i]+10]=2;
    break;
  }
  case 'Q' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbCd */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgCd */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgHg */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* CgHg */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* CdHg */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* CdHg */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* CgO1 */
    aa[iCa[i]+3][iCa[i]+10]=-1;                               /* CgNe2 */
    aa[iCa[i]+6][iCa[i]+9]=-1;                                /* CdO1 */
    aa[iCa[i]+6][iCa[i]+10]=-1;                               /* CdNe2 */
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* O1Ne2 */
    aa[iCa[i]+6][iCa[i]+11]=-1;                               /* CdHe */
    aa[iCa[i]+6][iCa[i]+12]=-1;                               /* CdHe */
    aa[iCa[i]+10][iCa[i]+11]=-1;                              /* Ne2He */
    aa[iCa[i]+10][iCa[i]+12]=-1;                              /* Ne2He */
    aa[iCa[i]+11][iCa[i]+12]=-1;                              /* HeHe */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+2][iCa[i]+9]=2;                                 /* chi3 */
    aa[iCa[i]+2][iCa[i]+10]=2;
    aa[iCa[i]+7][iCa[i]+9]=2;
    aa[iCa[i]+7][iCa[i]+10]=2;
    aa[iCa[i]+8][iCa[i]+9]=2;
    aa[iCa[i]+8][iCa[i]+10]=2;
    aa[iCa[i]+3][iCa[i]+11]=2;                                /* chi4 */
    aa[iCa[i]+3][iCa[i]+12]=2;
    aa[iCa[i]+9][iCa[i]+11]=2;
    aa[iCa[i]+9][iCa[i]+12]=2;

    break;
  }
  case 'K' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbCd */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgCd */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgHg */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* CgHg */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* CdHg */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* CdHg */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* CgCe */
    aa[iCa[i]+3][iCa[i]+10]=-1;                               /* CgHd */
    aa[iCa[i]+3][iCa[i]+11]=-1;                               /* CgHd */
    aa[iCa[i]+6][iCa[i]+9]=-1;                                /* CdCe */   
    aa[iCa[i]+6][iCa[i]+10]=-1;                               /* CdHd */   
    aa[iCa[i]+6][iCa[i]+11]=-1;                               /* CdHd */   
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* CeHd */
    aa[iCa[i]+9][iCa[i]+11]=-1;                               /* CeHd */
    aa[iCa[i]+10][iCa[i]+11]=-1;                              /* HdHd */
    aa[iCa[i]+6][iCa[i]+12]=-1;                               /* CdN */
    aa[iCa[i]+6][iCa[i]+13]=-1;                               /* CdHe */
    aa[iCa[i]+6][iCa[i]+14]=-1;                               /* CdHe */
    aa[iCa[i]+9][iCa[i]+12]=-1;                               /* CeN */
    aa[iCa[i]+9][iCa[i]+13]=-1;                               /* CeHe */
    aa[iCa[i]+9][iCa[i]+14]=-1;                               /* CeHe */
    aa[iCa[i]+12][iCa[i]+13]=-1;                              /* NHe */
    aa[iCa[i]+12][iCa[i]+14]=-1;                              /* NHe */
    aa[iCa[i]+13][iCa[i]+14]=-1;                              /* HeHe */
    aa[iCa[i]+9][iCa[i]+15]=-1;                               /* CeHz */
    aa[iCa[i]+9][iCa[i]+16]=-1;                               /* CeHz */
    aa[iCa[i]+9][iCa[i]+17]=-1;                               /* CeHz */
    aa[iCa[i]+12][iCa[i]+15]=-1;                              /* NHz */
    aa[iCa[i]+12][iCa[i]+16]=-1;                              /* NHz */
    aa[iCa[i]+12][iCa[i]+17]=-1;                              /* NHz */
    aa[iCa[i]+15][iCa[i]+16]=-1;                              /* HzHz */
    aa[iCa[i]+15][iCa[i]+17]=-1;                              /* HzHz */
    aa[iCa[i]+16][iCa[i]+17]=-1;                              /* HzHz */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+2][iCa[i]+9]=2;                                 /* chi3 */
    aa[iCa[i]+2][iCa[i]+10]=2;
    aa[iCa[i]+2][iCa[i]+11]=2;
    aa[iCa[i]+7][iCa[i]+9]=2;
    aa[iCa[i]+7][iCa[i]+10]=2;
    aa[iCa[i]+7][iCa[i]+11]=2;
    aa[iCa[i]+8][iCa[i]+9]=2;
    aa[iCa[i]+8][iCa[i]+10]=2;
    aa[iCa[i]+8][iCa[i]+11]=2;
    aa[iCa[i]+3][iCa[i]+12]=2;                                 /* chi4 */
    aa[iCa[i]+3][iCa[i]+13]=2;
    aa[iCa[i]+3][iCa[i]+14]=2;
    aa[iCa[i]+10][iCa[i]+12]=2;
    aa[iCa[i]+10][iCa[i]+13]=2;
    aa[iCa[i]+10][iCa[i]+14]=2;
    aa[iCa[i]+11][iCa[i]+12]=2;
    aa[iCa[i]+11][iCa[i]+13]=2;
    aa[iCa[i]+11][iCa[i]+14]=2;
    aa[iCa[i]+6][iCa[i]+15]=2;                                 /* chi5 */
    aa[iCa[i]+6][iCa[i]+16]=2;
    aa[iCa[i]+6][iCa[i]+17]=2;
    aa[iCa[i]+13][iCa[i]+15]=2;
    aa[iCa[i]+13][iCa[i]+16]=2;
    aa[iCa[i]+13][iCa[i]+17]=2;
    aa[iCa[i]+14][iCa[i]+15]=2;
    aa[iCa[i]+14][iCa[i]+16]=2;
    aa[iCa[i]+14][iCa[i]+17]=2;
    break;
  }
  case 'R' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+2][iCa[i]+3]=-1;                                /* CbCg */
    aa[iCa[i]+2][iCa[i]+4]=-1;                                /* CbHb */
    aa[iCa[i]+2][iCa[i]+5]=-1;                                /* CbHb */
    aa[iCa[i]+3][iCa[i]+4]=-1;                                /* CgHb */
    aa[iCa[i]+3][iCa[i]+5]=-1;                                /* CgHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */
    aa[iCa[i]+2][iCa[i]+6]=-1;                                /* CbCd */
    aa[iCa[i]+2][iCa[i]+7]=-1;                                /* CbHg */
    aa[iCa[i]+2][iCa[i]+8]=-1;                                /* CbHg */
    aa[iCa[i]+3][iCa[i]+6]=-1;                                /* CgCd */
    aa[iCa[i]+3][iCa[i]+7]=-1;                                /* CgHg */
    aa[iCa[i]+3][iCa[i]+8]=-1;                                /* CgHg */
    aa[iCa[i]+6][iCa[i]+7]=-1;                                /* CdHg */
    aa[iCa[i]+6][iCa[i]+8]=-1;                                /* CdHg */
    aa[iCa[i]+7][iCa[i]+8]=-1;                                /* HgHg */
    aa[iCa[i]+3][iCa[i]+9]=-1;                                /* CgNe */
    aa[iCa[i]+3][iCa[i]+10]=-1;                               /* CgHd */
    aa[iCa[i]+3][iCa[i]+11]=-1;                               /* CgHd */
    aa[iCa[i]+9][iCa[i]+10]=-1;                               /* NeHd */
    aa[iCa[i]+9][iCa[i]+11]=-1;                               /* NeHd */
    aa[iCa[i]+10][iCa[i]+11]=-1;                              /* HdHd */

    for (j=iCa[i]+9;j<iCa[i]+20;j++)                          /* Cd */
        aa[iCa[i]+6][j]=-1;
    for (j=iCa[i]+12;j<iCa[i]+20;j++)                         /* Ne */
        aa[iCa[i]+9][j]=-1;
    for (j=iCa[i]+12;j<iCa[i]+20;j++)                         /* Cz+ */
      for (k=j+1;k<iCa[i]+20;k++)                             /* CN3-group */
        aa[j][k]=-1;

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]][iCa[i]+8]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+8]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+8]=2;
    aa[iCa[i]+2][iCa[i]+9]=2;                                 /* chi3 */
    aa[iCa[i]+2][iCa[i]+10]=2;
    aa[iCa[i]+2][iCa[i]+11]=2;
    aa[iCa[i]+7][iCa[i]+9]=2;
    aa[iCa[i]+7][iCa[i]+10]=2;
    aa[iCa[i]+7][iCa[i]+11]=2;
    aa[iCa[i]+8][iCa[i]+9]=2;
    aa[iCa[i]+8][iCa[i]+10]=2;
    aa[iCa[i]+8][iCa[i]+11]=2;
    aa[iCa[i]+3][iCa[i]+12]=2;                                 /* chi4 */
    aa[iCa[i]+3][iCa[i]+15]=2;                                 /* CgHe */
    aa[iCa[i]+10][iCa[i]+12]=2;
    aa[iCa[i]+10][iCa[i]+15]=2;
    aa[iCa[i]+11][iCa[i]+12]=2;
    aa[iCa[i]+11][iCa[i]+15]=2;
    break;
  }
  case 'H' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */

    for (j=iCa[i]+3;j<iCa[i]+13;j++)                          /* CbX */
      aa[iCa[i]+2][j]=-1;

    for (j=iCa[i]+4;j<iCa[i]+13;j++)                          /* CgX */
      aa[iCa[i]+3][j]=-1;     

    for (j=iCa[i]+6;j<iCa[i]+13;j++)                          /* Nd1+ */
      for (k=j+1;k<iCa[i]+13;k++)                             /* pentagon */
        aa[j][k]=-1;

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    break;
  }
  case 'F' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */

    for (j=iCa[i]+3;j<iCa[i]+16;j++)                          /* CbX */
      aa[iCa[i]+2][j]=-1;

    for (j=iCa[i]+4;j<iCa[i]+16;j++)                          /* CgX */
      aa[iCa[i]+3][j]=-1;     

    for (j=iCa[i]+6;j<iCa[i]+16;j++)                          /* Cd1+hexagon*/
      for (k=j+1;k<iCa[i]+16;k++)
        aa[j][k]=-1;

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    break;
  }
  case 'Y' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */

    for (j=iCa[i]+3;j<iCa[i]+16;j++)                          /* CbX */
      aa[iCa[i]+2][j]=-1;

    for (j=iCa[i]+4;j<iCa[i]+16;j++)                          /* CgX */
      aa[iCa[i]+3][j]=-1;     

    for (j=iCa[i]+6;j<iCa[i]+16;j++)                          /*Cd1+hexagon*/
      for (k=j+1;k<iCa[i]+16;k++)  
        aa[j][k]=-1;
    
    aa[iCa[i]+10][iCa[i]+16]=-1;                              /* CzHn */
    aa[iCa[i]+11][iCa[i]+16]=-1;                              /* OnHn */

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;

    aa[iCa[i]+8][iCa[i]+16]=2;                                /* chi3 */
    aa[iCa[i]+9][iCa[i]+16]=2;
    break;
  }
  case 'W' : {
    aa[iCa[i]][iCa[i]+3]=-1;                                  /* CaCg */
    aa[iCa[i]][iCa[i]+4]=-1;                                  /* CaHb */
    aa[iCa[i]][iCa[i]+5]=-1;                                  /* CaHb */
    aa[iCa[i]+4][iCa[i]+5]=-1;                                /* HbHb */

    for (j=iCa[i]+3;j<iCa[i]+20;j++)                          /* CbX */
      aa[iCa[i]+2][j]=-1;

    for (j=iCa[i]+4;j<iCa[i]+20;j++)                          /* CgX */
      aa[iCa[i]+3][j]=-1;     

    for (j=iCa[i]+6;j<iCa[i]+20;j++)                         /* Cd1+pentagon*/
      for (k=j+1;k<iCa[i]+20;k++)                             /* +hexagon */
        aa[j][k]=-1;

    aa[iN[i]][iCa[i]+3]=2;                                    /* chi1 */
    aa[iN[i]][iCa[i]+4]=2;
    aa[iN[i]][iCa[i]+5]=2;
    aa[iCa[i]+1][iCa[i]+3]=2;
    aa[iCa[i]+1][iCa[i]+4]=2;
    aa[iCa[i]+1][iCa[i]+5]=2;
    aa[iCa[i]+3][iC[i]]=2;
    aa[iCa[i]+4][iC[i]]=2;
    aa[iCa[i]+5][iC[i]]=2;
    aa[iCa[i]][iCa[i]+6]=2;                                   /* chi2 */
    aa[iCa[i]][iCa[i]+7]=2;
    aa[iCa[i]+4][iCa[i]+6]=2;
    aa[iCa[i]+4][iCa[i]+7]=2;
    aa[iCa[i]+5][iCa[i]+6]=2;
    aa[iCa[i]+5][iCa[i]+7]=2;
    break;
  }
  default : {printf("WARNING: unknown sequence\n");}
  }
}
/****************************************************************************/

